/// <reference path="./gio-2.0.d.ts" />
/// <reference path="./gobject-2.0.d.ts" />
/// <reference path="./glib-2.0.d.ts" />
/// <reference path="./gmodule-2.0.d.ts" />

/**
 * Type Definitions for Gjs (https://gjs.guide/)
 *
 * These type definitions are automatically generated, do not edit them by hand.
 * If you found a bug fix it in `ts-for-gir` or create a bug report on https://github.com/gjsify/ts-for-gir
 *
 * The based EJS template file is used for the generated .d.ts file of each GIR module like Gtk-4.0, GObject-2.0, ...
 */

declare module 'gi://UDisks?version=2.0' {
    // Module dependencies
    import type Gio from 'gi://Gio?version=2.0';
    import type GObject from 'gi://GObject?version=2.0';
    import type GLib from 'gi://GLib?version=2.0';
    import type GModule from 'gi://GModule?version=2.0';

    export namespace UDisks {
        /**
         * UDisks-2.0
         */

        /**
         * Error codes for the #UDISKS_ERROR error domain and the
         * corresponding D-Bus error names.
         */
        class Error extends GLib.Error {
            static $gtype: GObject.GType<Error>;

            // Static fields

            /**
             * The operation failed.
             */
            static FAILED: number;
            /**
             * The operation was cancelled.
             */
            static CANCELLED: number;
            /**
             * The operation has already been cancelled.
             */
            static ALREADY_CANCELLED: number;
            /**
             * Not authorized to perform the requested operation.
             */
            static NOT_AUTHORIZED: number;
            /**
             * Like %UDISKS_ERROR_NOT_AUTHORIZED but authorization can be obtained through e.g. authentication.
             */
            static NOT_AUTHORIZED_CAN_OBTAIN: number;
            /**
             * Like %UDISKS_ERROR_NOT_AUTHORIZED but an authentication was shown and the user dismissed it.
             */
            static NOT_AUTHORIZED_DISMISSED: number;
            /**
             * The device is already mounted.
             */
            static ALREADY_MOUNTED: number;
            /**
             * The device is not mounted.
             */
            static NOT_MOUNTED: number;
            /**
             * Not permitted to use the requested option.
             */
            static OPTION_NOT_PERMITTED: number;
            /**
             * The device is mounted by another user.
             */
            static MOUNTED_BY_OTHER_USER: number;
            /**
             * The device is already unmounting.
             */
            static ALREADY_UNMOUNTING: number;
            /**
             * The operation is not supported due to missing driver/tool support.
             */
            static NOT_SUPPORTED: number;
            /**
             * The operation timed out.
             */
            static TIMED_OUT: number;
            /**
             * The operation would wake up a disk that is in a deep-sleep state.
             */
            static WOULD_WAKEUP: number;
            /**
             * Attempting to unmount a device that is busy.
             */
            static DEVICE_BUSY: number;
            static ISCSI_DAEMON_TRANSPORT_FAILED: number;
            static ISCSI_HOST_NOT_FOUND: number;
            static ISCSI_IDMB: number;
            static ISCSI_LOGIN_FAILED: number;
            static ISCSI_LOGIN_AUTH_FAILED: number;
            static ISCSI_LOGIN_FATAL: number;
            static ISCSI_LOGOUT_FAILED: number;
            static ISCSI_NO_FIRMWARE: number;
            static ISCSI_NO_OBJECTS_FOUND: number;
            static ISCSI_NOT_CONNECTED: number;
            static ISCSI_TRANSPORT_FAILED: number;
            static ISCSI_UNKNOWN_DISCOVERY_TYPE: number;

            // Constructors

            constructor(options: { message: string; code: number });
            _init(...args: any[]): void;

            // Static methods

            static quark(): GLib.Quark;
        }

        const ERROR_NUM_ENTRIES: number;
        /**
         * The major version of the libudisks2 header files.
         */
        const MAJOR_VERSION: number;
        /**
         * The micro version of the libudisks2 header files.
         */
        const MICRO_VERSION: number;
        /**
         * The minor version of the libudisks2 header files.
         */
        const MINOR_VERSION: number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function block_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function block_lvm2_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksBlockLVM2 interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function block_lvm2_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Overrides all #GObject properties in the #UDisksBlock interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function block_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function drive_ata_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksDriveAta interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function drive_ata_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function drive_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksDrive interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function drive_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function encrypted_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksEncrypted interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function encrypted_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        function error_quark(): GLib.Quark;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function filesystem_btrfs_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksFilesystemBTRFS interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function filesystem_btrfs_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function filesystem_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksFilesystem interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function filesystem_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function job_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksJob interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function job_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function logical_volume_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksLogicalVolume interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function logical_volume_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function loop_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksLoop interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function loop_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function manager_btrfs_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksManagerBTRFS interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function manager_btrfs_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function manager_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function manager_lvm2_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksManagerLVM2 interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function manager_lvm2_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function manager_nvme_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksManagerNVMe interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function manager_nvme_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Overrides all #GObject properties in the #UDisksManager interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function manager_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function mdraid_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksMDRaid interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function mdraid_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function nvme_controller_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksNVMeController interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function nvme_controller_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function nvme_fabrics_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksNVMeFabrics interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function nvme_fabrics_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function nvme_namespace_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksNVMeNamespace interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function nvme_namespace_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function partition_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksPartition interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function partition_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function partition_table_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksPartitionTable interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function partition_table_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function physical_volume_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksPhysicalVolume interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function physical_volume_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function swapspace_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksSwapspace interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function swapspace_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function vdo_volume_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksVDOVolume interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function vdo_volume_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link> D-Bus interface.
         * @returns A #GDBusInterfaceInfo. Do not free.
         */
        function volume_group_interface_info(): Gio.DBusInterfaceInfo;
        /**
         * Overrides all #GObject properties in the #UDisksVolumeGroup interface for a concrete class.
         * The properties are overridden in the order they are defined.
         * @param klass The class structure for a #GObject derived class.
         * @param property_id_begin The property id to assign to the first overridden property.
         * @returns The last property id.
         */
        function volume_group_override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        /**
         * Flags describing a partition type.
         */

        /**
         * Flags describing a partition type.
         */
        export namespace PartitionTypeInfoFlags {
            export const $gtype: GObject.GType<PartitionTypeInfoFlags>;
        }

        enum PartitionTypeInfoFlags {
            /**
             * No flags set.
             */
            NONE,
            /**
             * Partition type is used for swap.
             */
            SWAP,
            /**
             * Partition type is used for RAID/LVM or similar.
             */
            RAID,
            /**
             * Partition type indicates the partition is hidden (e.g. 'dos' type 0x1b "Hidden W95 FAT32"). Note that this is not the same as user-toggleable attributes/flags for a partition.
             */
            HIDDEN,
            /**
             * Partition type can only be used when creating a partition and e.g. should not be selectable in a "change partition type" user interface (e.g. 'dos' type 0x05, 0x0f and 0x85 for extended partitions).
             */
            CREATE_ONLY,
            /**
             * Partition type indicates the partition is part of the system / bootloader (e.g. 'dos' types 0xee, 0xff, 'gpt' types for 'EFI System partition' and 'BIOS Boot partition').
             */
            SYSTEM,
        }
        namespace BlockLVM2Proxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    BlockLVM2.ConstructorProps {}
        }

        /**
         * The #UDisksBlockLVM2Proxy structure contains only private data and should only be accessed using the provided API.
         */
        class BlockLVM2Proxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<BlockLVM2Proxy>, Gio.DBusInterface, Gio.Initable, BlockLVM2
        {
            static $gtype: GObject.GType<BlockLVM2Proxy>;

            // Constructors

            constructor(properties?: Partial<BlockLVM2Proxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): BlockLVM2Proxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): BlockLVM2Proxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): BlockLVM2Proxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): BlockLVM2Proxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_lvm2_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_block_lvm2_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<BlockLVM2Proxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_block_lvm2_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_lvm2_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_block_lvm2_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<BlockLVM2Proxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block-LVM2.LogicalVolume">"LogicalVolume"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get logical_volume(): string;
            set logical_volume(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block-LVM2.LogicalVolume">"LogicalVolume"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get logicalVolume(): string;
            set logicalVolume(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): BlockLVM2Proxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace BlockLVM2Skeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    BlockLVM2.ConstructorProps {}
        }

        /**
         * The #UDisksBlockLVM2Skeleton structure contains only private data and should only be accessed using the provided API.
         */
        class BlockLVM2Skeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, BlockLVM2 {
            static $gtype: GObject.GType<BlockLVM2Skeleton>;

            // Constructors

            constructor(properties?: Partial<BlockLVM2Skeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): BlockLVM2Skeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block-LVM2.LogicalVolume">"LogicalVolume"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get logical_volume(): string;
            set logical_volume(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block-LVM2.LogicalVolume">"LogicalVolume"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get logicalVolume(): string;
            set logicalVolume(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace BlockProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Block.ConstructorProps {}
        }

        /**
         * The #UDisksBlockProxy structure contains only private data and should only be accessed using the provided API.
         */
        class BlockProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<BlockProxy>, Gio.DBusInterface, Gio.Initable, Block
        {
            static $gtype: GObject.GType<BlockProxy>;

            // Constructors

            constructor(properties?: Partial<BlockProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): BlockProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): BlockProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): BlockProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): BlockProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_block_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<BlockProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_block_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_block_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<BlockProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Configuration">"Configuration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get configuration(): GLib.Variant;
            set configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.CryptoBackingDevice">"CryptoBackingDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get crypto_backing_device(): string;
            set crypto_backing_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.CryptoBackingDevice">"CryptoBackingDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cryptoBackingDevice(): string;
            set cryptoBackingDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Device">"Device"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get device(): string;
            set device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.DeviceNumber">"DeviceNumber"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get device_number(): number;
            set device_number(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.DeviceNumber">"DeviceNumber"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get deviceNumber(): number;
            set deviceNumber(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Drive">"Drive"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get drive(): string;
            set drive(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintAuto">"HintAuto"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_auto(): boolean;
            set hint_auto(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintAuto">"HintAuto"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintAuto(): boolean;
            set hintAuto(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIconName">"HintIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_icon_name(): string;
            set hint_icon_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIconName">"HintIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintIconName(): string;
            set hintIconName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIgnore">"HintIgnore"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_ignore(): boolean;
            set hint_ignore(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIgnore">"HintIgnore"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintIgnore(): boolean;
            set hintIgnore(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">"HintName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_name(): string;
            set hint_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">"HintName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintName(): string;
            set hintName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintPartitionable">"HintPartitionable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_partitionable(): boolean;
            set hint_partitionable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintPartitionable">"HintPartitionable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintPartitionable(): boolean;
            set hintPartitionable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSymbolicIconName">"HintSymbolicIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_symbolic_icon_name(): string;
            set hint_symbolic_icon_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSymbolicIconName">"HintSymbolicIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintSymbolicIconName(): string;
            set hintSymbolicIconName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSystem">"HintSystem"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_system(): boolean;
            set hint_system(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSystem">"HintSystem"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintSystem(): boolean;
            set hintSystem(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Id">"Id"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id(): string;
            set id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdLabel">"IdLabel"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_label(): string;
            set id_label(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdLabel">"IdLabel"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idLabel(): string;
            set idLabel(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdType">"IdType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_type(): string;
            set id_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdType">"IdType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idType(): string;
            set idType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUsage">"IdUsage"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_usage(): string;
            set id_usage(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUsage">"IdUsage"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idUsage(): string;
            set idUsage(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUUID">"IdUUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_uuid(): string;
            set id_uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUUID">"IdUUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idUuid(): string;
            set idUuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdVersion">"IdVersion"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_version(): string;
            set id_version(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdVersion">"IdVersion"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idVersion(): string;
            set idVersion(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaid">"MDRaid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraid(): string;
            set mdraid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaidMember">"MDRaidMember"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraid_member(): string;
            set mdraid_member(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaidMember">"MDRaidMember"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraidMember(): string;
            set mdraidMember(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.PreferredDevice">"PreferredDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get preferred_device(): string;
            set preferred_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.PreferredDevice">"PreferredDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get preferredDevice(): string;
            set preferredDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.ReadOnly">"ReadOnly"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_only(): boolean;
            set read_only(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.ReadOnly">"ReadOnly"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readOnly(): boolean;
            set readOnly(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Symlinks">"Symlinks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get symlinks(): string[];
            set symlinks(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.UserspaceMountOptions">"UserspaceMountOptions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get userspace_mount_options(): string[];
            set userspace_mount_options(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.UserspaceMountOptions">"UserspaceMountOptions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get userspaceMountOptions(): string[];
            set userspaceMountOptions(val: string[]);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): BlockProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_add_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_add_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_add_configuration_item() for the asynchronous version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_configuration_item_sync(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_format() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_block_call_get_secret_configuration().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_get_secret_configuration().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_secret_configuration_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_get_secret_configuration() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_secret_configuration_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_device_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_device() for the asynchronous version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_device_sync(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_backup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_backup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_backup_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_backup() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_backup_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_benchmark().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_benchmark().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_benchmark_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_benchmark() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_benchmark_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_restore().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_restore().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_restore_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_restore() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_restore_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_remove_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_remove_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_remove_configuration_item() for the asynchronous version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_configuration_item_sync(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rescan(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rescan(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rescan(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_rescan().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_rescan().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rescan_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_rescan() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rescan_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_update_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_update_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_update_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_update_configuration_item() for the asynchronous version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_update_configuration_item_sync(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_format(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param configuration Parameter to return.
             */
            complete_get_secret_configuration(invocation: Gio.DBusMethodInvocation, configuration: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_device(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_backup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_benchmark(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_restore(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_rescan(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_update_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksBlock::handle-add-configuration-item signal.
             * @param invocation
             * @param arg_item
             * @param arg_options
             */
            vfunc_handle_add_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-format signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_format(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-get-secret-configuration signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_secret_configuration(
                invocation: Gio.DBusMethodInvocation,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-device signal.
             * @param invocation
             * @param fd_list
             * @param arg_mode
             * @param arg_options
             */
            vfunc_handle_open_device(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_mode: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-backup signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_backup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-benchmark signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_benchmark(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-restore signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_restore(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-remove-configuration-item signal.
             * @param invocation
             * @param arg_item
             * @param arg_options
             */
            vfunc_handle_remove_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-rescan signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_rescan(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksBlock::handle-update-configuration-item signal.
             * @param invocation
             * @param arg_old_item
             * @param arg_new_item
             * @param arg_options
             */
            vfunc_handle_update_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace BlockSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Block.ConstructorProps {}
        }

        /**
         * The #UDisksBlockSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class BlockSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Block {
            static $gtype: GObject.GType<BlockSkeleton>;

            // Constructors

            constructor(properties?: Partial<BlockSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): BlockSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Configuration">"Configuration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get configuration(): GLib.Variant;
            set configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.CryptoBackingDevice">"CryptoBackingDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get crypto_backing_device(): string;
            set crypto_backing_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.CryptoBackingDevice">"CryptoBackingDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cryptoBackingDevice(): string;
            set cryptoBackingDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Device">"Device"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get device(): string;
            set device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.DeviceNumber">"DeviceNumber"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get device_number(): number;
            set device_number(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.DeviceNumber">"DeviceNumber"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get deviceNumber(): number;
            set deviceNumber(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Drive">"Drive"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get drive(): string;
            set drive(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintAuto">"HintAuto"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_auto(): boolean;
            set hint_auto(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintAuto">"HintAuto"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintAuto(): boolean;
            set hintAuto(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIconName">"HintIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_icon_name(): string;
            set hint_icon_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIconName">"HintIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintIconName(): string;
            set hintIconName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIgnore">"HintIgnore"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_ignore(): boolean;
            set hint_ignore(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIgnore">"HintIgnore"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintIgnore(): boolean;
            set hintIgnore(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">"HintName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_name(): string;
            set hint_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">"HintName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintName(): string;
            set hintName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintPartitionable">"HintPartitionable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_partitionable(): boolean;
            set hint_partitionable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintPartitionable">"HintPartitionable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintPartitionable(): boolean;
            set hintPartitionable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSymbolicIconName">"HintSymbolicIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_symbolic_icon_name(): string;
            set hint_symbolic_icon_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSymbolicIconName">"HintSymbolicIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintSymbolicIconName(): string;
            set hintSymbolicIconName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSystem">"HintSystem"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_system(): boolean;
            set hint_system(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSystem">"HintSystem"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintSystem(): boolean;
            set hintSystem(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Id">"Id"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id(): string;
            set id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdLabel">"IdLabel"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_label(): string;
            set id_label(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdLabel">"IdLabel"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idLabel(): string;
            set idLabel(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdType">"IdType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_type(): string;
            set id_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdType">"IdType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idType(): string;
            set idType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUsage">"IdUsage"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_usage(): string;
            set id_usage(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUsage">"IdUsage"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idUsage(): string;
            set idUsage(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUUID">"IdUUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_uuid(): string;
            set id_uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUUID">"IdUUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idUuid(): string;
            set idUuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdVersion">"IdVersion"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_version(): string;
            set id_version(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdVersion">"IdVersion"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idVersion(): string;
            set idVersion(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaid">"MDRaid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraid(): string;
            set mdraid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaidMember">"MDRaidMember"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraid_member(): string;
            set mdraid_member(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaidMember">"MDRaidMember"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraidMember(): string;
            set mdraidMember(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.PreferredDevice">"PreferredDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get preferred_device(): string;
            set preferred_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.PreferredDevice">"PreferredDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get preferredDevice(): string;
            set preferredDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.ReadOnly">"ReadOnly"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_only(): boolean;
            set read_only(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.ReadOnly">"ReadOnly"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readOnly(): boolean;
            set readOnly(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Symlinks">"Symlinks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get symlinks(): string[];
            set symlinks(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.UserspaceMountOptions">"UserspaceMountOptions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get userspace_mount_options(): string[];
            set userspace_mount_options(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.UserspaceMountOptions">"UserspaceMountOptions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get userspaceMountOptions(): string[];
            set userspaceMountOptions(val: string[]);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_add_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_add_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_add_configuration_item() for the asynchronous version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_configuration_item_sync(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_format() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_block_call_get_secret_configuration().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_get_secret_configuration().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_secret_configuration_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_get_secret_configuration() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_secret_configuration_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_device_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_device() for the asynchronous version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_device_sync(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_backup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_backup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_backup_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_backup() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_backup_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_benchmark().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_benchmark().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_benchmark_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_benchmark() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_benchmark_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_restore().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_restore().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_restore_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_restore() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_restore_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_remove_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_remove_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_remove_configuration_item() for the asynchronous version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_configuration_item_sync(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rescan(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rescan(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rescan(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_rescan().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_rescan().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rescan_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_rescan() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rescan_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_update_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_update_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_update_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_update_configuration_item() for the asynchronous version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_update_configuration_item_sync(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_format(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param configuration Parameter to return.
             */
            complete_get_secret_configuration(invocation: Gio.DBusMethodInvocation, configuration: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_device(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_backup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_benchmark(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_restore(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_rescan(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_update_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksBlock::handle-add-configuration-item signal.
             * @param invocation
             * @param arg_item
             * @param arg_options
             */
            vfunc_handle_add_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-format signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_format(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-get-secret-configuration signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_secret_configuration(
                invocation: Gio.DBusMethodInvocation,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-device signal.
             * @param invocation
             * @param fd_list
             * @param arg_mode
             * @param arg_options
             */
            vfunc_handle_open_device(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_mode: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-backup signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_backup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-benchmark signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_benchmark(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-restore signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_restore(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-remove-configuration-item signal.
             * @param invocation
             * @param arg_item
             * @param arg_options
             */
            vfunc_handle_remove_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-rescan signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_rescan(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksBlock::handle-update-configuration-item signal.
             * @param invocation
             * @param arg_old_item
             * @param arg_new_item
             * @param arg_options
             */
            vfunc_handle_update_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace Client {
            // Signal callback interfaces

            interface Changed {
                (): void;
            }

            // Constructor properties interface

            interface ConstructorProps
                extends GObject.Object.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.Initable.ConstructorProps {
                bus_connection: Gio.DBusConnection;
                busConnection: Gio.DBusConnection;
                manager: Manager;
                object_manager: Gio.DBusObjectManager;
                objectManager: Gio.DBusObjectManager;
            }
        }

        /**
         * #UDisksClient is used for accessing the UDisks service from a
         * client program.
         */
        class Client extends GObject.Object implements Gio.AsyncInitable<Client>, Gio.Initable {
            static $gtype: GObject.GType<Client>;

            // Properties

            /**
             * The #GDBusConnection used to create the #UDisksClient:object-manager, or
             * %NULL if one wasn't specified during construction of the #UDisksClient (in
             * which case a system bus connection is used).
             */
            get bus_connection(): Gio.DBusConnection;
            /**
             * The #GDBusConnection used to create the #UDisksClient:object-manager, or
             * %NULL if one wasn't specified during construction of the #UDisksClient (in
             * which case a system bus connection is used).
             */
            get busConnection(): Gio.DBusConnection;
            /**
             * The #UDisksManager interface on the well-known
             * <literal>/org/freedesktop/UDisks2/Manager</literal> object
             */
            get manager(): Manager;
            /**
             * The #GDBusObjectManager used by the #UDisksClient instance.
             */
            get object_manager(): Gio.DBusObjectManager;
            /**
             * The #GDBusObjectManager used by the #UDisksClient instance.
             */
            get objectManager(): Gio.DBusObjectManager;

            // Constructors

            constructor(properties?: Partial<Client.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): Client;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_connection_finish(res: Gio.AsyncResult): Client;

            static new_sync(cancellable?: Gio.Cancellable | null): Client;

            // Signals

            connect(id: string, callback: (...args: any[]) => any): number;
            connect_after(id: string, callback: (...args: any[]) => any): number;
            emit(id: string, ...args: any[]): void;
            connect(signal: 'changed', callback: (_source: this) => void): number;
            connect_after(signal: 'changed', callback: (_source: this) => void): number;
            emit(signal: 'changed'): void;

            // Static methods

            /**
             * Gets a human-readable and localized text string describing a
             * a job `operation`.
             *
             * For known job operation types, see the documentation for the
             * <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Operation">Job:Operation</link>
             * D-Bus property.
             * @param operation A job operation name.
             */
            static get_job_description_from_operation(operation: string): string;
            /**
             * Asynchronously gets a #UDisksClient. When the operation is
             * finished, `callback` will be invoked in the <link
             * linkend="g-main-context-push-thread-default">thread-default main
             * loop</link> of the thread you are calling this method from.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback Function that will be called when the result is ready.
             */
            static ['new'](
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<Client> | null,
            ): void;
            /**
             * Like #udisks_client_new() but takes a #GDBusConnection which will be used
             * when connecting to the UDisks daemon over D-Bus. This is useful for unit
             * tests which may want to use #GTestDBus to create a D-Bus daemon on the
             * session bus.
             * @param connection a #GDBusConnection. If %NULL, a system bus   connection will be used.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback Function that will be called when the result is ready.
             */
            static new_for_connection(
                connection?: Gio.DBusConnection | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<Client> | null,
            ): void;

            // Methods

            /**
             * Gets all RAID devices (e.g. <filename>/dev/md0</filename> and <filename>/dev/md1</filename>) for `raid`.
             *
             * This is usually only useful in <ulink
             * url="http://en.wikipedia.org/wiki/Split-brain_(computing)">split-brain
             * situations</ulink> — see udisks_client_get_block_for_mdraid() for
             * an example — and is normally used only to convey the problem in an
             * user interface.
             * @param raid A #UDisksMDRaid.
             * @returns A list of #UDisksBlock instances. The   returned list should be freed with g_list_free() after each   element has been freed with g_object_unref().
             */
            get_all_blocks_for_mdraid(raid: MDRaid): Block[];
            /**
             * Gets the #UDisksBlock corresponding to `block_device_number,` if any.
             * @param block_device_number A #dev_t to get a #UDisksBlock for.
             * @returns A #UDisksBlock or %NULL if not found.
             */
            get_block_for_dev(block_device_number: number): Block;
            /**
             * Gets a block device corresponding to `drive`. The returned block
             * device, if any, is for the whole disk drive, e.g. a partition block
             * device is never returned.
             *
             * Set `get_physical` to %TRUE if you need a block device that you can
             * send low-level SCSI commands with (for multipath, this returns one
             * of the physical paths). Set it to %FALSE if you need a block device
             * that you can read/write data with (for multipath, this returns the
             * mapped device).
             * @param drive A #UDisksDrive.
             * @param get_physical %TRUE to get a physical device, %FALSE to get the logical device.
             * @returns A #UDisksBlock or %NULL if the requested kind of block device is not available - use g_object_unref() to free the returned object.
             */
            get_block_for_drive(drive: Drive, get_physical: boolean): Block;
            /**
             * Gets all the #UDisksBlock instances with the given label, if any.
             * @param label The label.
             * @returns A list of #UDisksBlock instances. The   returned list should be freed with g_list_free() after each   element has been freed with g_object_unref().
             */
            get_block_for_label(label: string): Block[];
            /**
             * Gets the RAID device (e.g. <filename>/dev/md0</filename>) for `raid`.
             *
             * In the case of a <ulink
             * url="http://en.wikipedia.org/wiki/Split-brain_(computing)">split-brain
             * syndrome</ulink>, it is undefined which RAID device is
             * returned. For example this can happen if
             * <filename>/dev/sda</filename> and <filename>/dev/sdb</filename> are
             * components of a two-disk RAID-1 and <filename>/dev/md0</filename>
             * and <filename>/dev/md1</filename> are two degraded arrays, each one
             * using exactly one of the two devices. Use
             * udisks_client_get_all_blocks_for_mdraid() to get all RAID devices.
             * @param raid A #UDisksMDRaid.
             * @returns A #UDisksBlock or %NULL if no RAID device is running.
             */
            get_block_for_mdraid(raid: MDRaid): Block;
            /**
             * Gets all the #UDisksBlock instances with the given uuid, if any.
             * @param uuid The uuid.
             * @returns A list of #UDisksBlock instances. The   returned list should be freed with g_list_free() after each   element has been freed with g_object_unref().
             */
            get_block_for_uuid(uuid: string): Block[];
            /**
             * If `block` is an unlocked encrypted device, gets the cleartext device.
             * @param block A #UDisksBlock.
             * @returns A #UDisksBlock or %NULL. Free with g_object_unref() when done with it.
             */
            get_cleartext_block(block: Block): Block;
            /**
             * Gets the #UDisksDrive that `block` belongs to, if any.
             * @param block A #UDisksBlock.
             * @returns A #UDisksDrive or %NULL if there is no #UDisksDrive for @block - free the returned object with g_object_unref().
             */
            get_drive_for_block(block: Block): Drive;
            /**
             * Gets information about a #UDisksDrive object that is suitable to
             * present in an user interface. The returned strings are localized.
             * @param drive A #UDisksDrive.
             */
            get_drive_info(drive: Drive): [string, string, Gio.Icon | null, string, Gio.Icon | null];
            /**
             * Gets all siblings for `drive`.
             * @param drive A #UDisksDrive.
             * @returns A list of #UDisksDrive instances. The   returned list should be freed with g_list_free() after each element has been   freed with g_object_unref().
             */
            get_drive_siblings(drive: Drive): Drive[];
            /**
             * Gets a human readable localized string for `usage,` `type` and `version`.
             * @param usage Usage id e.g. "filesystem" or "crypto".
             * @param type Type e.g. "ext4" or "crypto_LUKS"
             * @param version Version.
             * @param long_string Whether to produce a long string.
             * @returns A string that should be freed with g_free().
             */
            get_id_for_display(usage: string, type: string, version: string, long_string: boolean): string;
            /**
             * Gets a human-readable and localized text string describing the
             * operation of `job`.
             *
             * For known job types, see the documentation for the
             * <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Operation">Job:Operation</link>
             * D-Bus property.
             * @param job A #UDisksJob.
             * @returns A string that should be freed with g_free().
             */
            get_job_description(job: Job): string;
            /**
             * Gets all the #UDisksJob instances that reference `object,` if any.
             * @param object A #UDisksObject.
             * @returns A list of #UDisksJob instances. The   returned list should be freed with g_list_free() after each   element has been freed with g_object_unref().
             */
            get_jobs_for_object(object: Object): Job[];
            /**
             * Gets the corresponding loop interface for `block`.
             *
             * This only works if `block` itself is a loop device or a partition of
             * a loop device.
             * @param block A #UDisksBlock.
             * @returns A #UDisksLoop or %NULL. Free with g_object_unref().
             */
            get_loop_for_block(block: Block): Loop;
            /**
             * Gets the #UDisksManager interface on the well-known
             * <literal>/org/freedesktop/UDisks2/Manager</literal> object.
             * @returns A #UDisksManager or %NULL if the udisksd daemon is not currently running. Do not free, the instance is owned by @client.
             */
            get_manager(): Manager;
            /**
             * Gets the #UDisksMDRaid that `block` is the block device for, if any.
             * @param block A #UDisksBlock.
             * @returns A #UDisksMDRaid or %NULL if there is no   #UDisksMDRaid for @block or @block is not a MD-RAID block   device. Free the returned object with g_object_unref().
             */
            get_mdraid_for_block(block: Block): MDRaid;
            /**
             * Gets a human-readable string of the media described by
             * `media_compat`. The returned information is localized.
             * @param media_compat An array of media types.
             * @returns A string that should be freed with g_free() or %NULL if unknown.
             */
            get_media_compat_for_display(media_compat: string): string;
            /**
             * Gets the physical block devices that are part of `raid`.
             * @param raid A #UDisksMDRaid.
             * @returns A list of #UDisksBlock instances. The   returned list should be freed with g_list_free() after each   element has been freed with g_object_unref().
             */
            get_members_for_mdraid(raid: MDRaid): Block[];
            /**
             * Convenience function for looking up an #UDisksObject for `object_path`.
             * @param object_path Object path.
             * @returns A #UDisksObject corresponding to @object_path or %NULL if not found. The returned object must be freed with g_object_unref().
             */
            get_object(object_path: string): Object;
            /**
             * Gets information about a #UDisksObject instance that is suitable to
             * present in an user interface. Information is returned in the
             * #UDisksObjectInfo object and is localized.
             * @param object A #UDisksObject.
             * @returns A #UDisksObjectInfo instance that should be freed with g_object_unref().
             */
            get_object_info(object: Object): ObjectInfo;
            /**
             * Gets the #GDBusObjectManager used by `client`.
             * @returns A #GDBusObjectManager. Do not free, the instance is owned by @client.
             */
            get_object_manager(): Gio.DBusObjectManager;
            /**
             * Gets information about `partition` that is suitable to present in an
             * user interface in a single line of text.
             *
             * The returned string is localized and includes things like the
             * partition type, flags (if any) and name (if any).
             * @param partition # #UDisksPartition.
             * @returns A string that should be freed with g_free().
             */
            get_partition_info(partition: Partition): string;
            /**
             * Gets the #UDisksPartitionTable corresponding to `partition`.
             * @param partition A #UDisksPartition.
             * @returns A #UDisksPartitionTable. Free with g_object_unref().
             */
            get_partition_table(partition: Partition): PartitionTable;
            /**
             * Gets a human readable localized string for `partition_table_type` and `partition_table_subtype`.
             * @param partition_table_type A partition table type e.g. 'dos' or 'gpt'.
             * @param partition_table_subtype A partition table type e.g. 'dos' or 'gpt'.
             * @returns A description of @partition_table_type and @partition_table_subtype or %NULL.
             */
            get_partition_table_subtype_for_display(
                partition_table_type: string,
                partition_table_subtype: string,
            ): string;
            /**
             * Gets all known subtypes for `partition_table_type`.
             * @param partition_table_type A partition table type e.g. 'dos' or 'gpt'.
             * @returns A %NULL-terminated array of strings. Only the container should be freed with g_free().
             */
            get_partition_table_subtypes(partition_table_type: string): string[];
            /**
             * Gets a human readable localized string for `partition_table_type`.
             * @param partition_table_type A partition table type e.g. 'dos' or 'gpt'.
             * @returns A description of @partition_table_type or %NULL.
             */
            get_partition_table_type_for_display(partition_table_type: string): string;
            /**
             * Like udisks_client_get_partition_type_for_display() but also takes
             * the partition table subtype into account, if available. This is
             * useful in scenarios where different subtypes is using the same
             * partition type.
             * @param partition_table_type A partitioning type e.g. 'dos' or 'gpt'.
             * @param partition_table_subtype A partitioning subtype or %NULL.
             * @param partition_type A partition type.
             * @returns A description of @partition_type or %NULL if unknown.
             */
            get_partition_type_and_subtype_for_display(
                partition_table_type: string,
                partition_table_subtype: string,
                partition_type: string,
            ): string;
            /**
             * Gets a human readable localized string for `partiton_table_type` and `partition_type`.
             * @param partition_table_type A partitioning type e.g. 'dos' or 'gpt'.
             * @param partition_type A partition type.
             * @returns A description of @partition_type or %NULL if unknown.
             */
            get_partition_type_for_display(partition_table_type: string, partition_type: string): string;
            /**
             * Gets information about all known partition types for `partition_table_type` and `partition_table_subtype`.
             * @param partition_table_type A partition table type e.g. 'dos' or 'gpt'.
             * @param partition_table_subtype A partition table subtype or %NULL to get all known types.
             * @returns A list of   #UDisksPartitionTypeInfo instances. The returned list should be freed   with g_list_free() after freeing each element with udisks_partition_type_info_free().
             */
            get_partition_type_infos(
                partition_table_type: string,
                partition_table_subtype?: string | null,
            ): PartitionTypeInfo[];
            /**
             * Gets all partitions of `table`.
             * @param table A #UDisksPartitionTable.
             * @returns A list of #UDisksPartition instances. The   returned list should be freed with g_list_free() after each   element has been freed with g_object_unref().
             */
            get_partitions(table: PartitionTable): Partition[];
            /**
             * Utility function to get a human-readable string that represents `size`.
             * @param size Size in bytes
             * @param use_pow2 Whether power-of-two units should be used instead of power-of-ten units.
             * @param long_string Whether to produce a long string.
             * @returns A string that should be freed with g_free().
             */
            get_size_for_display(size: number, use_pow2: boolean, long_string: boolean): string;
            /**
             * Like udisks_client_get_object() but doesn't increase the reference
             * count on the returned #UDisksObject.
             *
             * <warning>The returned object is only valid until removed so it is only safe to use this function on the thread where `client` was constructed. Use udisks_client_get_object() if on another thread.</warning>
             * @param object_path Object path.
             * @returns A #UDisksObject corresponding to @object_path or %NULL if not found.
             */
            peek_object(object_path: string): Object;
            /**
             * Queues up a #UDisksClient::changed signal and rate-limit it. See
             * the documentation for the #UDisksClient::changed property for more
             * information.
             */
            queue_changed(): void;
            /**
             * Blocks until all pending D-Bus messages have been delivered. Also
             * emits the (rate-limited) #UDisksClient::changed signal if changes
             * are currently pending.
             *
             * This is useful in two situations: 1. when using synchronous method
             * calls since e.g. D-Bus signals received while waiting for the reply
             * are queued up and dispatched after the synchronous call ends; and
             * 2. when using asynchronous calls where the return value references
             * a newly created object (such as the <link
             * linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">Manager.LoopSetup()</link> method).
             */
            settle(): void;

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): Client;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace DriveAtaProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    DriveAta.ConstructorProps {}
        }

        /**
         * The #UDisksDriveAtaProxy structure contains only private data and should only be accessed using the provided API.
         */
        class DriveAtaProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<DriveAtaProxy>, Gio.DBusInterface, Gio.Initable, DriveAta
        {
            static $gtype: GObject.GType<DriveAtaProxy>;

            // Constructors

            constructor(properties?: Partial<DriveAtaProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): DriveAtaProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): DriveAtaProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): DriveAtaProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): DriveAtaProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<DriveAtaProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_drive_ata_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<DriveAtaProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamEnabled">"AamEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_enabled(): boolean;
            set aam_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamEnabled">"AamEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamEnabled(): boolean;
            set aamEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamSupported">"AamSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_supported(): boolean;
            set aam_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamSupported">"AamSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamSupported(): boolean;
            set aamSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamVendorRecommendedValue">"AamVendorRecommendedValue"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_vendor_recommended_value(): number;
            set aam_vendor_recommended_value(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamVendorRecommendedValue">"AamVendorRecommendedValue"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamVendorRecommendedValue(): number;
            set aamVendorRecommendedValue(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmEnabled">"ApmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apm_enabled(): boolean;
            set apm_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmEnabled">"ApmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apmEnabled(): boolean;
            set apmEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmSupported">"ApmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apm_supported(): boolean;
            set apm_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmSupported">"ApmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apmSupported(): boolean;
            set apmSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmEnabled">"PmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pm_enabled(): boolean;
            set pm_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmEnabled">"PmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pmEnabled(): boolean;
            set pmEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmSupported">"PmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pm_supported(): boolean;
            set pm_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmSupported">"PmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pmSupported(): boolean;
            set pmSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadEnabled">"ReadLookaheadEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_lookahead_enabled(): boolean;
            set read_lookahead_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadEnabled">"ReadLookaheadEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readLookaheadEnabled(): boolean;
            set readLookaheadEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadSupported">"ReadLookaheadSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_lookahead_supported(): boolean;
            set read_lookahead_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadSupported">"ReadLookaheadSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readLookaheadSupported(): boolean;
            set readLookaheadSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEnhancedEraseUnitMinutes">"SecurityEnhancedEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_enhanced_erase_unit_minutes(): number;
            set security_enhanced_erase_unit_minutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEnhancedEraseUnitMinutes">"SecurityEnhancedEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityEnhancedEraseUnitMinutes(): number;
            set securityEnhancedEraseUnitMinutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnitMinutes">"SecurityEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_erase_unit_minutes(): number;
            set security_erase_unit_minutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnitMinutes">"SecurityEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityEraseUnitMinutes(): number;
            set securityEraseUnitMinutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityFrozen">"SecurityFrozen"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_frozen(): boolean;
            set security_frozen(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityFrozen">"SecurityFrozen"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityFrozen(): boolean;
            set securityFrozen(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartEnabled">"SmartEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_enabled(): boolean;
            set smart_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartEnabled">"SmartEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartEnabled(): boolean;
            set smartEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartFailing">"SmartFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_failing(): boolean;
            set smart_failing(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartFailing">"SmartFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartFailing(): boolean;
            set smartFailing(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailedInThePast">"SmartNumAttributesFailedInThePast"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_attributes_failed_in_the_past(): number;
            set smart_num_attributes_failed_in_the_past(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailedInThePast">"SmartNumAttributesFailedInThePast"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumAttributesFailedInThePast(): number;
            set smartNumAttributesFailedInThePast(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailing">"SmartNumAttributesFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_attributes_failing(): number;
            set smart_num_attributes_failing(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailing">"SmartNumAttributesFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumAttributesFailing(): number;
            set smartNumAttributesFailing(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumBadSectors">"SmartNumBadSectors"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_bad_sectors(): number;
            set smart_num_bad_sectors(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumBadSectors">"SmartNumBadSectors"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumBadSectors(): number;
            set smartNumBadSectors(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartPowerOnSeconds">"SmartPowerOnSeconds"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_power_on_seconds(): number;
            set smart_power_on_seconds(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartPowerOnSeconds">"SmartPowerOnSeconds"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartPowerOnSeconds(): number;
            set smartPowerOnSeconds(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_percent_remaining(): number;
            set smart_selftest_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestPercentRemaining(): number;
            set smartSelftestPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_status(): string;
            set smart_selftest_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestStatus(): string;
            set smartSelftestStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSupported">"SmartSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_supported(): boolean;
            set smart_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSupported">"SmartSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSupported(): boolean;
            set smartSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_temperature(): number;
            set smart_temperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartTemperature(): number;
            set smartTemperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_updated(): number;
            set smart_updated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartUpdated(): number;
            set smartUpdated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheEnabled">"WriteCacheEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get write_cache_enabled(): boolean;
            set write_cache_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheEnabled">"WriteCacheEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get writeCacheEnabled(): boolean;
            set writeCacheEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheSupported">"WriteCacheSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get write_cache_supported(): boolean;
            set write_cache_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheSupported">"WriteCacheSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get writeCacheSupported(): boolean;
            set writeCacheSupported(val: boolean);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): DriveAtaProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_get_state(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<number>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_get_state(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_get_state(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<number> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_get_state().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_get_state().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_get_state_finish(res: Gio.AsyncResult): [boolean, number];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_get_state() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_get_state_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, number];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_standby(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_standby(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_standby(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_standby().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_standby().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_standby_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_standby() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_standby_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_wakeup(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_wakeup(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_wakeup(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_wakeup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_wakeup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_wakeup_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_wakeup() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_wakeup_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_security_erase_unit(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_security_erase_unit(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_security_erase_unit(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_security_erase_unit().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_security_erase_unit().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_security_erase_unit_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_security_erase_unit() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_security_erase_unit_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_get_attributes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_get_attributes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_get_attributes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_selftest_abort().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_selftest_abort().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_selftest_abort() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_selftest_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_selftest_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_selftest_start() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_set_enabled().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_set_enabled().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_set_enabled_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_set_enabled() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_set_enabled_sync(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_update(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_update().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_update().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_update() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param state Parameter to return.
             */
            complete_pm_get_state(invocation: Gio.DBusMethodInvocation, state: number): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_pm_standby(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_pm_wakeup(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_security_erase_unit(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param attributes Parameter to return.
             */
            complete_smart_get_attributes(invocation: Gio.DBusMethodInvocation, attributes: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_abort(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_set_enabled(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_update(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-get-state signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_get_state(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-standby signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_standby(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-wakeup signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_wakeup(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-security-erase-unit signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_security_erase_unit(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-get-attributes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_get_attributes(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-selftest-abort signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_selftest_abort(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-selftest-start signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_smart_selftest_start(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-set-enabled signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_smart_set_enabled(
                invocation: Gio.DBusMethodInvocation,
                arg_value: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-update signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_update(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace DriveAtaSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    DriveAta.ConstructorProps {}
        }

        /**
         * The #UDisksDriveAtaSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class DriveAtaSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, DriveAta {
            static $gtype: GObject.GType<DriveAtaSkeleton>;

            // Constructors

            constructor(properties?: Partial<DriveAtaSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): DriveAtaSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamEnabled">"AamEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_enabled(): boolean;
            set aam_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamEnabled">"AamEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamEnabled(): boolean;
            set aamEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamSupported">"AamSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_supported(): boolean;
            set aam_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamSupported">"AamSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamSupported(): boolean;
            set aamSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamVendorRecommendedValue">"AamVendorRecommendedValue"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_vendor_recommended_value(): number;
            set aam_vendor_recommended_value(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamVendorRecommendedValue">"AamVendorRecommendedValue"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamVendorRecommendedValue(): number;
            set aamVendorRecommendedValue(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmEnabled">"ApmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apm_enabled(): boolean;
            set apm_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmEnabled">"ApmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apmEnabled(): boolean;
            set apmEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmSupported">"ApmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apm_supported(): boolean;
            set apm_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmSupported">"ApmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apmSupported(): boolean;
            set apmSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmEnabled">"PmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pm_enabled(): boolean;
            set pm_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmEnabled">"PmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pmEnabled(): boolean;
            set pmEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmSupported">"PmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pm_supported(): boolean;
            set pm_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmSupported">"PmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pmSupported(): boolean;
            set pmSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadEnabled">"ReadLookaheadEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_lookahead_enabled(): boolean;
            set read_lookahead_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadEnabled">"ReadLookaheadEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readLookaheadEnabled(): boolean;
            set readLookaheadEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadSupported">"ReadLookaheadSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_lookahead_supported(): boolean;
            set read_lookahead_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadSupported">"ReadLookaheadSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readLookaheadSupported(): boolean;
            set readLookaheadSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEnhancedEraseUnitMinutes">"SecurityEnhancedEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_enhanced_erase_unit_minutes(): number;
            set security_enhanced_erase_unit_minutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEnhancedEraseUnitMinutes">"SecurityEnhancedEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityEnhancedEraseUnitMinutes(): number;
            set securityEnhancedEraseUnitMinutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnitMinutes">"SecurityEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_erase_unit_minutes(): number;
            set security_erase_unit_minutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnitMinutes">"SecurityEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityEraseUnitMinutes(): number;
            set securityEraseUnitMinutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityFrozen">"SecurityFrozen"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_frozen(): boolean;
            set security_frozen(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityFrozen">"SecurityFrozen"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityFrozen(): boolean;
            set securityFrozen(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartEnabled">"SmartEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_enabled(): boolean;
            set smart_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartEnabled">"SmartEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartEnabled(): boolean;
            set smartEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartFailing">"SmartFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_failing(): boolean;
            set smart_failing(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartFailing">"SmartFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartFailing(): boolean;
            set smartFailing(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailedInThePast">"SmartNumAttributesFailedInThePast"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_attributes_failed_in_the_past(): number;
            set smart_num_attributes_failed_in_the_past(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailedInThePast">"SmartNumAttributesFailedInThePast"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumAttributesFailedInThePast(): number;
            set smartNumAttributesFailedInThePast(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailing">"SmartNumAttributesFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_attributes_failing(): number;
            set smart_num_attributes_failing(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailing">"SmartNumAttributesFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumAttributesFailing(): number;
            set smartNumAttributesFailing(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumBadSectors">"SmartNumBadSectors"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_bad_sectors(): number;
            set smart_num_bad_sectors(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumBadSectors">"SmartNumBadSectors"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumBadSectors(): number;
            set smartNumBadSectors(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartPowerOnSeconds">"SmartPowerOnSeconds"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_power_on_seconds(): number;
            set smart_power_on_seconds(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartPowerOnSeconds">"SmartPowerOnSeconds"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartPowerOnSeconds(): number;
            set smartPowerOnSeconds(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_percent_remaining(): number;
            set smart_selftest_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestPercentRemaining(): number;
            set smartSelftestPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_status(): string;
            set smart_selftest_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestStatus(): string;
            set smartSelftestStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSupported">"SmartSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_supported(): boolean;
            set smart_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSupported">"SmartSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSupported(): boolean;
            set smartSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_temperature(): number;
            set smart_temperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartTemperature(): number;
            set smartTemperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_updated(): number;
            set smart_updated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartUpdated(): number;
            set smartUpdated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheEnabled">"WriteCacheEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get write_cache_enabled(): boolean;
            set write_cache_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheEnabled">"WriteCacheEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get writeCacheEnabled(): boolean;
            set writeCacheEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheSupported">"WriteCacheSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get write_cache_supported(): boolean;
            set write_cache_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheSupported">"WriteCacheSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get writeCacheSupported(): boolean;
            set writeCacheSupported(val: boolean);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_get_state(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<number>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_get_state(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_get_state(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<number> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_get_state().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_get_state().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_get_state_finish(res: Gio.AsyncResult): [boolean, number];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_get_state() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_get_state_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, number];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_standby(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_standby(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_standby(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_standby().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_standby().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_standby_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_standby() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_standby_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_wakeup(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_wakeup(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_wakeup(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_wakeup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_wakeup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_wakeup_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_wakeup() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_wakeup_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_security_erase_unit(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_security_erase_unit(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_security_erase_unit(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_security_erase_unit().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_security_erase_unit().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_security_erase_unit_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_security_erase_unit() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_security_erase_unit_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_get_attributes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_get_attributes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_get_attributes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_selftest_abort().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_selftest_abort().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_selftest_abort() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_selftest_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_selftest_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_selftest_start() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_set_enabled().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_set_enabled().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_set_enabled_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_set_enabled() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_set_enabled_sync(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_update(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_update().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_update().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_update() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param state Parameter to return.
             */
            complete_pm_get_state(invocation: Gio.DBusMethodInvocation, state: number): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_pm_standby(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_pm_wakeup(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_security_erase_unit(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param attributes Parameter to return.
             */
            complete_smart_get_attributes(invocation: Gio.DBusMethodInvocation, attributes: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_abort(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_set_enabled(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_update(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-get-state signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_get_state(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-standby signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_standby(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-wakeup signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_wakeup(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-security-erase-unit signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_security_erase_unit(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-get-attributes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_get_attributes(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-selftest-abort signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_selftest_abort(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-selftest-start signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_smart_selftest_start(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-set-enabled signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_smart_set_enabled(
                invocation: Gio.DBusMethodInvocation,
                arg_value: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-update signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_update(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace DriveProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Drive.ConstructorProps {}
        }

        /**
         * The #UDisksDriveProxy structure contains only private data and should only be accessed using the provided API.
         */
        class DriveProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<DriveProxy>, Gio.DBusInterface, Gio.Initable, Drive
        {
            static $gtype: GObject.GType<DriveProxy>;

            // Constructors

            constructor(properties?: Partial<DriveProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): DriveProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): DriveProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): DriveProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): DriveProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_drive_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<DriveProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_drive_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_drive_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<DriveProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.CanPowerOff">"CanPowerOff"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get can_power_off(): boolean;
            set can_power_off(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.CanPowerOff">"CanPowerOff"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get canPowerOff(): boolean;
            set canPowerOff(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Configuration">"Configuration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get configuration(): GLib.Variant;
            set configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.ConnectionBus">"ConnectionBus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get connection_bus(): string;
            set connection_bus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.ConnectionBus">"ConnectionBus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get connectionBus(): string;
            set connectionBus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Ejectable">"Ejectable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get ejectable(): boolean;
            set ejectable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Id">"Id"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id(): string;
            set id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Media">"Media"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media(): string;
            set media(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaAvailable">"MediaAvailable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_available(): boolean;
            set media_available(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaAvailable">"MediaAvailable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaAvailable(): boolean;
            set mediaAvailable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaChangeDetected">"MediaChangeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_change_detected(): boolean;
            set media_change_detected(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaChangeDetected">"MediaChangeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaChangeDetected(): boolean;
            set mediaChangeDetected(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaCompatibility">"MediaCompatibility"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_compatibility(): string[];
            set media_compatibility(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaCompatibility">"MediaCompatibility"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaCompatibility(): string[];
            set mediaCompatibility(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaRemovable">"MediaRemovable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_removable(): boolean;
            set media_removable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaRemovable">"MediaRemovable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaRemovable(): boolean;
            set mediaRemovable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Model">"Model"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get model(): string;
            set model(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Optical">"Optical"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical(): boolean;
            set optical(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalBlank">"OpticalBlank"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_blank(): boolean;
            set optical_blank(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalBlank">"OpticalBlank"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalBlank(): boolean;
            set opticalBlank(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumAudioTracks">"OpticalNumAudioTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_audio_tracks(): number;
            set optical_num_audio_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumAudioTracks">"OpticalNumAudioTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumAudioTracks(): number;
            set opticalNumAudioTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumDataTracks">"OpticalNumDataTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_data_tracks(): number;
            set optical_num_data_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumDataTracks">"OpticalNumDataTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumDataTracks(): number;
            set opticalNumDataTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumSessions">"OpticalNumSessions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_sessions(): number;
            set optical_num_sessions(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumSessions">"OpticalNumSessions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumSessions(): number;
            set opticalNumSessions(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumTracks">"OpticalNumTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_tracks(): number;
            set optical_num_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumTracks">"OpticalNumTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumTracks(): number;
            set opticalNumTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Removable">"Removable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get removable(): boolean;
            set removable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Revision">"Revision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get revision(): string;
            set revision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.RotationRate">"RotationRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rotation_rate(): number;
            set rotation_rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.RotationRate">"RotationRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rotationRate(): number;
            set rotationRate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Seat">"Seat"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get seat(): string;
            set seat(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Serial">"Serial"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get serial(): string;
            set serial(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SiblingId">"SiblingId"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sibling_id(): string;
            set sibling_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SiblingId">"SiblingId"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get siblingId(): string;
            set siblingId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SortKey">"SortKey"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sort_key(): string;
            set sort_key(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SortKey">"SortKey"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sortKey(): string;
            set sortKey(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeDetected">"TimeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get time_detected(): number;
            set time_detected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeDetected">"TimeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get timeDetected(): number;
            set timeDetected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeMediaDetected">"TimeMediaDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get time_media_detected(): number;
            set time_media_detected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeMediaDetected">"TimeMediaDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get timeMediaDetected(): number;
            set timeMediaDetected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Vendor">"Vendor"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vendor(): string;
            set vendor(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.WWN">"WWN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get wwn(): string;
            set wwn(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): DriveProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_eject(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_eject(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_eject(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_eject().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_eject().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_eject_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_eject() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_eject_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_power_off(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_power_off(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_power_off(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_power_off().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_power_off().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_power_off_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_power_off() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_power_off_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_set_configuration().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_set_configuration().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_configuration_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_set_configuration() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_configuration_sync(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_eject(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_power_off(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_configuration(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksDrive::handle-eject signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_eject(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDrive::handle-power-off signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_power_off(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDrive::handle-set-configuration signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_configuration(
                invocation: Gio.DBusMethodInvocation,
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace DriveSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Drive.ConstructorProps {}
        }

        /**
         * The #UDisksDriveSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class DriveSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Drive {
            static $gtype: GObject.GType<DriveSkeleton>;

            // Constructors

            constructor(properties?: Partial<DriveSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): DriveSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.CanPowerOff">"CanPowerOff"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get can_power_off(): boolean;
            set can_power_off(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.CanPowerOff">"CanPowerOff"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get canPowerOff(): boolean;
            set canPowerOff(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Configuration">"Configuration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get configuration(): GLib.Variant;
            set configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.ConnectionBus">"ConnectionBus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get connection_bus(): string;
            set connection_bus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.ConnectionBus">"ConnectionBus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get connectionBus(): string;
            set connectionBus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Ejectable">"Ejectable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get ejectable(): boolean;
            set ejectable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Id">"Id"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id(): string;
            set id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Media">"Media"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media(): string;
            set media(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaAvailable">"MediaAvailable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_available(): boolean;
            set media_available(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaAvailable">"MediaAvailable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaAvailable(): boolean;
            set mediaAvailable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaChangeDetected">"MediaChangeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_change_detected(): boolean;
            set media_change_detected(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaChangeDetected">"MediaChangeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaChangeDetected(): boolean;
            set mediaChangeDetected(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaCompatibility">"MediaCompatibility"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_compatibility(): string[];
            set media_compatibility(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaCompatibility">"MediaCompatibility"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaCompatibility(): string[];
            set mediaCompatibility(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaRemovable">"MediaRemovable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_removable(): boolean;
            set media_removable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaRemovable">"MediaRemovable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaRemovable(): boolean;
            set mediaRemovable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Model">"Model"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get model(): string;
            set model(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Optical">"Optical"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical(): boolean;
            set optical(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalBlank">"OpticalBlank"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_blank(): boolean;
            set optical_blank(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalBlank">"OpticalBlank"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalBlank(): boolean;
            set opticalBlank(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumAudioTracks">"OpticalNumAudioTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_audio_tracks(): number;
            set optical_num_audio_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumAudioTracks">"OpticalNumAudioTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumAudioTracks(): number;
            set opticalNumAudioTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumDataTracks">"OpticalNumDataTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_data_tracks(): number;
            set optical_num_data_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumDataTracks">"OpticalNumDataTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumDataTracks(): number;
            set opticalNumDataTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumSessions">"OpticalNumSessions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_sessions(): number;
            set optical_num_sessions(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumSessions">"OpticalNumSessions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumSessions(): number;
            set opticalNumSessions(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumTracks">"OpticalNumTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_tracks(): number;
            set optical_num_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumTracks">"OpticalNumTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumTracks(): number;
            set opticalNumTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Removable">"Removable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get removable(): boolean;
            set removable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Revision">"Revision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get revision(): string;
            set revision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.RotationRate">"RotationRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rotation_rate(): number;
            set rotation_rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.RotationRate">"RotationRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rotationRate(): number;
            set rotationRate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Seat">"Seat"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get seat(): string;
            set seat(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Serial">"Serial"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get serial(): string;
            set serial(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SiblingId">"SiblingId"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sibling_id(): string;
            set sibling_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SiblingId">"SiblingId"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get siblingId(): string;
            set siblingId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SortKey">"SortKey"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sort_key(): string;
            set sort_key(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SortKey">"SortKey"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sortKey(): string;
            set sortKey(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeDetected">"TimeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get time_detected(): number;
            set time_detected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeDetected">"TimeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get timeDetected(): number;
            set timeDetected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeMediaDetected">"TimeMediaDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get time_media_detected(): number;
            set time_media_detected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeMediaDetected">"TimeMediaDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get timeMediaDetected(): number;
            set timeMediaDetected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Vendor">"Vendor"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vendor(): string;
            set vendor(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.WWN">"WWN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get wwn(): string;
            set wwn(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_eject(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_eject(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_eject(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_eject().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_eject().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_eject_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_eject() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_eject_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_power_off(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_power_off(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_power_off(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_power_off().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_power_off().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_power_off_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_power_off() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_power_off_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_set_configuration().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_set_configuration().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_configuration_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_set_configuration() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_configuration_sync(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_eject(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_power_off(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_configuration(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksDrive::handle-eject signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_eject(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDrive::handle-power-off signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_power_off(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDrive::handle-set-configuration signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_configuration(
                invocation: Gio.DBusMethodInvocation,
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace EncryptedProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Encrypted.ConstructorProps {}
        }

        /**
         * The #UDisksEncryptedProxy structure contains only private data and should only be accessed using the provided API.
         */
        class EncryptedProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<EncryptedProxy>, Gio.DBusInterface, Gio.Initable, Encrypted
        {
            static $gtype: GObject.GType<EncryptedProxy>;

            // Constructors

            constructor(properties?: Partial<EncryptedProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): EncryptedProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): EncryptedProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): EncryptedProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): EncryptedProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_encrypted_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<EncryptedProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_encrypted_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_encrypted_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<EncryptedProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.CleartextDevice">"CleartextDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cleartext_device(): string;
            set cleartext_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.CleartextDevice">"CleartextDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cleartextDevice(): string;
            set cleartextDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.HintEncryptionType">"HintEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_encryption_type(): string;
            set hint_encryption_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.HintEncryptionType">"HintEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintEncryptionType(): string;
            set hintEncryptionType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.MetadataSize">"MetadataSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadata_size(): number;
            set metadata_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.MetadataSize">"MetadataSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadataSize(): number;
            set metadataSize(val: number);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): EncryptedProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_change_passphrase().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_change_passphrase().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_change_passphrase_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_change_passphrase() for the asynchronous version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_change_passphrase_sync(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_lock(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_lock(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_lock(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_lock().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_lock().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_lock_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_lock() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_lock_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_unlock().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_unlock().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unlock_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_unlock() for the asynchronous version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unlock_sync(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_change_passphrase(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_lock(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param cleartext_device Parameter to return.
             */
            complete_unlock(invocation: Gio.DBusMethodInvocation, cleartext_device: string): void;
            /**
             * Handler for the #UDisksEncrypted::handle-change-passphrase signal.
             * @param invocation
             * @param arg_passphrase
             * @param arg_new_passphrase
             * @param arg_options
             */
            vfunc_handle_change_passphrase(
                invocation: Gio.DBusMethodInvocation,
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-lock signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_lock(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-unlock signal.
             * @param invocation
             * @param arg_passphrase
             * @param arg_options
             */
            vfunc_handle_unlock(
                invocation: Gio.DBusMethodInvocation,
                arg_passphrase: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace EncryptedSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Encrypted.ConstructorProps {}
        }

        /**
         * The #UDisksEncryptedSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class EncryptedSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Encrypted {
            static $gtype: GObject.GType<EncryptedSkeleton>;

            // Constructors

            constructor(properties?: Partial<EncryptedSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): EncryptedSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.CleartextDevice">"CleartextDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cleartext_device(): string;
            set cleartext_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.CleartextDevice">"CleartextDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cleartextDevice(): string;
            set cleartextDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.HintEncryptionType">"HintEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_encryption_type(): string;
            set hint_encryption_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.HintEncryptionType">"HintEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintEncryptionType(): string;
            set hintEncryptionType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.MetadataSize">"MetadataSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadata_size(): number;
            set metadata_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.MetadataSize">"MetadataSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadataSize(): number;
            set metadataSize(val: number);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_change_passphrase().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_change_passphrase().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_change_passphrase_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_change_passphrase() for the asynchronous version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_change_passphrase_sync(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_lock(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_lock(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_lock(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_lock().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_lock().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_lock_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_lock() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_lock_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_unlock().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_unlock().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unlock_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_unlock() for the asynchronous version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unlock_sync(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_change_passphrase(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_lock(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param cleartext_device Parameter to return.
             */
            complete_unlock(invocation: Gio.DBusMethodInvocation, cleartext_device: string): void;
            /**
             * Handler for the #UDisksEncrypted::handle-change-passphrase signal.
             * @param invocation
             * @param arg_passphrase
             * @param arg_new_passphrase
             * @param arg_options
             */
            vfunc_handle_change_passphrase(
                invocation: Gio.DBusMethodInvocation,
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-lock signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_lock(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-unlock signal.
             * @param invocation
             * @param arg_passphrase
             * @param arg_options
             */
            vfunc_handle_unlock(
                invocation: Gio.DBusMethodInvocation,
                arg_passphrase: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace FilesystemBTRFSProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    FilesystemBTRFS.ConstructorProps {}
        }

        /**
         * The #UDisksFilesystemBTRFSProxy structure contains only private data and should only be accessed using the provided API.
         */
        class FilesystemBTRFSProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<FilesystemBTRFSProxy>, Gio.DBusInterface, Gio.Initable, FilesystemBTRFS
        {
            static $gtype: GObject.GType<FilesystemBTRFSProxy>;

            // Constructors

            constructor(properties?: Partial<FilesystemBTRFSProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): FilesystemBTRFSProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): FilesystemBTRFSProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): FilesystemBTRFSProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): FilesystemBTRFSProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<FilesystemBTRFSProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_filesystem_btrfs_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<FilesystemBTRFSProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.label">"label"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get label(): string;
            set label(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.num_devices">"num_devices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get num_devices(): number;
            set num_devices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.num_devices">"num_devices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get numDevices(): number;
            set numDevices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.used">"used"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get used(): number;
            set used(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.uuid">"uuid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): FilesystemBTRFSProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_add_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_create_snapshot().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_create_snapshot().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot() for the asynchronous version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_sync(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_create_subvolume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_create_subvolume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_subvolume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_subvolume_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, number]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, number]> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_get_subvolumes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_get_subvolumes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_subvolumes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, number];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes() for the asynchronous version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_subvolumes_sync(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, number];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_remove_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_remove_subvolume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_remove_subvolume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_subvolume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_subvolume_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_repair() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_snapshot(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_subvolume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param subvolumes Parameter to return.
             * @param subvolumes_cnt Parameter to return.
             */
            complete_get_subvolumes(
                invocation: Gio.DBusMethodInvocation,
                subvolumes: GLib.Variant,
                subvolumes_cnt: number,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_subvolume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-add-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-create-snapshot signal.
             * @param invocation
             * @param arg_source
             * @param arg_dest
             * @param arg_ro
             * @param arg_options
             */
            vfunc_handle_create_snapshot(
                invocation: Gio.DBusMethodInvocation,
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-create-subvolume signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_create_subvolume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-get-subvolumes signal.
             * @param invocation
             * @param arg_snapshots_only
             * @param arg_options
             */
            vfunc_handle_get_subvolumes(
                invocation: Gio.DBusMethodInvocation,
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-remove-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-remove-subvolume signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_remove_subvolume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-repair signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_repair(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace FilesystemBTRFSSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    FilesystemBTRFS.ConstructorProps {}
        }

        /**
         * The #UDisksFilesystemBTRFSSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class FilesystemBTRFSSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, FilesystemBTRFS {
            static $gtype: GObject.GType<FilesystemBTRFSSkeleton>;

            // Constructors

            constructor(properties?: Partial<FilesystemBTRFSSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): FilesystemBTRFSSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.label">"label"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get label(): string;
            set label(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.num_devices">"num_devices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get num_devices(): number;
            set num_devices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.num_devices">"num_devices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get numDevices(): number;
            set numDevices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.used">"used"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get used(): number;
            set used(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.uuid">"uuid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_add_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_create_snapshot().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_create_snapshot().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot() for the asynchronous version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_sync(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_create_subvolume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_create_subvolume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_subvolume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_subvolume_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, number]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, number]> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_get_subvolumes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_get_subvolumes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_subvolumes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, number];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes() for the asynchronous version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_subvolumes_sync(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, number];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_remove_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_remove_subvolume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_remove_subvolume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_subvolume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_subvolume_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_repair() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_snapshot(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_subvolume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param subvolumes Parameter to return.
             * @param subvolumes_cnt Parameter to return.
             */
            complete_get_subvolumes(
                invocation: Gio.DBusMethodInvocation,
                subvolumes: GLib.Variant,
                subvolumes_cnt: number,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_subvolume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-add-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-create-snapshot signal.
             * @param invocation
             * @param arg_source
             * @param arg_dest
             * @param arg_ro
             * @param arg_options
             */
            vfunc_handle_create_snapshot(
                invocation: Gio.DBusMethodInvocation,
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-create-subvolume signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_create_subvolume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-get-subvolumes signal.
             * @param invocation
             * @param arg_snapshots_only
             * @param arg_options
             */
            vfunc_handle_get_subvolumes(
                invocation: Gio.DBusMethodInvocation,
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-remove-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-remove-subvolume signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_remove_subvolume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-repair signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_repair(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace FilesystemProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Filesystem.ConstructorProps {}
        }

        /**
         * The #UDisksFilesystemProxy structure contains only private data and should only be accessed using the provided API.
         */
        class FilesystemProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<FilesystemProxy>, Gio.DBusInterface, Gio.Initable, Filesystem
        {
            static $gtype: GObject.GType<FilesystemProxy>;

            // Constructors

            constructor(properties?: Partial<FilesystemProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): FilesystemProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): FilesystemProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): FilesystemProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): FilesystemProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_filesystem_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<FilesystemProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_filesystem_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_filesystem_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<FilesystemProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.MountPoints">"MountPoints"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mount_points(): string[];
            set mount_points(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.MountPoints">"MountPoints"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mountPoints(): string[];
            set mountPoints(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): FilesystemProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_check(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_check(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_check(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_check().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_check().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_check_finish(res: Gio.AsyncResult): [boolean, boolean];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_check() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_check_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, boolean];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_mount(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mount(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mount(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_mount().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_mount().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mount_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_mount() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mount_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): [boolean, boolean];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_repair() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, boolean];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_take_ownership(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_take_ownership(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_take_ownership(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_take_ownership().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_take_ownership().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_take_ownership_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_take_ownership() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_take_ownership_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_unmount(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unmount(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unmount(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_unmount().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_unmount().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unmount_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_unmount() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unmount_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param consistent Parameter to return.
             */
            complete_check(invocation: Gio.DBusMethodInvocation, consistent: boolean): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param mount_path Parameter to return.
             */
            complete_mount(invocation: Gio.DBusMethodInvocation, mount_path: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param repaired Parameter to return.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation, repaired: boolean): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_take_ownership(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_unmount(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksFilesystem::handle-check signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_check(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-mount signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_mount(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-repair signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_repair(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-take-ownership signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_take_ownership(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-unmount signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_unmount(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace FilesystemSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Filesystem.ConstructorProps {}
        }

        /**
         * The #UDisksFilesystemSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class FilesystemSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Filesystem {
            static $gtype: GObject.GType<FilesystemSkeleton>;

            // Constructors

            constructor(properties?: Partial<FilesystemSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): FilesystemSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.MountPoints">"MountPoints"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mount_points(): string[];
            set mount_points(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.MountPoints">"MountPoints"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mountPoints(): string[];
            set mountPoints(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_check(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_check(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_check(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_check().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_check().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_check_finish(res: Gio.AsyncResult): [boolean, boolean];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_check() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_check_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, boolean];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_mount(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mount(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mount(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_mount().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_mount().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mount_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_mount() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mount_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): [boolean, boolean];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_repair() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, boolean];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_take_ownership(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_take_ownership(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_take_ownership(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_take_ownership().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_take_ownership().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_take_ownership_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_take_ownership() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_take_ownership_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_unmount(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unmount(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unmount(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_unmount().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_unmount().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unmount_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_unmount() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unmount_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param consistent Parameter to return.
             */
            complete_check(invocation: Gio.DBusMethodInvocation, consistent: boolean): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param mount_path Parameter to return.
             */
            complete_mount(invocation: Gio.DBusMethodInvocation, mount_path: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param repaired Parameter to return.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation, repaired: boolean): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_take_ownership(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_unmount(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksFilesystem::handle-check signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_check(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-mount signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_mount(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-repair signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_repair(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-take-ownership signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_take_ownership(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-unmount signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_unmount(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace JobProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Job.ConstructorProps {}
        }

        /**
         * The #UDisksJobProxy structure contains only private data and should only be accessed using the provided API.
         */
        class JobProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<JobProxy>, Gio.DBusInterface, Gio.Initable, Job
        {
            static $gtype: GObject.GType<JobProxy>;

            // Constructors

            constructor(properties?: Partial<JobProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): JobProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): JobProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): JobProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): JobProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_job_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<JobProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_job_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_job_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<JobProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Bytes">"Bytes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bytes(): number;
            set bytes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Cancelable">"Cancelable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cancelable(): boolean;
            set cancelable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ExpectedEndTime">"ExpectedEndTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get expected_end_time(): number;
            set expected_end_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ExpectedEndTime">"ExpectedEndTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get expectedEndTime(): number;
            set expectedEndTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Objects">"Objects"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get objects(): string[];
            set objects(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Operation">"Operation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operation(): string;
            set operation(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Progress">"Progress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progress(): number;
            set progress(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ProgressValid">"ProgressValid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progress_valid(): boolean;
            set progress_valid(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ProgressValid">"ProgressValid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progressValid(): boolean;
            set progressValid(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Rate">"Rate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rate(): number;
            set rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartTime">"StartTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get start_time(): number;
            set start_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartTime">"StartTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get startTime(): number;
            set startTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartedByUID">"StartedByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get started_by_uid(): number;
            set started_by_uid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartedByUID">"StartedByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get startedByUid(): number;
            set startedByUid(val: number);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): JobProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cancel(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cancel(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cancel(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_job_call_cancel().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_job_call_cancel().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cancel_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_job_call_cancel() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cancel_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cancel(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Emits the <link linkend="gdbus-signal-org-freedesktop-UDisks2-Job.Completed">"Completed"</link> D-Bus signal.
             * @param arg_success Argument to pass with the signal.
             * @param arg_message Argument to pass with the signal.
             */
            emit_completed(arg_success: boolean, arg_message: string): void;
            /**
             * Handler for the #UDisksJob::completed signal.
             * @param arg_success
             * @param arg_message
             */
            vfunc_completed(arg_success: boolean, arg_message: string): void;
            /**
             * Handler for the #UDisksJob::handle-cancel signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cancel(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace JobSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Job.ConstructorProps {}
        }

        /**
         * The #UDisksJobSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class JobSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Job {
            static $gtype: GObject.GType<JobSkeleton>;

            // Constructors

            constructor(properties?: Partial<JobSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): JobSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Bytes">"Bytes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bytes(): number;
            set bytes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Cancelable">"Cancelable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cancelable(): boolean;
            set cancelable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ExpectedEndTime">"ExpectedEndTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get expected_end_time(): number;
            set expected_end_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ExpectedEndTime">"ExpectedEndTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get expectedEndTime(): number;
            set expectedEndTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Objects">"Objects"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get objects(): string[];
            set objects(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Operation">"Operation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operation(): string;
            set operation(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Progress">"Progress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progress(): number;
            set progress(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ProgressValid">"ProgressValid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progress_valid(): boolean;
            set progress_valid(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ProgressValid">"ProgressValid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progressValid(): boolean;
            set progressValid(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Rate">"Rate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rate(): number;
            set rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartTime">"StartTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get start_time(): number;
            set start_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartTime">"StartTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get startTime(): number;
            set startTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartedByUID">"StartedByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get started_by_uid(): number;
            set started_by_uid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartedByUID">"StartedByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get startedByUid(): number;
            set startedByUid(val: number);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cancel(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cancel(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cancel(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_job_call_cancel().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_job_call_cancel().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cancel_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_job_call_cancel() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cancel_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cancel(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Emits the <link linkend="gdbus-signal-org-freedesktop-UDisks2-Job.Completed">"Completed"</link> D-Bus signal.
             * @param arg_success Argument to pass with the signal.
             * @param arg_message Argument to pass with the signal.
             */
            emit_completed(arg_success: boolean, arg_message: string): void;
            /**
             * Handler for the #UDisksJob::completed signal.
             * @param arg_success
             * @param arg_message
             */
            vfunc_completed(arg_success: boolean, arg_message: string): void;
            /**
             * Handler for the #UDisksJob::handle-cancel signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cancel(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace LogicalVolumeProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    LogicalVolume.ConstructorProps {}
        }

        /**
         * The #UDisksLogicalVolumeProxy structure contains only private data and should only be accessed using the provided API.
         */
        class LogicalVolumeProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<LogicalVolumeProxy>, Gio.DBusInterface, Gio.Initable, LogicalVolume
        {
            static $gtype: GObject.GType<LogicalVolumeProxy>;

            // Constructors

            constructor(properties?: Partial<LogicalVolumeProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): LogicalVolumeProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): LogicalVolumeProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): LogicalVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): LogicalVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<LogicalVolumeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_logical_volume_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<LogicalVolumeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Active">"Active"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active(): boolean;
            set active(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.BlockDevice">"BlockDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get block_device(): string;
            set block_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.BlockDevice">"BlockDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get blockDevice(): string;
            set blockDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.DataAllocatedRatio">"DataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get data_allocated_ratio(): number;
            set data_allocated_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.DataAllocatedRatio">"DataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get dataAllocatedRatio(): number;
            set dataAllocatedRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Layout">"Layout"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get layout(): string;
            set layout(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.MetadataAllocatedRatio">"MetadataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadata_allocated_ratio(): number;
            set metadata_allocated_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.MetadataAllocatedRatio">"MetadataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadataAllocatedRatio(): number;
            set metadataAllocatedRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Origin">"Origin"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get origin(): string;
            set origin(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Structure">"Structure"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get structure(): GLib.Variant;
            set structure(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.SyncRatio">"SyncRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_ratio(): number;
            set sync_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.SyncRatio">"SyncRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRatio(): number;
            set syncRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ThinPool">"ThinPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get thin_pool(): string;
            set thin_pool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ThinPool">"ThinPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get thinPool(): string;
            set thinPool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volume_group(): string;
            set volume_group(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volumeGroup(): string;
            set volumeGroup(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): LogicalVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_activate(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_activate(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_activate(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_activate().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_activate().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_activate_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_activate() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_activate_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_attach().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_attach().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_attach_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_attach() for the asynchronous version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_attach_sync(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_detach(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_detach(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_detach(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_detach().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_detach().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_detach_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_detach() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_detach_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_split(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_split(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_split(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_split().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_split().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_split_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_split() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_split_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_create_snapshot().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_create_snapshot().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_create_snapshot() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_deactivate(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_deactivate(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_deactivate(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_deactivate().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_deactivate().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_deactivate_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_deactivate() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_deactivate_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_rename().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_rename().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_rename() for the asynchronous version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_sync(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_repair() for the asynchronous version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_pvs: string, arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_resize() for the asynchronous version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_activate(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_attach(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_detach(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_split(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_snapshot(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_deactivate(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_rename(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksLogicalVolume::handle-activate signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_activate(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-attach signal.
             * @param invocation
             * @param arg_cache_name
             * @param arg_options
             */
            vfunc_handle_cache_attach(
                invocation: Gio.DBusMethodInvocation,
                arg_cache_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-detach signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cache_detach(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-split signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cache_split(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-create-snapshot signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_snapshot(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-deactivate signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_deactivate(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-rename signal.
             * @param invocation
             * @param arg_new_name
             * @param arg_options
             */
            vfunc_handle_rename(
                invocation: Gio.DBusMethodInvocation,
                arg_new_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-repair signal.
             * @param invocation
             * @param arg_pvs
             * @param arg_options
             */
            vfunc_handle_repair(
                invocation: Gio.DBusMethodInvocation,
                arg_pvs: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-resize signal.
             * @param invocation
             * @param arg_new_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_new_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace LogicalVolumeSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    LogicalVolume.ConstructorProps {}
        }

        /**
         * The #UDisksLogicalVolumeSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class LogicalVolumeSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, LogicalVolume {
            static $gtype: GObject.GType<LogicalVolumeSkeleton>;

            // Constructors

            constructor(properties?: Partial<LogicalVolumeSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): LogicalVolumeSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Active">"Active"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active(): boolean;
            set active(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.BlockDevice">"BlockDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get block_device(): string;
            set block_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.BlockDevice">"BlockDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get blockDevice(): string;
            set blockDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.DataAllocatedRatio">"DataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get data_allocated_ratio(): number;
            set data_allocated_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.DataAllocatedRatio">"DataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get dataAllocatedRatio(): number;
            set dataAllocatedRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Layout">"Layout"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get layout(): string;
            set layout(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.MetadataAllocatedRatio">"MetadataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadata_allocated_ratio(): number;
            set metadata_allocated_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.MetadataAllocatedRatio">"MetadataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadataAllocatedRatio(): number;
            set metadataAllocatedRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Origin">"Origin"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get origin(): string;
            set origin(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Structure">"Structure"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get structure(): GLib.Variant;
            set structure(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.SyncRatio">"SyncRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_ratio(): number;
            set sync_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.SyncRatio">"SyncRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRatio(): number;
            set syncRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ThinPool">"ThinPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get thin_pool(): string;
            set thin_pool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ThinPool">"ThinPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get thinPool(): string;
            set thinPool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volume_group(): string;
            set volume_group(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volumeGroup(): string;
            set volumeGroup(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_activate(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_activate(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_activate(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_activate().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_activate().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_activate_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_activate() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_activate_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_attach().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_attach().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_attach_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_attach() for the asynchronous version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_attach_sync(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_detach(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_detach(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_detach(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_detach().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_detach().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_detach_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_detach() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_detach_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_split(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_split(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_split(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_split().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_split().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_split_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_split() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_split_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_create_snapshot().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_create_snapshot().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_create_snapshot() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_deactivate(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_deactivate(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_deactivate(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_deactivate().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_deactivate().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_deactivate_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_deactivate() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_deactivate_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_rename().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_rename().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_rename() for the asynchronous version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_sync(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_repair() for the asynchronous version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_pvs: string, arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_resize() for the asynchronous version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_activate(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_attach(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_detach(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_split(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_snapshot(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_deactivate(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_rename(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksLogicalVolume::handle-activate signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_activate(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-attach signal.
             * @param invocation
             * @param arg_cache_name
             * @param arg_options
             */
            vfunc_handle_cache_attach(
                invocation: Gio.DBusMethodInvocation,
                arg_cache_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-detach signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cache_detach(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-split signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cache_split(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-create-snapshot signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_snapshot(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-deactivate signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_deactivate(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-rename signal.
             * @param invocation
             * @param arg_new_name
             * @param arg_options
             */
            vfunc_handle_rename(
                invocation: Gio.DBusMethodInvocation,
                arg_new_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-repair signal.
             * @param invocation
             * @param arg_pvs
             * @param arg_options
             */
            vfunc_handle_repair(
                invocation: Gio.DBusMethodInvocation,
                arg_pvs: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-resize signal.
             * @param invocation
             * @param arg_new_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_new_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace LoopProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Loop.ConstructorProps {}
        }

        /**
         * The #UDisksLoopProxy structure contains only private data and should only be accessed using the provided API.
         */
        class LoopProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<LoopProxy>, Gio.DBusInterface, Gio.Initable, Loop
        {
            static $gtype: GObject.GType<LoopProxy>;

            // Constructors

            constructor(properties?: Partial<LoopProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): LoopProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): LoopProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): LoopProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): LoopProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_loop_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<LoopProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_loop_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_loop_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<LoopProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.Autoclear">"Autoclear"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get autoclear(): boolean;
            set autoclear(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.BackingFile">"BackingFile"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get backing_file(): string;
            set backing_file(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.BackingFile">"BackingFile"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get backingFile(): string;
            set backingFile(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.SetupByUID">"SetupByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get setup_by_uid(): number;
            set setup_by_uid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.SetupByUID">"SetupByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get setupByUid(): number;
            set setupByUid(val: number);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): LoopProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_loop_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_loop_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_loop_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_loop_call_set_autoclear().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_loop_call_set_autoclear().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_autoclear_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_loop_call_set_autoclear() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_autoclear_sync(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_autoclear(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksLoop::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLoop::handle-set-autoclear signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_autoclear(
                invocation: Gio.DBusMethodInvocation,
                arg_value: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace LoopSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Loop.ConstructorProps {}
        }

        /**
         * The #UDisksLoopSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class LoopSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Loop {
            static $gtype: GObject.GType<LoopSkeleton>;

            // Constructors

            constructor(properties?: Partial<LoopSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): LoopSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.Autoclear">"Autoclear"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get autoclear(): boolean;
            set autoclear(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.BackingFile">"BackingFile"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get backing_file(): string;
            set backing_file(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.BackingFile">"BackingFile"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get backingFile(): string;
            set backingFile(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.SetupByUID">"SetupByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get setup_by_uid(): number;
            set setup_by_uid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.SetupByUID">"SetupByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get setupByUid(): number;
            set setupByUid(val: number);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_loop_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_loop_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_loop_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_loop_call_set_autoclear().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_loop_call_set_autoclear().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_autoclear_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_loop_call_set_autoclear() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_autoclear_sync(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_autoclear(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksLoop::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLoop::handle-set-autoclear signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_autoclear(
                invocation: Gio.DBusMethodInvocation,
                arg_value: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace MDRaidProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    MDRaid.ConstructorProps {}
        }

        /**
         * The #UDisksMDRaidProxy structure contains only private data and should only be accessed using the provided API.
         */
        class MDRaidProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<MDRaidProxy>, Gio.DBusInterface, Gio.Initable, MDRaid
        {
            static $gtype: GObject.GType<MDRaidProxy>;

            // Constructors

            constructor(properties?: Partial<MDRaidProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): MDRaidProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): MDRaidProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): MDRaidProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): MDRaidProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_mdraid_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<MDRaidProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_mdraid_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_mdraid_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<MDRaidProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ActiveDevices">"ActiveDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active_devices(): GLib.Variant;
            set active_devices(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ActiveDevices">"ActiveDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get activeDevices(): GLib.Variant;
            set activeDevices(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.BitmapLocation">"BitmapLocation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bitmap_location(): string;
            set bitmap_location(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.BitmapLocation">"BitmapLocation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bitmapLocation(): string;
            set bitmapLocation(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChunkSize">"ChunkSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get chunk_size(): number;
            set chunk_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChunkSize">"ChunkSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get chunkSize(): number;
            set chunkSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Degraded">"Degraded"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get degraded(): number;
            set degraded(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Level">"Level"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get level(): string;
            set level(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.NumDevices">"NumDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get num_devices(): number;
            set num_devices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.NumDevices">"NumDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get numDevices(): number;
            set numDevices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Running">"Running"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get running(): boolean;
            set running(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncAction">"SyncAction"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_action(): string;
            set sync_action(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncAction">"SyncAction"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncAction(): string;
            set syncAction(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncCompleted">"SyncCompleted"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_completed(): number;
            set sync_completed(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncCompleted">"SyncCompleted"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncCompleted(): number;
            set syncCompleted(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRate">"SyncRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_rate(): number;
            set sync_rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRate">"SyncRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRate(): number;
            set syncRate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRemainingTime">"SyncRemainingTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_remaining_time(): number;
            set sync_remaining_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRemainingTime">"SyncRemainingTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRemainingTime(): number;
            set syncRemainingTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): MDRaidProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_add_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_remove_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_request_sync_action().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_request_sync_action().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_request_sync_action_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_request_sync_action() for the asynchronous version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_request_sync_action_sync(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_set_bitmap_location().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_set_bitmap_location().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_bitmap_location_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_set_bitmap_location() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_bitmap_location_sync(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_start(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_start() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_stop(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_stop().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_stop().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_stop() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_request_sync_action(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_bitmap_location(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_stop(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksMDRaid::handle-add-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-remove-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-request-sync-action signal.
             * @param invocation
             * @param arg_sync_action
             * @param arg_options
             */
            vfunc_handle_request_sync_action(
                invocation: Gio.DBusMethodInvocation,
                arg_sync_action: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-set-bitmap-location signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_bitmap_location(
                invocation: Gio.DBusMethodInvocation,
                arg_value: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-start signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_start(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-stop signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_stop(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace MDRaidSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    MDRaid.ConstructorProps {}
        }

        /**
         * The #UDisksMDRaidSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class MDRaidSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, MDRaid {
            static $gtype: GObject.GType<MDRaidSkeleton>;

            // Constructors

            constructor(properties?: Partial<MDRaidSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): MDRaidSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ActiveDevices">"ActiveDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active_devices(): GLib.Variant;
            set active_devices(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ActiveDevices">"ActiveDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get activeDevices(): GLib.Variant;
            set activeDevices(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.BitmapLocation">"BitmapLocation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bitmap_location(): string;
            set bitmap_location(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.BitmapLocation">"BitmapLocation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bitmapLocation(): string;
            set bitmapLocation(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChunkSize">"ChunkSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get chunk_size(): number;
            set chunk_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChunkSize">"ChunkSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get chunkSize(): number;
            set chunkSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Degraded">"Degraded"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get degraded(): number;
            set degraded(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Level">"Level"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get level(): string;
            set level(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.NumDevices">"NumDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get num_devices(): number;
            set num_devices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.NumDevices">"NumDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get numDevices(): number;
            set numDevices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Running">"Running"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get running(): boolean;
            set running(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncAction">"SyncAction"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_action(): string;
            set sync_action(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncAction">"SyncAction"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncAction(): string;
            set syncAction(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncCompleted">"SyncCompleted"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_completed(): number;
            set sync_completed(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncCompleted">"SyncCompleted"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncCompleted(): number;
            set syncCompleted(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRate">"SyncRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_rate(): number;
            set sync_rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRate">"SyncRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRate(): number;
            set syncRate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRemainingTime">"SyncRemainingTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_remaining_time(): number;
            set sync_remaining_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRemainingTime">"SyncRemainingTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRemainingTime(): number;
            set syncRemainingTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_add_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_remove_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_request_sync_action().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_request_sync_action().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_request_sync_action_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_request_sync_action() for the asynchronous version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_request_sync_action_sync(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_set_bitmap_location().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_set_bitmap_location().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_bitmap_location_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_set_bitmap_location() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_bitmap_location_sync(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_start(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_start() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_stop(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_stop().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_stop().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_stop() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_request_sync_action(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_bitmap_location(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_stop(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksMDRaid::handle-add-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-remove-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-request-sync-action signal.
             * @param invocation
             * @param arg_sync_action
             * @param arg_options
             */
            vfunc_handle_request_sync_action(
                invocation: Gio.DBusMethodInvocation,
                arg_sync_action: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-set-bitmap-location signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_bitmap_location(
                invocation: Gio.DBusMethodInvocation,
                arg_value: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-start signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_start(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-stop signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_stop(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerBTRFSProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    ManagerBTRFS.ConstructorProps {}
        }

        /**
         * The #UDisksManagerBTRFSProxy structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerBTRFSProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<ManagerBTRFSProxy>, Gio.DBusInterface, Gio.Initable, ManagerBTRFS
        {
            static $gtype: GObject.GType<ManagerBTRFSProxy>;

            // Constructors

            constructor(properties?: Partial<ManagerBTRFSProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): ManagerBTRFSProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): ManagerBTRFSProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerBTRFSProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerBTRFSProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerBTRFSProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_manager_btrfs_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerBTRFSProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): ManagerBTRFSProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_btrfs_call_create_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_btrfs_call_create_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_volume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_btrfs_call_create_volume() for the asynchronous version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_volume_sync(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_volume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksManagerBTRFS::handle-create-volume signal.
             * @param invocation
             * @param arg_devices
             * @param arg_label
             * @param arg_data_level
             * @param arg_md_level
             * @param arg_options
             */
            vfunc_handle_create_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerBTRFSSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    ManagerBTRFS.ConstructorProps {}
        }

        /**
         * The #UDisksManagerBTRFSSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerBTRFSSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, ManagerBTRFS {
            static $gtype: GObject.GType<ManagerBTRFSSkeleton>;

            // Constructors

            constructor(properties?: Partial<ManagerBTRFSSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): ManagerBTRFSSkeleton;

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_btrfs_call_create_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_btrfs_call_create_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_volume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_btrfs_call_create_volume() for the asynchronous version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_volume_sync(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_volume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksManagerBTRFS::handle-create-volume signal.
             * @param invocation
             * @param arg_devices
             * @param arg_label
             * @param arg_data_level
             * @param arg_md_level
             * @param arg_options
             */
            vfunc_handle_create_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerLVM2Proxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    ManagerLVM2.ConstructorProps {}
        }

        /**
         * The #UDisksManagerLVM2Proxy structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerLVM2Proxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<ManagerLVM2Proxy>, Gio.DBusInterface, Gio.Initable, ManagerLVM2
        {
            static $gtype: GObject.GType<ManagerLVM2Proxy>;

            // Constructors

            constructor(properties?: Partial<ManagerLVM2Proxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): ManagerLVM2Proxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): ManagerLVM2Proxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerLVM2Proxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerLVM2Proxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerLVM2Proxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_manager_lvm2_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerLVM2Proxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): ManagerLVM2Proxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_lvm2_call_volume_group_create().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_lvm2_call_volume_group_create().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_volume_group_create_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_lvm2_call_volume_group_create() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_volume_group_create_sync(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_volume_group_create(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Handler for the #UDisksManagerLVM2::handle-volume-group-create signal.
             * @param invocation
             * @param arg_name
             * @param arg_blocks
             * @param arg_options
             */
            vfunc_handle_volume_group_create(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerLVM2Skeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    ManagerLVM2.ConstructorProps {}
        }

        /**
         * The #UDisksManagerLVM2Skeleton structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerLVM2Skeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, ManagerLVM2 {
            static $gtype: GObject.GType<ManagerLVM2Skeleton>;

            // Constructors

            constructor(properties?: Partial<ManagerLVM2Skeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): ManagerLVM2Skeleton;

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_lvm2_call_volume_group_create().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_lvm2_call_volume_group_create().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_volume_group_create_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_lvm2_call_volume_group_create() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_volume_group_create_sync(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_volume_group_create(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Handler for the #UDisksManagerLVM2::handle-volume-group-create signal.
             * @param invocation
             * @param arg_name
             * @param arg_blocks
             * @param arg_options
             */
            vfunc_handle_volume_group_create(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerNVMeProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    ManagerNVMe.ConstructorProps {}
        }

        /**
         * The #UDisksManagerNVMeProxy structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerNVMeProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<ManagerNVMeProxy>, Gio.DBusInterface, Gio.Initable, ManagerNVMe
        {
            static $gtype: GObject.GType<ManagerNVMeProxy>;

            // Constructors

            constructor(properties?: Partial<ManagerNVMeProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): ManagerNVMeProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): ManagerNVMeProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerNVMeProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerNVMeProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerNVMeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_manager_nvme_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerNVMeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_id(): string;
            set host_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostId(): string;
            set hostId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_nqn(): string;
            set host_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostNqn(): string;
            set hostNqn(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): ManagerNVMeProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_connect().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_connect().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_connect_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_connect() for the asynchronous version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_connect_sync(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_set_host_id().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_set_host_id().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_id_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_set_host_id() for the asynchronous version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_id_sync(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_set_host_nqn().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_set_host_nqn().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_nqn_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_set_host_nqn() for the asynchronous version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_nqn_sync(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param nvme_ctrl Parameter to return.
             */
            complete_connect(invocation: Gio.DBusMethodInvocation, nvme_ctrl: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_host_id(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_host_nqn(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksManagerNVMe::handle-connect signal.
             * @param invocation
             * @param arg_subsysnqn
             * @param arg_transport
             * @param arg_transport_addr
             * @param arg_options
             */
            vfunc_handle_connect(
                invocation: Gio.DBusMethodInvocation,
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManagerNVMe::handle-set-host-id signal.
             * @param invocation
             * @param arg_hostid
             * @param arg_options
             */
            vfunc_handle_set_host_id(
                invocation: Gio.DBusMethodInvocation,
                arg_hostid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManagerNVMe::handle-set-host-nqn signal.
             * @param invocation
             * @param arg_hostnqn
             * @param arg_options
             */
            vfunc_handle_set_host_nqn(
                invocation: Gio.DBusMethodInvocation,
                arg_hostnqn: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerNVMeSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    ManagerNVMe.ConstructorProps {}
        }

        /**
         * The #UDisksManagerNVMeSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerNVMeSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, ManagerNVMe {
            static $gtype: GObject.GType<ManagerNVMeSkeleton>;

            // Constructors

            constructor(properties?: Partial<ManagerNVMeSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): ManagerNVMeSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_id(): string;
            set host_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostId(): string;
            set hostId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_nqn(): string;
            set host_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostNqn(): string;
            set hostNqn(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_connect().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_connect().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_connect_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_connect() for the asynchronous version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_connect_sync(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_set_host_id().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_set_host_id().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_id_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_set_host_id() for the asynchronous version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_id_sync(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_set_host_nqn().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_set_host_nqn().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_nqn_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_set_host_nqn() for the asynchronous version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_nqn_sync(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param nvme_ctrl Parameter to return.
             */
            complete_connect(invocation: Gio.DBusMethodInvocation, nvme_ctrl: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_host_id(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_host_nqn(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksManagerNVMe::handle-connect signal.
             * @param invocation
             * @param arg_subsysnqn
             * @param arg_transport
             * @param arg_transport_addr
             * @param arg_options
             */
            vfunc_handle_connect(
                invocation: Gio.DBusMethodInvocation,
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManagerNVMe::handle-set-host-id signal.
             * @param invocation
             * @param arg_hostid
             * @param arg_options
             */
            vfunc_handle_set_host_id(
                invocation: Gio.DBusMethodInvocation,
                arg_hostid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManagerNVMe::handle-set-host-nqn signal.
             * @param invocation
             * @param arg_hostnqn
             * @param arg_options
             */
            vfunc_handle_set_host_nqn(
                invocation: Gio.DBusMethodInvocation,
                arg_hostnqn: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Manager.ConstructorProps {}
        }

        /**
         * The #UDisksManagerProxy structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<ManagerProxy>, Gio.DBusInterface, Gio.Initable, Manager
        {
            static $gtype: GObject.GType<ManagerProxy>;

            // Constructors

            constructor(properties?: Partial<ManagerProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): ManagerProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): ManagerProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ManagerProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_manager_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_manager_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_manager_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ManagerProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.DefaultEncryptionType">"DefaultEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get default_encryption_type(): string;
            set default_encryption_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.DefaultEncryptionType">"DefaultEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get defaultEncryptionType(): string;
            set defaultEncryptionType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedEncryptionTypes">"SupportedEncryptionTypes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supported_encryption_types(): string[];
            set supported_encryption_types(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedEncryptionTypes">"SupportedEncryptionTypes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supportedEncryptionTypes(): string[];
            set supportedEncryptionTypes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedFilesystems">"SupportedFilesystems"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supported_filesystems(): string[];
            set supported_filesystems(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedFilesystems">"SupportedFilesystems"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supportedFilesystems(): string[];
            set supportedFilesystems(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.Version">"Version"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get version(): string;
            set version(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): ManagerProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_check(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_check(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_check(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_check().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_check().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_check_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_check() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_check_sync(arg_type: string, cancellable?: Gio.Cancellable | null): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_format(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_format(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_format(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_format_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_format() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_format_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_repair(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_repair(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_repair(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_repair_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_repair() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_repair_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_resize(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_resize(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_resize(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_resize_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_resize() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_resize_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_call_enable_module().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_enable_module().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_module_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_enable_module() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_module_sync(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_modules(arg_enable: boolean, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_modules(
                arg_enable: boolean,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_modules(
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_call_enable_modules().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_enable_modules().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_modules_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_enable_modules() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_modules_sync(arg_enable: boolean, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string[] | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string[] | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_get_block_devices().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_get_block_devices().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_block_devices_finish(res: Gio.AsyncResult): [boolean, string[] | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_get_block_devices() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_block_devices_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string[] | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[string, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[string, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_manager_call_loop_setup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_loop_setup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_loop_setup_finish(res: Gio.AsyncResult): [boolean, string, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_loop_setup() for the asynchronous version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_loop_setup_sync(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_call_mdraid_create().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_mdraid_create().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mdraid_create_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_mdraid_create() for the asynchronous version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mdraid_create_sync(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string[] | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string[] | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_resolve_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_resolve_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resolve_device_finish(res: Gio.AsyncResult): [boolean, string[] | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_resolve_device() for the asynchronous version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resolve_device_sync(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string[] | null];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_check(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_format(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_repair(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_resize(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_module(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_modules(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param block_objects Parameter to return.
             */
            complete_get_block_devices(invocation: Gio.DBusMethodInvocation, block_objects: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param resulting_device Parameter to return.
             */
            complete_loop_setup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                resulting_device: string,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param resulting_array Parameter to return.
             */
            complete_mdraid_create(invocation: Gio.DBusMethodInvocation, resulting_array: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param devices Parameter to return.
             */
            complete_resolve_device(invocation: Gio.DBusMethodInvocation, devices: string): void;
            /**
             * Handler for the #UDisksManager::handle-can-check signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_check(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-format signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_format(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-repair signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_repair(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-resize signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_resize(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-enable-module signal.
             * @param invocation
             * @param arg_name
             * @param arg_enable
             */
            vfunc_handle_enable_module(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_enable: boolean,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-enable-modules signal.
             * @param invocation
             * @param arg_enable
             */
            vfunc_handle_enable_modules(invocation: Gio.DBusMethodInvocation, arg_enable: boolean): boolean;
            /**
             * Handler for the #UDisksManager::handle-get-block-devices signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_block_devices(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksManager::handle-loop-setup signal.
             * @param invocation
             * @param fd_list
             * @param arg_fd
             * @param arg_options
             */
            vfunc_handle_loop_setup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-mdraid-create signal.
             * @param invocation
             * @param arg_blocks
             * @param arg_level
             * @param arg_name
             * @param arg_chunk
             * @param arg_options
             */
            vfunc_handle_mdraid_create(
                invocation: Gio.DBusMethodInvocation,
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-resolve-device signal.
             * @param invocation
             * @param arg_devspec
             * @param arg_options
             */
            vfunc_handle_resolve_device(
                invocation: Gio.DBusMethodInvocation,
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ManagerSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Manager.ConstructorProps {}
        }

        /**
         * The #UDisksManagerSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class ManagerSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Manager {
            static $gtype: GObject.GType<ManagerSkeleton>;

            // Constructors

            constructor(properties?: Partial<ManagerSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): ManagerSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.DefaultEncryptionType">"DefaultEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get default_encryption_type(): string;
            set default_encryption_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.DefaultEncryptionType">"DefaultEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get defaultEncryptionType(): string;
            set defaultEncryptionType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedEncryptionTypes">"SupportedEncryptionTypes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supported_encryption_types(): string[];
            set supported_encryption_types(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedEncryptionTypes">"SupportedEncryptionTypes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supportedEncryptionTypes(): string[];
            set supportedEncryptionTypes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedFilesystems">"SupportedFilesystems"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supported_filesystems(): string[];
            set supported_filesystems(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedFilesystems">"SupportedFilesystems"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supportedFilesystems(): string[];
            set supportedFilesystems(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.Version">"Version"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get version(): string;
            set version(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_check(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_check(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_check(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_check().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_check().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_check_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_check() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_check_sync(arg_type: string, cancellable?: Gio.Cancellable | null): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_format(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_format(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_format(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_format_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_format() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_format_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_repair(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_repair(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_repair(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_repair_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_repair() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_repair_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_resize(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_resize(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_resize(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_resize_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_resize() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_resize_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_call_enable_module().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_enable_module().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_module_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_enable_module() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_module_sync(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_modules(arg_enable: boolean, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_modules(
                arg_enable: boolean,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_modules(
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_call_enable_modules().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_enable_modules().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_modules_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_enable_modules() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_modules_sync(arg_enable: boolean, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string[] | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string[] | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_get_block_devices().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_get_block_devices().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_block_devices_finish(res: Gio.AsyncResult): [boolean, string[] | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_get_block_devices() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_block_devices_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string[] | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[string, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[string, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_manager_call_loop_setup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_loop_setup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_loop_setup_finish(res: Gio.AsyncResult): [boolean, string, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_loop_setup() for the asynchronous version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_loop_setup_sync(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_call_mdraid_create().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_mdraid_create().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mdraid_create_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_mdraid_create() for the asynchronous version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mdraid_create_sync(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string[] | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string[] | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_resolve_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_resolve_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resolve_device_finish(res: Gio.AsyncResult): [boolean, string[] | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_resolve_device() for the asynchronous version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resolve_device_sync(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string[] | null];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_check(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_format(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_repair(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_resize(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_module(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_modules(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param block_objects Parameter to return.
             */
            complete_get_block_devices(invocation: Gio.DBusMethodInvocation, block_objects: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param resulting_device Parameter to return.
             */
            complete_loop_setup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                resulting_device: string,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param resulting_array Parameter to return.
             */
            complete_mdraid_create(invocation: Gio.DBusMethodInvocation, resulting_array: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param devices Parameter to return.
             */
            complete_resolve_device(invocation: Gio.DBusMethodInvocation, devices: string): void;
            /**
             * Handler for the #UDisksManager::handle-can-check signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_check(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-format signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_format(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-repair signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_repair(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-resize signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_resize(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-enable-module signal.
             * @param invocation
             * @param arg_name
             * @param arg_enable
             */
            vfunc_handle_enable_module(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_enable: boolean,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-enable-modules signal.
             * @param invocation
             * @param arg_enable
             */
            vfunc_handle_enable_modules(invocation: Gio.DBusMethodInvocation, arg_enable: boolean): boolean;
            /**
             * Handler for the #UDisksManager::handle-get-block-devices signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_block_devices(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksManager::handle-loop-setup signal.
             * @param invocation
             * @param fd_list
             * @param arg_fd
             * @param arg_options
             */
            vfunc_handle_loop_setup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-mdraid-create signal.
             * @param invocation
             * @param arg_blocks
             * @param arg_level
             * @param arg_name
             * @param arg_chunk
             * @param arg_options
             */
            vfunc_handle_mdraid_create(
                invocation: Gio.DBusMethodInvocation,
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-resolve-device signal.
             * @param invocation
             * @param arg_devspec
             * @param arg_options
             */
            vfunc_handle_resolve_device(
                invocation: Gio.DBusMethodInvocation,
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace NVMeControllerProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    NVMeController.ConstructorProps {}
        }

        /**
         * The #UDisksNVMeControllerProxy structure contains only private data and should only be accessed using the provided API.
         */
        class NVMeControllerProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<NVMeControllerProxy>, Gio.DBusInterface, Gio.Initable, NVMeController
        {
            static $gtype: GObject.GType<NVMeControllerProxy>;

            // Constructors

            constructor(properties?: Partial<NVMeControllerProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): NVMeControllerProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): NVMeControllerProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): NVMeControllerProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): NVMeControllerProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<NVMeControllerProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_nvme_controller_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<NVMeControllerProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.ControllerID">"ControllerID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get controller_id(): number;
            set controller_id(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.ControllerID">"ControllerID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get controllerId(): number;
            set controllerId(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.FGUID">"FGUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get fguid(): string;
            set fguid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.NVMeRevision">"NVMeRevision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nvme_revision(): string;
            set nvme_revision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.NVMeRevision">"NVMeRevision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nvmeRevision(): string;
            set nvmeRevision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizePercentRemaining">"SanitizePercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitize_percent_remaining(): number;
            set sanitize_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizePercentRemaining">"SanitizePercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitizePercentRemaining(): number;
            set sanitizePercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStatus">"SanitizeStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitize_status(): string;
            set sanitize_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStatus">"SanitizeStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitizeStatus(): string;
            set sanitizeStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartCriticalWarning">"SmartCriticalWarning"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_critical_warning(): string[];
            set smart_critical_warning(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartCriticalWarning">"SmartCriticalWarning"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartCriticalWarning(): string[];
            set smartCriticalWarning(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartPowerOnHours">"SmartPowerOnHours"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_power_on_hours(): number;
            set smart_power_on_hours(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartPowerOnHours">"SmartPowerOnHours"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartPowerOnHours(): number;
            set smartPowerOnHours(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_percent_remaining(): number;
            set smart_selftest_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestPercentRemaining(): number;
            set smartSelftestPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_status(): string;
            set smart_selftest_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestStatus(): string;
            set smartSelftestStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_temperature(): number;
            set smart_temperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartTemperature(): number;
            set smartTemperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_updated(): number;
            set smart_updated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartUpdated(): number;
            set smartUpdated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.State">"State"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get state(): string;
            set state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SubsystemNQN">"SubsystemNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get subsystem_nqn(): string;
            set subsystem_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SubsystemNQN">"SubsystemNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get subsystemNqn(): string;
            set subsystemNqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.UnallocatedCapacity">"UnallocatedCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get unallocated_capacity(): number;
            set unallocated_capacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.UnallocatedCapacity">"UnallocatedCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get unallocatedCapacity(): number;
            set unallocatedCapacity(val: number);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): NVMeControllerProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_sanitize_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_sanitize_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_sanitize_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_sanitize_start() for the asynchronous version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_sanitize_start_sync(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_get_attributes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_get_attributes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_get_attributes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_selftest_abort().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_selftest_abort().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_selftest_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_selftest_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_selftest_start() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_update(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_update().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_update().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_update() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_sanitize_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param attributes Parameter to return.
             */
            complete_smart_get_attributes(invocation: Gio.DBusMethodInvocation, attributes: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_abort(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_update(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksNVMeController::handle-sanitize-start signal.
             * @param invocation
             * @param arg_action
             * @param arg_options
             */
            vfunc_handle_sanitize_start(
                invocation: Gio.DBusMethodInvocation,
                arg_action: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-get-attributes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_get_attributes(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-selftest-abort signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_selftest_abort(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-selftest-start signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_smart_selftest_start(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-update signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_update(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace NVMeControllerSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    NVMeController.ConstructorProps {}
        }

        /**
         * The #UDisksNVMeControllerSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class NVMeControllerSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, NVMeController {
            static $gtype: GObject.GType<NVMeControllerSkeleton>;

            // Constructors

            constructor(properties?: Partial<NVMeControllerSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): NVMeControllerSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.ControllerID">"ControllerID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get controller_id(): number;
            set controller_id(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.ControllerID">"ControllerID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get controllerId(): number;
            set controllerId(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.FGUID">"FGUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get fguid(): string;
            set fguid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.NVMeRevision">"NVMeRevision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nvme_revision(): string;
            set nvme_revision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.NVMeRevision">"NVMeRevision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nvmeRevision(): string;
            set nvmeRevision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizePercentRemaining">"SanitizePercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitize_percent_remaining(): number;
            set sanitize_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizePercentRemaining">"SanitizePercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitizePercentRemaining(): number;
            set sanitizePercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStatus">"SanitizeStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitize_status(): string;
            set sanitize_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStatus">"SanitizeStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitizeStatus(): string;
            set sanitizeStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartCriticalWarning">"SmartCriticalWarning"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_critical_warning(): string[];
            set smart_critical_warning(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartCriticalWarning">"SmartCriticalWarning"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartCriticalWarning(): string[];
            set smartCriticalWarning(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartPowerOnHours">"SmartPowerOnHours"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_power_on_hours(): number;
            set smart_power_on_hours(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartPowerOnHours">"SmartPowerOnHours"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartPowerOnHours(): number;
            set smartPowerOnHours(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_percent_remaining(): number;
            set smart_selftest_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestPercentRemaining(): number;
            set smartSelftestPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_status(): string;
            set smart_selftest_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestStatus(): string;
            set smartSelftestStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_temperature(): number;
            set smart_temperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartTemperature(): number;
            set smartTemperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_updated(): number;
            set smart_updated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartUpdated(): number;
            set smartUpdated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.State">"State"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get state(): string;
            set state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SubsystemNQN">"SubsystemNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get subsystem_nqn(): string;
            set subsystem_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SubsystemNQN">"SubsystemNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get subsystemNqn(): string;
            set subsystemNqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.UnallocatedCapacity">"UnallocatedCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get unallocated_capacity(): number;
            set unallocated_capacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.UnallocatedCapacity">"UnallocatedCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get unallocatedCapacity(): number;
            set unallocatedCapacity(val: number);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_sanitize_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_sanitize_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_sanitize_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_sanitize_start() for the asynchronous version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_sanitize_start_sync(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_get_attributes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_get_attributes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_get_attributes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_selftest_abort().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_selftest_abort().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_selftest_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_selftest_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_selftest_start() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_update(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_update().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_update().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_update() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_sanitize_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param attributes Parameter to return.
             */
            complete_smart_get_attributes(invocation: Gio.DBusMethodInvocation, attributes: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_abort(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_update(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksNVMeController::handle-sanitize-start signal.
             * @param invocation
             * @param arg_action
             * @param arg_options
             */
            vfunc_handle_sanitize_start(
                invocation: Gio.DBusMethodInvocation,
                arg_action: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-get-attributes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_get_attributes(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-selftest-abort signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_selftest_abort(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-selftest-start signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_smart_selftest_start(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-update signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_update(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace NVMeFabricsProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    NVMeFabrics.ConstructorProps {}
        }

        /**
         * The #UDisksNVMeFabricsProxy structure contains only private data and should only be accessed using the provided API.
         */
        class NVMeFabricsProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<NVMeFabricsProxy>, Gio.DBusInterface, Gio.Initable, NVMeFabrics
        {
            static $gtype: GObject.GType<NVMeFabricsProxy>;

            // Constructors

            constructor(properties?: Partial<NVMeFabricsProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): NVMeFabricsProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): NVMeFabricsProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): NVMeFabricsProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): NVMeFabricsProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<NVMeFabricsProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_nvme_fabrics_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<NVMeFabricsProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_id(): string;
            set host_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostId(): string;
            set hostId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_nqn(): string;
            set host_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostNqn(): string;
            set hostNqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.Transport">"Transport"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transport(): string;
            set transport(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.TransportAddress">"TransportAddress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transport_address(): string;
            set transport_address(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.TransportAddress">"TransportAddress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transportAddress(): string;
            set transportAddress(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): NVMeFabricsProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_disconnect(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_disconnect(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_disconnect(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_fabrics_call_disconnect().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_fabrics_call_disconnect().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_disconnect_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_fabrics_call_disconnect() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_disconnect_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_disconnect(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksNVMeFabrics::handle-disconnect signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_disconnect(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace NVMeFabricsSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    NVMeFabrics.ConstructorProps {}
        }

        /**
         * The #UDisksNVMeFabricsSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class NVMeFabricsSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, NVMeFabrics {
            static $gtype: GObject.GType<NVMeFabricsSkeleton>;

            // Constructors

            constructor(properties?: Partial<NVMeFabricsSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): NVMeFabricsSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_id(): string;
            set host_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostId(): string;
            set hostId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_nqn(): string;
            set host_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostNqn(): string;
            set hostNqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.Transport">"Transport"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transport(): string;
            set transport(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.TransportAddress">"TransportAddress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transport_address(): string;
            set transport_address(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.TransportAddress">"TransportAddress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transportAddress(): string;
            set transportAddress(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_disconnect(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_disconnect(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_disconnect(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_fabrics_call_disconnect().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_fabrics_call_disconnect().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_disconnect_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_fabrics_call_disconnect() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_disconnect_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_disconnect(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksNVMeFabrics::handle-disconnect signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_disconnect(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace NVMeNamespaceProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    NVMeNamespace.ConstructorProps {}
        }

        /**
         * The #UDisksNVMeNamespaceProxy structure contains only private data and should only be accessed using the provided API.
         */
        class NVMeNamespaceProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<NVMeNamespaceProxy>, Gio.DBusInterface, Gio.Initable, NVMeNamespace
        {
            static $gtype: GObject.GType<NVMeNamespaceProxy>;

            // Constructors

            constructor(properties?: Partial<NVMeNamespaceProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): NVMeNamespaceProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): NVMeNamespaceProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): NVMeNamespaceProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): NVMeNamespaceProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<NVMeNamespaceProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_nvme_namespace_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<NVMeNamespaceProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.EUI64">"EUI64"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get eui64(): string;
            set eui64(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormatPercentRemaining">"FormatPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get format_percent_remaining(): number;
            set format_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormatPercentRemaining">"FormatPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formatPercentRemaining(): number;
            set formatPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormattedLBASize">"FormattedLBASize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formatted_lbasize(): GLib.Variant;
            set formatted_lbasize(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormattedLBASize">"FormattedLBASize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formattedLbasize(): GLib.Variant;
            set formattedLbasize(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.LBAFormats">"LBAFormats"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get lbaformats(): GLib.Variant;
            set lbaformats(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceCapacity">"NamespaceCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_capacity(): number;
            set namespace_capacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceCapacity">"NamespaceCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceCapacity(): number;
            set namespaceCapacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceSize">"NamespaceSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_size(): number;
            set namespace_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceSize">"NamespaceSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceSize(): number;
            set namespaceSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceUtilization">"NamespaceUtilization"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_utilization(): number;
            set namespace_utilization(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceUtilization">"NamespaceUtilization"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceUtilization(): number;
            set namespaceUtilization(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NGUID">"NGUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nguid(): string;
            set nguid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NSID">"NSID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nsid(): number;
            set nsid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.WWN">"WWN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get wwn(): string;
            set wwn(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): NVMeNamespaceProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_format_namespace(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format_namespace(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format_namespace(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_namespace_call_format_namespace().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_namespace_call_format_namespace().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_namespace_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_namespace_call_format_namespace() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_namespace_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_format_namespace(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksNVMeNamespace::handle-format-namespace signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_format_namespace(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace NVMeNamespaceSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    NVMeNamespace.ConstructorProps {}
        }

        /**
         * The #UDisksNVMeNamespaceSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class NVMeNamespaceSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, NVMeNamespace {
            static $gtype: GObject.GType<NVMeNamespaceSkeleton>;

            // Constructors

            constructor(properties?: Partial<NVMeNamespaceSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): NVMeNamespaceSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.EUI64">"EUI64"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get eui64(): string;
            set eui64(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormatPercentRemaining">"FormatPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get format_percent_remaining(): number;
            set format_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormatPercentRemaining">"FormatPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formatPercentRemaining(): number;
            set formatPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormattedLBASize">"FormattedLBASize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formatted_lbasize(): GLib.Variant;
            set formatted_lbasize(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormattedLBASize">"FormattedLBASize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formattedLbasize(): GLib.Variant;
            set formattedLbasize(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.LBAFormats">"LBAFormats"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get lbaformats(): GLib.Variant;
            set lbaformats(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceCapacity">"NamespaceCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_capacity(): number;
            set namespace_capacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceCapacity">"NamespaceCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceCapacity(): number;
            set namespaceCapacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceSize">"NamespaceSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_size(): number;
            set namespace_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceSize">"NamespaceSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceSize(): number;
            set namespaceSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceUtilization">"NamespaceUtilization"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_utilization(): number;
            set namespace_utilization(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceUtilization">"NamespaceUtilization"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceUtilization(): number;
            set namespaceUtilization(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NGUID">"NGUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nguid(): string;
            set nguid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NSID">"NSID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nsid(): number;
            set nsid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.WWN">"WWN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get wwn(): string;
            set wwn(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_format_namespace(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format_namespace(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format_namespace(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_namespace_call_format_namespace().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_namespace_call_format_namespace().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_namespace_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_namespace_call_format_namespace() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_namespace_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_format_namespace(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksNVMeNamespace::handle-format-namespace signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_format_namespace(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ObjectInfo {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {}
        }

        /**
         * Detailed information about the D-Bus interfaces (such as
         * #UDisksBlock and #UDisksDrive) on a #UDisksObject that is suitable
         * to display in an user interface. Use
         * udisks_client_get_object_info() to get #UDisksObjectInfo objects.
         * Note that #UDisksObjectInfo is an immutable object; once it has
         * been created it cannot be modified further.
         *
         * The <link
         * linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">HintName</link>
         * and/or <link
         * linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">HintIconName</link>
         * propreties on associated #UDisksBlock interfaces (if any) may
         * influence what udisks_object_info_get_icon() and
         * udisks_object_info_get_media_icon() returns.
         *
         * The value return by udisks_object_info_get_one_liner() is designed
         * to contain enough information such that it is all that needs to be
         * shown about the object. As a result for e.g.  block devices or
         * drives it contains the special device file
         * e.g. <filename>/dev/sda</filename>.
         */
        class ObjectInfo extends GObject.Object {
            static $gtype: GObject.GType<ObjectInfo>;

            // Constructors

            constructor(properties?: Partial<ObjectInfo.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            // Methods

            /**
             * Gets the description.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_description(): string;
            /**
             * Gets the icon.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_icon(): Gio.Icon;
            /**
             * Gets the symbolic icon.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_icon_symbolic(): Gio.Icon;
            /**
             * Gets the media description.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_media_description(): string;
            /**
             * Gets the media icon.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_media_icon(): Gio.Icon;
            /**
             * Gets the symbolic media icon.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_media_icon_symbolic(): Gio.Icon;
            /**
             * Gets the name.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_name(): string;
            /**
             * Gets the #UDisksObject that `info` is for
             * @returns The object - do not free or unref, the reference belongs to @info.
             */
            get_object(): Object;
            /**
             * Gets a one-line description.
             * @returns The value or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_one_liner(): string;
            /**
             * Gets the sort-key for `info`. This can be used with g_strcmp0() to
             * sort objects.
             * @returns The sort key or %NULL. Do not free or unref, the value belongs to @info.
             */
            get_sort_key(): string;
        }

        namespace ObjectManagerClient {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusObjectManagerClient.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusObjectManager.ConstructorProps,
                    Gio.Initable.ConstructorProps {}
        }

        /**
         * The #UDisksObjectManagerClient structure contains only private data and should only be accessed using the provided API.
         */
        class ObjectManagerClient
            extends Gio.DBusObjectManagerClient
            implements Gio.AsyncInitable<ObjectManagerClient>, Gio.DBusObjectManager, Gio.Initable
        {
            static $gtype: GObject.GType<ObjectManagerClient>;

            // Constructors

            constructor(properties?: Partial<ObjectManagerClient.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): ObjectManagerClient;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): ObjectManagerClient;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusObjectManagerClientFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ObjectManagerClient;
            // Conflicted with Gio.DBusObjectManagerClient.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusObjectManagerClientFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): ObjectManagerClient;
            // Conflicted with Gio.DBusObjectManagerClient.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * A #GDBusProxyTypeFunc that maps `interface_name` to the generated #GDBusObjectProxy derived and #GDBusProxy derived types.
             * @param manager A #GDBusObjectManagerClient.
             * @param object_path The object path of the remote object (unused).
             * @param interface_name Interface name of the remote object or %NULL to get the object proxy #GType.
             * @param user_data User data (unused).
             */
            static get_proxy_type(
                manager: Gio.DBusObjectManagerClient,
                object_path: string,
                interface_name?: string | null,
                user_data?: any | null,
            ): GObject.GType;
            /**
             * Asynchronously creates #GDBusObjectManagerClient using udisks_object_manager_client_get_proxy_type() as the #GDBusProxyTypeFunc. See g_dbus_object_manager_client_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_object_manager_client_new_finish() to get the result of the operation.
             *
             * See udisks_object_manager_client_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusObjectManagerClientFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusObjectManagerClientFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ObjectManagerClient> | null,
            ): void;
            // Conflicted with Gio.DBusObjectManagerClient.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_object_manager_client_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_object_manager_client_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_object_manager_client_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusObjectManagerClientFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusObjectManagerClientFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<ObjectManagerClient> | null,
            ): void;
            // Conflicted with Gio.DBusObjectManagerClient.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): ObjectManagerClient;
            // Conflicted with Gio.DBusObjectManagerClient.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the interface proxy for `interface_name` at `object_path,` if
             * any.
             * @param object_path Object path to look up.
             * @param interface_name D-Bus interface name to look up.
             * @returns A #GDBusInterface instance or %NULL. Free   with g_object_unref().
             */
            get_interface(object_path: string, interface_name: string): Gio.DBusInterface | null;
            /**
             * Gets the #GDBusObject at `object_path,` if any.
             * @param object_path Object path to look up.
             * @returns A #GDBusObject or %NULL. Free with   g_object_unref().
             */
            get_object(object_path: string): Gio.DBusObject | null;
            /**
             * Gets the object path that `manager` is for.
             * @returns A string owned by @manager. Do not free.
             */
            get_object_path(): string;
            /**
             * Gets all #GDBusObject objects known to `manager`.
             * @returns A list of   #GDBusObject objects. The returned list should be freed with   g_list_free() after each element has been freed with   g_object_unref().
             */
            get_objects(): Gio.DBusObject[];
            /**
             * Gets the interface proxy for `interface_name` at `object_path,` if
             * any.
             * @param object_path Object path to look up.
             * @param interface_name D-Bus interface name to look up.
             */
            vfunc_get_interface(object_path: string, interface_name: string): Gio.DBusInterface | null;
            /**
             * Gets the #GDBusObject at `object_path,` if any.
             * @param object_path Object path to look up.
             */
            vfunc_get_object(object_path: string): Gio.DBusObject | null;
            /**
             * Gets the object path that `manager` is for.
             */
            vfunc_get_object_path(): string;
            /**
             * Gets all #GDBusObject objects known to `manager`.
             */
            vfunc_get_objects(): Gio.DBusObject[];
            /**
             * Signal handler for the #GDBusObjectManager::interface-added signal.
             * @param object
             * @param interface_
             */
            vfunc_interface_added(object: Gio.DBusObject, interface_: Gio.DBusInterface): void;
            /**
             * Signal handler for the #GDBusObjectManager::interface-removed signal.
             * @param object
             * @param interface_
             */
            vfunc_interface_removed(object: Gio.DBusObject, interface_: Gio.DBusInterface): void;
            /**
             * Signal handler for the #GDBusObjectManager::object-added signal.
             * @param object
             */
            vfunc_object_added(object: Gio.DBusObject): void;
            /**
             * Signal handler for the #GDBusObjectManager::object-removed signal.
             * @param object
             */
            vfunc_object_removed(object: Gio.DBusObject): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ObjectProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusObjectProxy.ConstructorProps,
                    Gio.DBusObject.ConstructorProps,
                    Object.ConstructorProps {}
        }

        /**
         * The #UDisksObjectProxy structure contains only private data and should only be accessed using the provided API.
         */
        class ObjectProxy extends Gio.DBusObjectProxy implements Gio.DBusObject, Object {
            static $gtype: GObject.GType<ObjectProxy>;

            // Constructors

            constructor(properties?: Partial<ObjectProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](connection: Gio.DBusConnection, object_path: string): ObjectProxy;

            // Inherited properties
            /**
             * The #UDisksBlock instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get block(): Block;
            set block(val: Block);
            /**
             * The #UDisksBlockLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get block_lvm2(): BlockLVM2;
            set block_lvm2(val: BlockLVM2);
            /**
             * The #UDisksBlockLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get blockLvm2(): BlockLVM2;
            set blockLvm2(val: BlockLVM2);
            /**
             * The #UDisksDrive instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get drive(): Drive;
            set drive(val: Drive);
            /**
             * The #UDisksDriveAta instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get drive_ata(): DriveAta;
            set drive_ata(val: DriveAta);
            /**
             * The #UDisksDriveAta instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get driveAta(): DriveAta;
            set driveAta(val: DriveAta);
            /**
             * The #UDisksEncrypted instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get encrypted(): Encrypted;
            set encrypted(val: Encrypted);
            /**
             * The #UDisksFilesystem instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystem(): Filesystem;
            set filesystem(val: Filesystem);
            /**
             * The #UDisksFilesystemBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystem_btrfs(): FilesystemBTRFS;
            set filesystem_btrfs(val: FilesystemBTRFS);
            /**
             * The #UDisksFilesystemBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystemBtrfs(): FilesystemBTRFS;
            set filesystemBtrfs(val: FilesystemBTRFS);
            /**
             * The #UDisksJob instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get job(): Job;
            set job(val: Job);
            /**
             * The #UDisksLogicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get logical_volume(): LogicalVolume;
            set logical_volume(val: LogicalVolume);
            /**
             * The #UDisksLogicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get logicalVolume(): LogicalVolume;
            set logicalVolume(val: LogicalVolume);
            /**
             * The #UDisksLoop instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get loop(): Loop;
            set loop(val: Loop);
            /**
             * The #UDisksManager instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager(): Manager;
            set manager(val: Manager);
            /**
             * The #UDisksManagerBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_btrfs(): ManagerBTRFS;
            set manager_btrfs(val: ManagerBTRFS);
            /**
             * The #UDisksManagerBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerBtrfs(): ManagerBTRFS;
            set managerBtrfs(val: ManagerBTRFS);
            /**
             * The #UDisksManagerLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_lvm2(): ManagerLVM2;
            set manager_lvm2(val: ManagerLVM2);
            /**
             * The #UDisksManagerLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerLvm2(): ManagerLVM2;
            set managerLvm2(val: ManagerLVM2);
            /**
             * The #UDisksManagerNVMe instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_nvme(): ManagerNVMe;
            set manager_nvme(val: ManagerNVMe);
            /**
             * The #UDisksManagerNVMe instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerNvme(): ManagerNVMe;
            set managerNvme(val: ManagerNVMe);
            /**
             * The #UDisksMDRaid instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get mdraid(): MDRaid;
            set mdraid(val: MDRaid);
            /**
             * The #UDisksNVMeController instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_controller(): NVMeController;
            set nvme_controller(val: NVMeController);
            /**
             * The #UDisksNVMeController instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeController(): NVMeController;
            set nvmeController(val: NVMeController);
            /**
             * The #UDisksNVMeFabrics instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_fabrics(): NVMeFabrics;
            set nvme_fabrics(val: NVMeFabrics);
            /**
             * The #UDisksNVMeFabrics instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeFabrics(): NVMeFabrics;
            set nvmeFabrics(val: NVMeFabrics);
            /**
             * The #UDisksNVMeNamespace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_namespace(): NVMeNamespace;
            set nvme_namespace(val: NVMeNamespace);
            /**
             * The #UDisksNVMeNamespace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeNamespace(): NVMeNamespace;
            set nvmeNamespace(val: NVMeNamespace);
            /**
             * The #UDisksPartition instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partition(): Partition;
            set partition(val: Partition);
            /**
             * The #UDisksPartitionTable instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partition_table(): PartitionTable;
            set partition_table(val: PartitionTable);
            /**
             * The #UDisksPartitionTable instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partitionTable(): PartitionTable;
            set partitionTable(val: PartitionTable);
            /**
             * The #UDisksPhysicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get physical_volume(): PhysicalVolume;
            set physical_volume(val: PhysicalVolume);
            /**
             * The #UDisksPhysicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get physicalVolume(): PhysicalVolume;
            set physicalVolume(val: PhysicalVolume);
            /**
             * The #UDisksSwapspace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get swapspace(): Swapspace;
            set swapspace(val: Swapspace);
            /**
             * The #UDisksVDOVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get vdo_volume(): VDOVolume;
            set vdo_volume(val: VDOVolume);
            /**
             * The #UDisksVDOVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get vdoVolume(): VDOVolume;
            set vdoVolume(val: VDOVolume);
            /**
             * The #UDisksVolumeGroup instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get volume_group(): VolumeGroup;
            set volume_group(val: VolumeGroup);
            /**
             * The #UDisksVolumeGroup instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get volumeGroup(): VolumeGroup;
            set volumeGroup(val: VolumeGroup);

            // Inherited methods
            /**
             * Gets the D-Bus interface with name `interface_name` associated with
             * `object,` if any.
             * @param interface_name A D-Bus interface name.
             * @returns %NULL if not found, otherwise a   #GDBusInterface that must be freed with g_object_unref().
             */
            get_interface(interface_name: string): Gio.DBusInterface | null;
            /**
             * Gets the D-Bus interfaces associated with `object`.
             * @returns A list of #GDBusInterface instances.   The returned list must be freed by g_list_free() after each element has been freed   with g_object_unref().
             */
            get_interfaces(): Gio.DBusInterface[];
            /**
             * Gets the object path for `object`.
             * @returns A string owned by @object. Do not free.
             */
            get_object_path(): string;
            /**
             * Gets the D-Bus interface with name `interface_name` associated with
             * `object,` if any.
             * @param interface_name A D-Bus interface name.
             */
            vfunc_get_interface(interface_name: string): Gio.DBusInterface | null;
            /**
             * Gets the D-Bus interfaces associated with `object`.
             */
            vfunc_get_interfaces(): Gio.DBusInterface[];
            /**
             * Gets the object path for `object`.
             */
            vfunc_get_object_path(): string;
            /**
             * Signal handler for the #GDBusObject::interface-added signal.
             * @param interface_
             */
            vfunc_interface_added(interface_: Gio.DBusInterface): void;
            /**
             * Signal handler for the #GDBusObject::interface-removed signal.
             * @param interface_
             */
            vfunc_interface_removed(interface_: Gio.DBusInterface): void;
            /**
             * Gets the #UDisksBlock instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link> on `object,` if any.
             * @returns A #UDisksBlock that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_block(): Block | null;
            /**
             * Gets the #UDisksBlockLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link> on `object,` if any.
             * @returns A #UDisksBlockLVM2 that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_block_lvm2(): BlockLVM2 | null;
            /**
             * Gets the #UDisksDrive instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link> on `object,` if any.
             * @returns A #UDisksDrive that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_drive(): Drive | null;
            /**
             * Gets the #UDisksDriveAta instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link> on `object,` if any.
             * @returns A #UDisksDriveAta that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_drive_ata(): DriveAta | null;
            /**
             * Gets the #UDisksEncrypted instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link> on `object,` if any.
             * @returns A #UDisksEncrypted that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_encrypted(): Encrypted | null;
            /**
             * Gets the #UDisksFilesystem instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link> on `object,` if any.
             * @returns A #UDisksFilesystem that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_filesystem(): Filesystem | null;
            /**
             * Gets the #UDisksFilesystemBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link> on `object,` if any.
             * @returns A #UDisksFilesystemBTRFS that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_filesystem_btrfs(): FilesystemBTRFS | null;
            /**
             * Gets the #UDisksJob instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link> on `object,` if any.
             * @returns A #UDisksJob that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_job(): Job | null;
            /**
             * Gets the #UDisksLogicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link> on `object,` if any.
             * @returns A #UDisksLogicalVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_logical_volume(): LogicalVolume | null;
            /**
             * Gets the #UDisksLoop instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link> on `object,` if any.
             * @returns A #UDisksLoop that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_loop(): Loop | null;
            /**
             * Gets the #UDisksManager instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link> on `object,` if any.
             * @returns A #UDisksManager that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager(): Manager | null;
            /**
             * Gets the #UDisksManagerBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link> on `object,` if any.
             * @returns A #UDisksManagerBTRFS that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_btrfs(): ManagerBTRFS | null;
            /**
             * Gets the #UDisksManagerLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link> on `object,` if any.
             * @returns A #UDisksManagerLVM2 that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_lvm2(): ManagerLVM2 | null;
            /**
             * Gets the #UDisksManagerNVMe instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link> on `object,` if any.
             * @returns A #UDisksManagerNVMe that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_nvme(): ManagerNVMe | null;
            /**
             * Gets the #UDisksMDRaid instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link> on `object,` if any.
             * @returns A #UDisksMDRaid that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_mdraid(): MDRaid | null;
            /**
             * Gets the #UDisksNVMeController instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link> on `object,` if any.
             * @returns A #UDisksNVMeController that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_controller(): NVMeController | null;
            /**
             * Gets the #UDisksNVMeFabrics instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link> on `object,` if any.
             * @returns A #UDisksNVMeFabrics that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_fabrics(): NVMeFabrics | null;
            /**
             * Gets the #UDisksNVMeNamespace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link> on `object,` if any.
             * @returns A #UDisksNVMeNamespace that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_namespace(): NVMeNamespace | null;
            /**
             * Gets the #UDisksPartition instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link> on `object,` if any.
             * @returns A #UDisksPartition that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_partition(): Partition | null;
            /**
             * Gets the #UDisksPartitionTable instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link> on `object,` if any.
             * @returns A #UDisksPartitionTable that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_partition_table(): PartitionTable | null;
            /**
             * Gets the #UDisksPhysicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link> on `object,` if any.
             * @returns A #UDisksPhysicalVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_physical_volume(): PhysicalVolume | null;
            /**
             * Gets the #UDisksSwapspace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link> on `object,` if any.
             * @returns A #UDisksSwapspace that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_swapspace(): Swapspace | null;
            /**
             * Gets the #UDisksVDOVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link> on `object,` if any.
             * @returns A #UDisksVDOVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_vdo_volume(): VDOVolume | null;
            /**
             * Gets the #UDisksVolumeGroup instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link> on `object,` if any.
             * @returns A #UDisksVolumeGroup that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_volume_group(): VolumeGroup | null;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace ObjectSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusObjectSkeleton.ConstructorProps,
                    Gio.DBusObject.ConstructorProps,
                    Object.ConstructorProps {}
        }

        /**
         * The #UDisksObjectSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class ObjectSkeleton extends Gio.DBusObjectSkeleton implements Gio.DBusObject, Object {
            static $gtype: GObject.GType<ObjectSkeleton>;

            // Constructors

            constructor(properties?: Partial<ObjectSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](object_path: string): ObjectSkeleton;

            // Methods

            /**
             * Sets the #UDisksBlock instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link> on `object`.
             * @param interface_ A #UDisksBlock or %NULL to clear the interface.
             */
            set_block(interface_?: Block | null): void;
            /**
             * Sets the #UDisksBlockLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link> on `object`.
             * @param interface_ A #UDisksBlockLVM2 or %NULL to clear the interface.
             */
            set_block_lvm2(interface_?: BlockLVM2 | null): void;
            /**
             * Sets the #UDisksDrive instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link> on `object`.
             * @param interface_ A #UDisksDrive or %NULL to clear the interface.
             */
            set_drive(interface_?: Drive | null): void;
            /**
             * Sets the #UDisksDriveAta instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link> on `object`.
             * @param interface_ A #UDisksDriveAta or %NULL to clear the interface.
             */
            set_drive_ata(interface_?: DriveAta | null): void;
            /**
             * Sets the #UDisksEncrypted instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link> on `object`.
             * @param interface_ A #UDisksEncrypted or %NULL to clear the interface.
             */
            set_encrypted(interface_?: Encrypted | null): void;
            /**
             * Sets the #UDisksFilesystem instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link> on `object`.
             * @param interface_ A #UDisksFilesystem or %NULL to clear the interface.
             */
            set_filesystem(interface_?: Filesystem | null): void;
            /**
             * Sets the #UDisksFilesystemBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link> on `object`.
             * @param interface_ A #UDisksFilesystemBTRFS or %NULL to clear the interface.
             */
            set_filesystem_btrfs(interface_?: FilesystemBTRFS | null): void;
            /**
             * Sets the #UDisksJob instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link> on `object`.
             * @param interface_ A #UDisksJob or %NULL to clear the interface.
             */
            set_job(interface_?: Job | null): void;
            /**
             * Sets the #UDisksLogicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link> on `object`.
             * @param interface_ A #UDisksLogicalVolume or %NULL to clear the interface.
             */
            set_logical_volume(interface_?: LogicalVolume | null): void;
            /**
             * Sets the #UDisksLoop instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link> on `object`.
             * @param interface_ A #UDisksLoop or %NULL to clear the interface.
             */
            set_loop(interface_?: Loop | null): void;
            /**
             * Sets the #UDisksManager instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link> on `object`.
             * @param interface_ A #UDisksManager or %NULL to clear the interface.
             */
            set_manager(interface_?: Manager | null): void;
            /**
             * Sets the #UDisksManagerBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link> on `object`.
             * @param interface_ A #UDisksManagerBTRFS or %NULL to clear the interface.
             */
            set_manager_btrfs(interface_?: ManagerBTRFS | null): void;
            /**
             * Sets the #UDisksManagerLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link> on `object`.
             * @param interface_ A #UDisksManagerLVM2 or %NULL to clear the interface.
             */
            set_manager_lvm2(interface_?: ManagerLVM2 | null): void;
            /**
             * Sets the #UDisksManagerNVMe instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link> on `object`.
             * @param interface_ A #UDisksManagerNVMe or %NULL to clear the interface.
             */
            set_manager_nvme(interface_?: ManagerNVMe | null): void;
            /**
             * Sets the #UDisksMDRaid instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link> on `object`.
             * @param interface_ A #UDisksMDRaid or %NULL to clear the interface.
             */
            set_mdraid(interface_?: MDRaid | null): void;
            /**
             * Sets the #UDisksNVMeController instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link> on `object`.
             * @param interface_ A #UDisksNVMeController or %NULL to clear the interface.
             */
            set_nvme_controller(interface_?: NVMeController | null): void;
            /**
             * Sets the #UDisksNVMeFabrics instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link> on `object`.
             * @param interface_ A #UDisksNVMeFabrics or %NULL to clear the interface.
             */
            set_nvme_fabrics(interface_?: NVMeFabrics | null): void;
            /**
             * Sets the #UDisksNVMeNamespace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link> on `object`.
             * @param interface_ A #UDisksNVMeNamespace or %NULL to clear the interface.
             */
            set_nvme_namespace(interface_?: NVMeNamespace | null): void;
            /**
             * Sets the #UDisksPartition instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link> on `object`.
             * @param interface_ A #UDisksPartition or %NULL to clear the interface.
             */
            set_partition(interface_?: Partition | null): void;
            /**
             * Sets the #UDisksPartitionTable instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link> on `object`.
             * @param interface_ A #UDisksPartitionTable or %NULL to clear the interface.
             */
            set_partition_table(interface_?: PartitionTable | null): void;
            /**
             * Sets the #UDisksPhysicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link> on `object`.
             * @param interface_ A #UDisksPhysicalVolume or %NULL to clear the interface.
             */
            set_physical_volume(interface_?: PhysicalVolume | null): void;
            /**
             * Sets the #UDisksSwapspace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link> on `object`.
             * @param interface_ A #UDisksSwapspace or %NULL to clear the interface.
             */
            set_swapspace(interface_?: Swapspace | null): void;
            /**
             * Sets the #UDisksVDOVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link> on `object`.
             * @param interface_ A #UDisksVDOVolume or %NULL to clear the interface.
             */
            set_vdo_volume(interface_?: VDOVolume | null): void;
            /**
             * Sets the #UDisksVolumeGroup instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link> on `object`.
             * @param interface_ A #UDisksVolumeGroup or %NULL to clear the interface.
             */
            set_volume_group(interface_?: VolumeGroup | null): void;

            // Inherited properties
            /**
             * The #UDisksBlock instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get block(): Block;
            set block(val: Block);
            /**
             * The #UDisksBlockLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get block_lvm2(): BlockLVM2;
            set block_lvm2(val: BlockLVM2);
            /**
             * The #UDisksBlockLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get blockLvm2(): BlockLVM2;
            set blockLvm2(val: BlockLVM2);
            /**
             * The #UDisksDrive instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get drive(): Drive;
            set drive(val: Drive);
            /**
             * The #UDisksDriveAta instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get drive_ata(): DriveAta;
            set drive_ata(val: DriveAta);
            /**
             * The #UDisksDriveAta instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get driveAta(): DriveAta;
            set driveAta(val: DriveAta);
            /**
             * The #UDisksEncrypted instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get encrypted(): Encrypted;
            set encrypted(val: Encrypted);
            /**
             * The #UDisksFilesystem instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystem(): Filesystem;
            set filesystem(val: Filesystem);
            /**
             * The #UDisksFilesystemBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystem_btrfs(): FilesystemBTRFS;
            set filesystem_btrfs(val: FilesystemBTRFS);
            /**
             * The #UDisksFilesystemBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystemBtrfs(): FilesystemBTRFS;
            set filesystemBtrfs(val: FilesystemBTRFS);
            /**
             * The #UDisksJob instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get job(): Job;
            set job(val: Job);
            /**
             * The #UDisksLogicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get logical_volume(): LogicalVolume;
            set logical_volume(val: LogicalVolume);
            /**
             * The #UDisksLogicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get logicalVolume(): LogicalVolume;
            set logicalVolume(val: LogicalVolume);
            /**
             * The #UDisksLoop instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get loop(): Loop;
            set loop(val: Loop);
            /**
             * The #UDisksManager instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager(): Manager;
            set manager(val: Manager);
            /**
             * The #UDisksManagerBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_btrfs(): ManagerBTRFS;
            set manager_btrfs(val: ManagerBTRFS);
            /**
             * The #UDisksManagerBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerBtrfs(): ManagerBTRFS;
            set managerBtrfs(val: ManagerBTRFS);
            /**
             * The #UDisksManagerLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_lvm2(): ManagerLVM2;
            set manager_lvm2(val: ManagerLVM2);
            /**
             * The #UDisksManagerLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerLvm2(): ManagerLVM2;
            set managerLvm2(val: ManagerLVM2);
            /**
             * The #UDisksManagerNVMe instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_nvme(): ManagerNVMe;
            set manager_nvme(val: ManagerNVMe);
            /**
             * The #UDisksManagerNVMe instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerNvme(): ManagerNVMe;
            set managerNvme(val: ManagerNVMe);
            /**
             * The #UDisksMDRaid instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get mdraid(): MDRaid;
            set mdraid(val: MDRaid);
            /**
             * The #UDisksNVMeController instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_controller(): NVMeController;
            set nvme_controller(val: NVMeController);
            /**
             * The #UDisksNVMeController instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeController(): NVMeController;
            set nvmeController(val: NVMeController);
            /**
             * The #UDisksNVMeFabrics instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_fabrics(): NVMeFabrics;
            set nvme_fabrics(val: NVMeFabrics);
            /**
             * The #UDisksNVMeFabrics instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeFabrics(): NVMeFabrics;
            set nvmeFabrics(val: NVMeFabrics);
            /**
             * The #UDisksNVMeNamespace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_namespace(): NVMeNamespace;
            set nvme_namespace(val: NVMeNamespace);
            /**
             * The #UDisksNVMeNamespace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeNamespace(): NVMeNamespace;
            set nvmeNamespace(val: NVMeNamespace);
            /**
             * The #UDisksPartition instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partition(): Partition;
            set partition(val: Partition);
            /**
             * The #UDisksPartitionTable instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partition_table(): PartitionTable;
            set partition_table(val: PartitionTable);
            /**
             * The #UDisksPartitionTable instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partitionTable(): PartitionTable;
            set partitionTable(val: PartitionTable);
            /**
             * The #UDisksPhysicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get physical_volume(): PhysicalVolume;
            set physical_volume(val: PhysicalVolume);
            /**
             * The #UDisksPhysicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get physicalVolume(): PhysicalVolume;
            set physicalVolume(val: PhysicalVolume);
            /**
             * The #UDisksSwapspace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get swapspace(): Swapspace;
            set swapspace(val: Swapspace);
            /**
             * The #UDisksVDOVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get vdo_volume(): VDOVolume;
            set vdo_volume(val: VDOVolume);
            /**
             * The #UDisksVDOVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get vdoVolume(): VDOVolume;
            set vdoVolume(val: VDOVolume);
            /**
             * The #UDisksVolumeGroup instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get volume_group(): VolumeGroup;
            set volume_group(val: VolumeGroup);
            /**
             * The #UDisksVolumeGroup instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get volumeGroup(): VolumeGroup;
            set volumeGroup(val: VolumeGroup);

            // Inherited methods
            /**
             * Gets the D-Bus interface with name `interface_name` associated with
             * `object,` if any.
             * @param interface_name A D-Bus interface name.
             * @returns %NULL if not found, otherwise a   #GDBusInterface that must be freed with g_object_unref().
             */
            get_interface(interface_name: string): Gio.DBusInterface | null;
            /**
             * Gets the D-Bus interfaces associated with `object`.
             * @returns A list of #GDBusInterface instances.   The returned list must be freed by g_list_free() after each element has been freed   with g_object_unref().
             */
            get_interfaces(): Gio.DBusInterface[];
            /**
             * Gets the object path for `object`.
             * @returns A string owned by @object. Do not free.
             */
            get_object_path(): string;
            /**
             * Gets the D-Bus interface with name `interface_name` associated with
             * `object,` if any.
             * @param interface_name A D-Bus interface name.
             */
            vfunc_get_interface(interface_name: string): Gio.DBusInterface | null;
            /**
             * Gets the D-Bus interfaces associated with `object`.
             */
            vfunc_get_interfaces(): Gio.DBusInterface[];
            /**
             * Gets the object path for `object`.
             */
            vfunc_get_object_path(): string;
            /**
             * Signal handler for the #GDBusObject::interface-added signal.
             * @param interface_
             */
            vfunc_interface_added(interface_: Gio.DBusInterface): void;
            /**
             * Signal handler for the #GDBusObject::interface-removed signal.
             * @param interface_
             */
            vfunc_interface_removed(interface_: Gio.DBusInterface): void;
            /**
             * Gets the #UDisksBlock instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link> on `object,` if any.
             * @returns A #UDisksBlock that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_block(): Block | null;
            /**
             * Gets the #UDisksBlockLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link> on `object,` if any.
             * @returns A #UDisksBlockLVM2 that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_block_lvm2(): BlockLVM2 | null;
            /**
             * Gets the #UDisksDrive instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link> on `object,` if any.
             * @returns A #UDisksDrive that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_drive(): Drive | null;
            /**
             * Gets the #UDisksDriveAta instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link> on `object,` if any.
             * @returns A #UDisksDriveAta that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_drive_ata(): DriveAta | null;
            /**
             * Gets the #UDisksEncrypted instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link> on `object,` if any.
             * @returns A #UDisksEncrypted that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_encrypted(): Encrypted | null;
            /**
             * Gets the #UDisksFilesystem instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link> on `object,` if any.
             * @returns A #UDisksFilesystem that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_filesystem(): Filesystem | null;
            /**
             * Gets the #UDisksFilesystemBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link> on `object,` if any.
             * @returns A #UDisksFilesystemBTRFS that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_filesystem_btrfs(): FilesystemBTRFS | null;
            /**
             * Gets the #UDisksJob instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link> on `object,` if any.
             * @returns A #UDisksJob that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_job(): Job | null;
            /**
             * Gets the #UDisksLogicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link> on `object,` if any.
             * @returns A #UDisksLogicalVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_logical_volume(): LogicalVolume | null;
            /**
             * Gets the #UDisksLoop instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link> on `object,` if any.
             * @returns A #UDisksLoop that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_loop(): Loop | null;
            /**
             * Gets the #UDisksManager instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link> on `object,` if any.
             * @returns A #UDisksManager that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager(): Manager | null;
            /**
             * Gets the #UDisksManagerBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link> on `object,` if any.
             * @returns A #UDisksManagerBTRFS that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_btrfs(): ManagerBTRFS | null;
            /**
             * Gets the #UDisksManagerLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link> on `object,` if any.
             * @returns A #UDisksManagerLVM2 that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_lvm2(): ManagerLVM2 | null;
            /**
             * Gets the #UDisksManagerNVMe instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link> on `object,` if any.
             * @returns A #UDisksManagerNVMe that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_nvme(): ManagerNVMe | null;
            /**
             * Gets the #UDisksMDRaid instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link> on `object,` if any.
             * @returns A #UDisksMDRaid that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_mdraid(): MDRaid | null;
            /**
             * Gets the #UDisksNVMeController instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link> on `object,` if any.
             * @returns A #UDisksNVMeController that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_controller(): NVMeController | null;
            /**
             * Gets the #UDisksNVMeFabrics instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link> on `object,` if any.
             * @returns A #UDisksNVMeFabrics that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_fabrics(): NVMeFabrics | null;
            /**
             * Gets the #UDisksNVMeNamespace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link> on `object,` if any.
             * @returns A #UDisksNVMeNamespace that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_namespace(): NVMeNamespace | null;
            /**
             * Gets the #UDisksPartition instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link> on `object,` if any.
             * @returns A #UDisksPartition that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_partition(): Partition | null;
            /**
             * Gets the #UDisksPartitionTable instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link> on `object,` if any.
             * @returns A #UDisksPartitionTable that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_partition_table(): PartitionTable | null;
            /**
             * Gets the #UDisksPhysicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link> on `object,` if any.
             * @returns A #UDisksPhysicalVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_physical_volume(): PhysicalVolume | null;
            /**
             * Gets the #UDisksSwapspace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link> on `object,` if any.
             * @returns A #UDisksSwapspace that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_swapspace(): Swapspace | null;
            /**
             * Gets the #UDisksVDOVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link> on `object,` if any.
             * @returns A #UDisksVDOVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_vdo_volume(): VDOVolume | null;
            /**
             * Gets the #UDisksVolumeGroup instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link> on `object,` if any.
             * @returns A #UDisksVolumeGroup that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_volume_group(): VolumeGroup | null;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace PartitionProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Partition.ConstructorProps {}
        }

        /**
         * The #UDisksPartitionProxy structure contains only private data and should only be accessed using the provided API.
         */
        class PartitionProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<PartitionProxy>, Gio.DBusInterface, Gio.Initable, Partition
        {
            static $gtype: GObject.GType<PartitionProxy>;

            // Constructors

            constructor(properties?: Partial<PartitionProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): PartitionProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): PartitionProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): PartitionProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): PartitionProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_partition_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<PartitionProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_partition_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_partition_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<PartitionProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Flags">"Flags"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get flags(): number;
            set flags(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContained">"IsContained"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get is_contained(): boolean;
            set is_contained(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContained">"IsContained"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get isContained(): boolean;
            set isContained(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContainer">"IsContainer"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get is_container(): boolean;
            set is_container(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContainer">"IsContainer"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get isContainer(): boolean;
            set isContainer(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Number">"Number"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get number(): number;
            set number(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Offset">"Offset"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get offset(): number;
            set offset(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Table">"Table"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get table(): string;
            set table(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): PartitionProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_flags().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_flags().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_flags_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_flags() for the asynchronous version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_flags_sync(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_name().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_name().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_name_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_name() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_name_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_type().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_type().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_type_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_type() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_type_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_flags(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_name(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_type(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksPartition::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksPartition::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-flags signal.
             * @param invocation
             * @param arg_flags
             * @param arg_options
             */
            vfunc_handle_set_flags(
                invocation: Gio.DBusMethodInvocation,
                arg_flags: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-name signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_set_name(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-type signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_set_type(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace PartitionSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Partition.ConstructorProps {}
        }

        /**
         * The #UDisksPartitionSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class PartitionSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Partition {
            static $gtype: GObject.GType<PartitionSkeleton>;

            // Constructors

            constructor(properties?: Partial<PartitionSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): PartitionSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Flags">"Flags"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get flags(): number;
            set flags(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContained">"IsContained"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get is_contained(): boolean;
            set is_contained(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContained">"IsContained"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get isContained(): boolean;
            set isContained(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContainer">"IsContainer"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get is_container(): boolean;
            set is_container(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContainer">"IsContainer"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get isContainer(): boolean;
            set isContainer(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Number">"Number"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get number(): number;
            set number(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Offset">"Offset"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get offset(): number;
            set offset(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Table">"Table"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get table(): string;
            set table(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_flags().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_flags().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_flags_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_flags() for the asynchronous version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_flags_sync(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_name().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_name().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_name_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_name() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_name_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_type().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_type().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_type_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_type() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_type_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_flags(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_name(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_type(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksPartition::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksPartition::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-flags signal.
             * @param invocation
             * @param arg_flags
             * @param arg_options
             */
            vfunc_handle_set_flags(
                invocation: Gio.DBusMethodInvocation,
                arg_flags: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-name signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_set_name(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-type signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_set_type(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace PartitionTableProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    PartitionTable.ConstructorProps {}
        }

        /**
         * The #UDisksPartitionTableProxy structure contains only private data and should only be accessed using the provided API.
         */
        class PartitionTableProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<PartitionTableProxy>, Gio.DBusInterface, Gio.Initable, PartitionTable
        {
            static $gtype: GObject.GType<PartitionTableProxy>;

            // Constructors

            constructor(properties?: Partial<PartitionTableProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): PartitionTableProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): PartitionTableProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): PartitionTableProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): PartitionTableProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_partition_table_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<PartitionTableProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_partition_table_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_partition_table_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<PartitionTableProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PartitionTable.Partitions">"Partitions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get partitions(): string[];
            set partitions(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PartitionTable.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): PartitionTableProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_partition_table_call_create_partition_and_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_table_call_create_partition_and_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_and_format_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_table_call_create_partition_and_format() for the asynchronous version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_and_format_sync(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Finishes an operation started with udisks_partition_table_call_create_partition().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_table_call_create_partition().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_table_call_create_partition() for the asynchronous version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_sync(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param created_partition Parameter to return.
             */
            complete_create_partition(invocation: Gio.DBusMethodInvocation, created_partition: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param created_partition Parameter to return.
             */
            complete_create_partition_and_format(invocation: Gio.DBusMethodInvocation, created_partition: string): void;
            /**
             * Handler for the #UDisksPartitionTable::handle-create-partition signal.
             * @param invocation
             * @param arg_offset
             * @param arg_size
             * @param arg_type
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_create_partition(
                invocation: Gio.DBusMethodInvocation,
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartitionTable::handle-create-partition-and-format signal.
             * @param invocation
             * @param arg_offset
             * @param arg_size
             * @param arg_type
             * @param arg_name
             * @param arg_options
             * @param arg_format_type
             * @param arg_format_options
             */
            vfunc_handle_create_partition_and_format(
                invocation: Gio.DBusMethodInvocation,
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace PartitionTableSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    PartitionTable.ConstructorProps {}
        }

        /**
         * The #UDisksPartitionTableSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class PartitionTableSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, PartitionTable {
            static $gtype: GObject.GType<PartitionTableSkeleton>;

            // Constructors

            constructor(properties?: Partial<PartitionTableSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): PartitionTableSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PartitionTable.Partitions">"Partitions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get partitions(): string[];
            set partitions(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PartitionTable.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_partition_table_call_create_partition_and_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_table_call_create_partition_and_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_and_format_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_table_call_create_partition_and_format() for the asynchronous version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_and_format_sync(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Finishes an operation started with udisks_partition_table_call_create_partition().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_table_call_create_partition().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_table_call_create_partition() for the asynchronous version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_sync(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param created_partition Parameter to return.
             */
            complete_create_partition(invocation: Gio.DBusMethodInvocation, created_partition: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param created_partition Parameter to return.
             */
            complete_create_partition_and_format(invocation: Gio.DBusMethodInvocation, created_partition: string): void;
            /**
             * Handler for the #UDisksPartitionTable::handle-create-partition signal.
             * @param invocation
             * @param arg_offset
             * @param arg_size
             * @param arg_type
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_create_partition(
                invocation: Gio.DBusMethodInvocation,
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartitionTable::handle-create-partition-and-format signal.
             * @param invocation
             * @param arg_offset
             * @param arg_size
             * @param arg_type
             * @param arg_name
             * @param arg_options
             * @param arg_format_type
             * @param arg_format_options
             */
            vfunc_handle_create_partition_and_format(
                invocation: Gio.DBusMethodInvocation,
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace PhysicalVolumeProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    PhysicalVolume.ConstructorProps {}
        }

        /**
         * The #UDisksPhysicalVolumeProxy structure contains only private data and should only be accessed using the provided API.
         */
        class PhysicalVolumeProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<PhysicalVolumeProxy>, Gio.DBusInterface, Gio.Initable, PhysicalVolume
        {
            static $gtype: GObject.GType<PhysicalVolumeProxy>;

            // Constructors

            constructor(properties?: Partial<PhysicalVolumeProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): PhysicalVolumeProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): PhysicalVolumeProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): PhysicalVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): PhysicalVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_physical_volume_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_physical_volume_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<PhysicalVolumeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_physical_volume_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_physical_volume_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_physical_volume_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<PhysicalVolumeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get free_size(): number;
            set free_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get freeSize(): number;
            set freeSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volume_group(): string;
            set volume_group(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volumeGroup(): string;
            set volumeGroup(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): PhysicalVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace PhysicalVolumeSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    PhysicalVolume.ConstructorProps {}
        }

        /**
         * The #UDisksPhysicalVolumeSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class PhysicalVolumeSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, PhysicalVolume {
            static $gtype: GObject.GType<PhysicalVolumeSkeleton>;

            // Constructors

            constructor(properties?: Partial<PhysicalVolumeSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): PhysicalVolumeSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get free_size(): number;
            set free_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get freeSize(): number;
            set freeSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volume_group(): string;
            set volume_group(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volumeGroup(): string;
            set volumeGroup(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace SwapspaceProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    Swapspace.ConstructorProps {}
        }

        /**
         * The #UDisksSwapspaceProxy structure contains only private data and should only be accessed using the provided API.
         */
        class SwapspaceProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<SwapspaceProxy>, Gio.DBusInterface, Gio.Initable, Swapspace
        {
            static $gtype: GObject.GType<SwapspaceProxy>;

            // Constructors

            constructor(properties?: Partial<SwapspaceProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): SwapspaceProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): SwapspaceProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): SwapspaceProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): SwapspaceProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_swapspace_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<SwapspaceProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_swapspace_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_swapspace_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<SwapspaceProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Swapspace.Active">"Active"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active(): boolean;
            set active(val: boolean);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): SwapspaceProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_start(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_start() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_stop(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_stop().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_stop().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_stop() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_stop(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksSwapspace::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-start signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_start(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-stop signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_stop(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace SwapspaceSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Swapspace.ConstructorProps {}
        }

        /**
         * The #UDisksSwapspaceSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class SwapspaceSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, Swapspace {
            static $gtype: GObject.GType<SwapspaceSkeleton>;

            // Constructors

            constructor(properties?: Partial<SwapspaceSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): SwapspaceSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Swapspace.Active">"Active"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active(): boolean;
            set active(val: boolean);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_start(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_start() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_stop(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_stop().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_stop().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_stop() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_stop(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksSwapspace::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-start signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_start(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-stop signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_stop(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace VDOVolumeProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    VDOVolume.ConstructorProps {}
        }

        /**
         * The #UDisksVDOVolumeProxy structure contains only private data and should only be accessed using the provided API.
         */
        class VDOVolumeProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<VDOVolumeProxy>, Gio.DBusInterface, Gio.Initable, VDOVolume
        {
            static $gtype: GObject.GType<VDOVolumeProxy>;

            // Constructors

            constructor(properties?: Partial<VDOVolumeProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): VDOVolumeProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): VDOVolumeProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): VDOVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): VDOVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<VDOVolumeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_vdo_volume_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<VDOVolumeProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.Compression">"Compression"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compression(): boolean;
            set compression(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.CompressionState">"CompressionState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compression_state(): string;
            set compression_state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.CompressionState">"CompressionState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compressionState(): string;
            set compressionState(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.Deduplication">"Deduplication"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get deduplication(): boolean;
            set deduplication(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.IndexState">"IndexState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get index_state(): string;
            set index_state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.IndexState">"IndexState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get indexState(): string;
            set indexState(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.OperatingMode">"OperatingMode"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operating_mode(): string;
            set operating_mode(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.OperatingMode">"OperatingMode"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operatingMode(): string;
            set operatingMode(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.UsedSize">"UsedSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get used_size(): number;
            set used_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.UsedSize">"UsedSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get usedSize(): number;
            set usedSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.VDOPool">"VDOPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vdo_pool(): string;
            set vdo_pool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.VDOPool">"VDOPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vdoPool(): string;
            set vdoPool(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): VDOVolumeProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_enable_compression().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_enable_compression().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_compression_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_enable_compression() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_compression_sync(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_enable_deduplication().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_enable_deduplication().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_deduplication_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_enable_deduplication() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_deduplication_sync(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_get_statistics().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_get_statistics().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_statistics_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_get_statistics() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_statistics_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_resize_logical().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_resize_logical().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_logical_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_resize_logical() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_logical_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_resize_physical().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_resize_physical().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_physical_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_resize_physical() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_physical_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_compression(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_deduplication(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param stats Parameter to return.
             */
            complete_get_statistics(invocation: Gio.DBusMethodInvocation, stats: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize_logical(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize_physical(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksVDOVolume::handle-enable-compression signal.
             * @param invocation
             * @param arg_enable
             * @param arg_options
             */
            vfunc_handle_enable_compression(
                invocation: Gio.DBusMethodInvocation,
                arg_enable: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-enable-deduplication signal.
             * @param invocation
             * @param arg_enable
             * @param arg_options
             */
            vfunc_handle_enable_deduplication(
                invocation: Gio.DBusMethodInvocation,
                arg_enable: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-get-statistics signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_statistics(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-resize-logical signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize_logical(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-resize-physical signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize_physical(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace VDOVolumeSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    VDOVolume.ConstructorProps {}
        }

        /**
         * The #UDisksVDOVolumeSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class VDOVolumeSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, VDOVolume {
            static $gtype: GObject.GType<VDOVolumeSkeleton>;

            // Constructors

            constructor(properties?: Partial<VDOVolumeSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): VDOVolumeSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.Compression">"Compression"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compression(): boolean;
            set compression(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.CompressionState">"CompressionState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compression_state(): string;
            set compression_state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.CompressionState">"CompressionState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compressionState(): string;
            set compressionState(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.Deduplication">"Deduplication"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get deduplication(): boolean;
            set deduplication(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.IndexState">"IndexState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get index_state(): string;
            set index_state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.IndexState">"IndexState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get indexState(): string;
            set indexState(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.OperatingMode">"OperatingMode"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operating_mode(): string;
            set operating_mode(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.OperatingMode">"OperatingMode"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operatingMode(): string;
            set operatingMode(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.UsedSize">"UsedSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get used_size(): number;
            set used_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.UsedSize">"UsedSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get usedSize(): number;
            set usedSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.VDOPool">"VDOPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vdo_pool(): string;
            set vdo_pool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.VDOPool">"VDOPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vdoPool(): string;
            set vdoPool(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_enable_compression().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_enable_compression().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_compression_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_enable_compression() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_compression_sync(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_enable_deduplication().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_enable_deduplication().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_deduplication_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_enable_deduplication() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_deduplication_sync(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_get_statistics().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_get_statistics().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_statistics_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_get_statistics() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_statistics_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_resize_logical().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_resize_logical().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_logical_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_resize_logical() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_logical_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_resize_physical().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_resize_physical().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_physical_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_resize_physical() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_physical_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_compression(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_deduplication(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param stats Parameter to return.
             */
            complete_get_statistics(invocation: Gio.DBusMethodInvocation, stats: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize_logical(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize_physical(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Handler for the #UDisksVDOVolume::handle-enable-compression signal.
             * @param invocation
             * @param arg_enable
             * @param arg_options
             */
            vfunc_handle_enable_compression(
                invocation: Gio.DBusMethodInvocation,
                arg_enable: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-enable-deduplication signal.
             * @param invocation
             * @param arg_enable
             * @param arg_options
             */
            vfunc_handle_enable_deduplication(
                invocation: Gio.DBusMethodInvocation,
                arg_enable: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-get-statistics signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_statistics(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-resize-logical signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize_logical(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-resize-physical signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize_physical(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace VolumeGroupProxy {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusProxy.ConstructorProps,
                    Gio.AsyncInitable.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    Gio.Initable.ConstructorProps,
                    VolumeGroup.ConstructorProps {}
        }

        /**
         * The #UDisksVolumeGroupProxy structure contains only private data and should only be accessed using the provided API.
         */
        class VolumeGroupProxy
            extends Gio.DBusProxy
            implements Gio.AsyncInitable<VolumeGroupProxy>, Gio.DBusInterface, Gio.Initable, VolumeGroup
        {
            static $gtype: GObject.GType<VolumeGroupProxy>;

            // Constructors

            constructor(properties?: Partial<VolumeGroupProxy.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static new_finish(res: Gio.AsyncResult): VolumeGroupProxy;
            // Conflicted with Gio.AsyncInitable.new_finish

            static new_finish(...args: never[]): any;

            static new_for_bus_finish(res: Gio.AsyncResult): VolumeGroupProxy;

            static new_for_bus_sync(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): VolumeGroupProxy;
            // Conflicted with Gio.DBusProxy.new_for_bus_sync

            static new_for_bus_sync(...args: never[]): any;

            static new_sync(
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
            ): VolumeGroupProxy;
            // Conflicted with Gio.DBusProxy.new_sync

            static new_sync(...args: never[]): any;

            // Static methods

            /**
             * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link>. See g_dbus_proxy_new() for more details.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_proxy_new_finish() to get the result of the operation.
             *
             * See udisks_volume_group_proxy_new_sync() for the synchronous, blocking version of this constructor.
             * @param connection A #GDBusConnection.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static ['new'](
                connection: Gio.DBusConnection,
                flags: Gio.DBusProxyFlags,
                name: string | null,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<VolumeGroupProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new
            static ['new'](...args: never[]): any;
            /**
             * Like udisks_volume_group_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
             *
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_proxy_new_for_bus_finish() to get the result of the operation.
             *
             * See udisks_volume_group_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
             * @param bus_type A #GBusType.
             * @param flags Flags from the #GDBusProxyFlags enumeration.
             * @param name A bus name (well-known or unique).
             * @param object_path An object path.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied.
             */
            static new_for_bus(
                bus_type: Gio.BusType,
                flags: Gio.DBusProxyFlags,
                name: string,
                object_path: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<VolumeGroupProxy> | null,
            ): void;
            // Conflicted with Gio.DBusProxy.new_for_bus
            static new_for_bus(...args: never[]): any;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.ExtentSize">"ExtentSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get extent_size(): number;
            set extent_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.ExtentSize">"ExtentSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get extentSize(): number;
            set extentSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get free_size(): number;
            set free_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get freeSize(): number;
            set freeSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.MissingPhysicalVolumes">"MissingPhysicalVolumes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get missing_physical_volumes(): string[];
            set missing_physical_volumes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.MissingPhysicalVolumes">"MissingPhysicalVolumes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get missingPhysicalVolumes(): string[];
            set missingPhysicalVolumes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.NeedsPolling">"NeedsPolling"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get needs_polling(): boolean;
            set needs_polling(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.NeedsPolling">"NeedsPolling"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get needsPolling(): boolean;
            set needsPolling(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            init_async(io_priority: number, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             * @returns %TRUE if successful. If an error has occurred, this function will return %FALSE and set @error appropriately if present.
             */
            init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Finishes the async construction for the various g_async_initable_new
             * calls, returning the created object or %NULL on error.
             * @param res the #GAsyncResult from the callback
             * @returns a newly created #GObject,      or %NULL on error. Free with g_object_unref().
             */
            new_finish(res: Gio.AsyncResult): VolumeGroupProxy;
            // Conflicted with Gio.DBusProxy.new_finish
            new_finish(...args: never[]): any;
            /**
             * Starts asynchronous initialization of the object implementing the
             * interface. This must be done before any real use of the object after
             * initial construction. If the object also implements #GInitable you can
             * optionally call g_initable_init() instead.
             *
             * This method is intended for language bindings. If writing in C,
             * g_async_initable_new_async() should typically be used instead.
             *
             * When the initialization is finished, `callback` will be called. You can
             * then call g_async_initable_init_finish() to get the result of the
             * initialization.
             *
             * Implementations may also support cancellation. If `cancellable` is not
             * %NULL, then initialization can be cancelled by triggering the cancellable
             * object from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL, and
             * the object doesn't support cancellable initialization, the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * As with #GInitable, if the object is not initialized, or initialization
             * returns with an error, then all operations on the object except
             * g_object_ref() and g_object_unref() are considered to be invalid, and
             * have undefined behaviour. They will often fail with g_critical() or
             * g_warning(), but this must not be relied on.
             *
             * Callers should not assume that a class which implements #GAsyncInitable can
             * be initialized multiple times; for more information, see g_initable_init().
             * If a class explicitly supports being initialized multiple times,
             * implementation requires yielding all subsequent calls to init_async() on the
             * results of the first call.
             *
             * For classes that also support the #GInitable interface, the default
             * implementation of this method will run the g_initable_init() function
             * in a thread, so if you want to support asynchronous initialization via
             * threads, just implement the #GAsyncInitable interface without overriding
             * any interface methods.
             * @param io_priority the [I/O priority](iface.AsyncResult.html#io-priority) of the operation
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @param callback a #GAsyncReadyCallback to call when the request is satisfied
             */
            vfunc_init_async(
                io_priority: number,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Finishes asynchronous initialization and returns the result.
             * See g_async_initable_init_async().
             * @param res a #GAsyncResult.
             */
            vfunc_init_finish(res: Gio.AsyncResult): boolean;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             * @returns %TRUE if successful. If an error has occurred, this function will     return %FALSE and set @error appropriately if present.
             */
            init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Initializes the object implementing the interface.
             *
             * This method is intended for language bindings. If writing in C,
             * g_initable_new() should typically be used instead.
             *
             * The object must be initialized before any real use after initial
             * construction, either with this function or g_async_initable_init_async().
             *
             * Implementations may also support cancellation. If `cancellable` is not %NULL,
             * then initialization can be cancelled by triggering the cancellable object
             * from another thread. If the operation was cancelled, the error
             * %G_IO_ERROR_CANCELLED will be returned. If `cancellable` is not %NULL and
             * the object doesn't support cancellable initialization the error
             * %G_IO_ERROR_NOT_SUPPORTED will be returned.
             *
             * If the object is not initialized, or initialization returns with an
             * error, then all operations on the object except g_object_ref() and
             * g_object_unref() are considered to be invalid, and have undefined
             * behaviour. See the [description][iface`Gio`.Initable#description] for more details.
             *
             * Callers should not assume that a class which implements #GInitable can be
             * initialized multiple times, unless the class explicitly documents itself as
             * supporting this. Generally, a class’ implementation of init() can assume
             * (and assert) that it will only be called once. Previously, this documentation
             * recommended all #GInitable implementations should be idempotent; that
             * recommendation was relaxed in GLib 2.54.
             *
             * If a class explicitly supports being initialized multiple times, it is
             * recommended that the method is idempotent: multiple calls with the same
             * arguments should return the same results. Only the first call initializes
             * the object; further calls return the result of the first call.
             *
             * One reason why a class might need to support idempotent initialization is if
             * it is designed to be used via the singleton pattern, with a
             * #GObjectClass.constructor that sometimes returns an existing instance.
             * In this pattern, a caller would expect to be able to call g_initable_init()
             * on the result of g_object_new(), regardless of whether it is in fact a new
             * instance.
             * @param cancellable optional #GCancellable object, %NULL to ignore.
             */
            vfunc_init(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_add_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_plain_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_plain_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_plain_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_plain_volume_with_layout().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_plain_volume_with_layout().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_with_layout_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_with_layout_sync(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_thin_pool_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_thin_pool_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_pool_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_thin_pool_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_pool_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_thin_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_thin_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_thin_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_vdo_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_vdo_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_vdo_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_vdo_volume() for the asynchronous version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_vdo_volume_sync(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_delete() for the asynchronous version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_empty_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_empty_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_empty_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_empty_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_empty_device_sync(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_poll(cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_poll(cancellable: Gio.Cancellable | null, callback: Gio.AsyncReadyCallback<this> | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_poll(
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_poll().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_poll().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_poll_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_poll() for the asynchronous version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_poll_sync(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_remove_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_remove_missing_physical_volumes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_remove_missing_physical_volumes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_missing_physical_volumes_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_missing_physical_volumes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_rename().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_rename().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_rename() for the asynchronous version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_sync(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_plain_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_plain_volume_with_layout(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_thin_pool_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_thin_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_vdo_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_empty_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_poll(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_missing_physical_volumes(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_rename(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Handler for the #UDisksVolumeGroup::handle-add-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-plain-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_plain_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-plain-volume-with-layout signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_layout
             * @param arg_pvs
             * @param arg_options
             */
            vfunc_handle_create_plain_volume_with_layout(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-thin-pool-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_thin_pool_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-thin-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_pool
             * @param arg_options
             */
            vfunc_handle_create_thin_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-vdo-volume signal.
             * @param invocation
             * @param arg_lv_name
             * @param arg_pool_name
             * @param arg_data_size
             * @param arg_virtual_size
             * @param arg_index_memory
             * @param arg_compression
             * @param arg_deduplication
             * @param arg_write_policy
             * @param arg_options
             */
            vfunc_handle_create_vdo_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-delete signal.
             * @param invocation
             * @param arg_wipe
             * @param arg_options
             */
            vfunc_handle_delete(
                invocation: Gio.DBusMethodInvocation,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-empty-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_options
             */
            vfunc_handle_empty_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-poll signal.
             * @param invocation
             */
            vfunc_handle_poll(invocation: Gio.DBusMethodInvocation): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-remove-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_wipe
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-remove-missing-physical-volumes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_remove_missing_physical_volumes(
                invocation: Gio.DBusMethodInvocation,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-rename signal.
             * @param invocation
             * @param arg_new_name
             * @param arg_options
             */
            vfunc_handle_rename(
                invocation: Gio.DBusMethodInvocation,
                arg_new_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        namespace VolumeGroupSkeleton {
            // Constructor properties interface

            interface ConstructorProps
                extends Gio.DBusInterfaceSkeleton.ConstructorProps,
                    Gio.DBusInterface.ConstructorProps,
                    VolumeGroup.ConstructorProps {}
        }

        /**
         * The #UDisksVolumeGroupSkeleton structure contains only private data and should only be accessed using the provided API.
         */
        class VolumeGroupSkeleton extends Gio.DBusInterfaceSkeleton implements Gio.DBusInterface, VolumeGroup {
            static $gtype: GObject.GType<VolumeGroupSkeleton>;

            // Constructors

            constructor(properties?: Partial<VolumeGroupSkeleton.ConstructorProps>, ...args: any[]);

            _init(...args: any[]): void;

            static ['new'](): VolumeGroupSkeleton;

            // Inherited properties
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.ExtentSize">"ExtentSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get extent_size(): number;
            set extent_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.ExtentSize">"ExtentSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get extentSize(): number;
            set extentSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get free_size(): number;
            set free_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get freeSize(): number;
            set freeSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.MissingPhysicalVolumes">"MissingPhysicalVolumes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get missing_physical_volumes(): string[];
            set missing_physical_volumes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.MissingPhysicalVolumes">"MissingPhysicalVolumes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get missingPhysicalVolumes(): string[];
            set missingPhysicalVolumes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.NeedsPolling">"NeedsPolling"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get needs_polling(): boolean;
            set needs_polling(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.NeedsPolling">"NeedsPolling"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get needsPolling(): boolean;
            set needsPolling(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Inherited methods
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             * @returns A #GDBusObject or %NULL. The returned reference should be freed with g_object_unref().
             */
            get_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             * @returns A #GDBusInterfaceInfo. Do not free.
             */
            get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            set_object(object?: Gio.DBusObject | null): void;
            /**
             * Gets the #GDBusObject that `interface_` belongs to, if any.
             */
            vfunc_dup_object(): Gio.DBusObject | null;
            /**
             * Gets D-Bus introspection information for the D-Bus interface
             * implemented by `interface_`.
             */
            vfunc_get_info(): Gio.DBusInterfaceInfo;
            /**
             * Sets the #GDBusObject for `interface_` to `object`.
             *
             * Note that `interface_` will hold a weak reference to `object`.
             * @param object A #GDBusObject or %NULL.
             */
            vfunc_set_object(object?: Gio.DBusObject | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_add_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_plain_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_plain_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_plain_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_plain_volume_with_layout().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_plain_volume_with_layout().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_with_layout_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_with_layout_sync(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_thin_pool_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_thin_pool_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_pool_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_thin_pool_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_pool_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_thin_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_thin_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_thin_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_vdo_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_vdo_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_vdo_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_vdo_volume() for the asynchronous version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_vdo_volume_sync(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_delete() for the asynchronous version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_empty_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_empty_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_empty_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_empty_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_empty_device_sync(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_poll(cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_poll(cancellable: Gio.Cancellable | null, callback: Gio.AsyncReadyCallback<this> | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_poll(
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_poll().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_poll().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_poll_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_poll() for the asynchronous version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_poll_sync(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_remove_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_remove_missing_physical_volumes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_remove_missing_physical_volumes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_missing_physical_volumes_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_missing_physical_volumes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_rename().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_rename().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_rename() for the asynchronous version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_sync(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_plain_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_plain_volume_with_layout(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_thin_pool_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_thin_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_vdo_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_empty_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_poll(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_missing_physical_volumes(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_rename(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Handler for the #UDisksVolumeGroup::handle-add-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-plain-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_plain_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-plain-volume-with-layout signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_layout
             * @param arg_pvs
             * @param arg_options
             */
            vfunc_handle_create_plain_volume_with_layout(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-thin-pool-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_thin_pool_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-thin-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_pool
             * @param arg_options
             */
            vfunc_handle_create_thin_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-vdo-volume signal.
             * @param invocation
             * @param arg_lv_name
             * @param arg_pool_name
             * @param arg_data_size
             * @param arg_virtual_size
             * @param arg_index_memory
             * @param arg_compression
             * @param arg_deduplication
             * @param arg_write_policy
             * @param arg_options
             */
            vfunc_handle_create_vdo_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-delete signal.
             * @param invocation
             * @param arg_wipe
             * @param arg_options
             */
            vfunc_handle_delete(
                invocation: Gio.DBusMethodInvocation,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-empty-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_options
             */
            vfunc_handle_empty_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-poll signal.
             * @param invocation
             */
            vfunc_handle_poll(invocation: Gio.DBusMethodInvocation): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-remove-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_wipe
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-remove-missing-physical-volumes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_remove_missing_physical_volumes(
                invocation: Gio.DBusMethodInvocation,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-rename signal.
             * @param invocation
             * @param arg_new_name
             * @param arg_options
             */
            vfunc_handle_rename(
                invocation: Gio.DBusMethodInvocation,
                arg_new_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target`.
             *
             * Whenever the `source_property` is changed the `target_property` is
             * updated using the same value. For instance:
             *
             *
             * ```c
             *   g_object_bind_property (action, "active", widget, "sensitive", 0);
             * ```
             *
             *
             * Will result in the "sensitive" property of the widget #GObject instance to be
             * updated with the same value of the "active" property of the action #GObject
             * instance.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well.
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. To remove the binding without affecting the
             * `source` and the `target` you can just call g_object_unref() on the returned
             * #GBinding instance.
             *
             * Removing the binding by calling g_object_unref() on it must only be done if
             * the binding, `source` and `target` are only used from a single thread and it
             * is clear that both `source` and `target` outlive the binding. Especially it
             * is not safe to rely on this if the binding, `source` or `target` can be
             * finalized from different threads. Keep another reference to the binding and
             * use g_binding_unbind() instead to be on the safe side.
             *
             * A #GObject can have multiple bindings.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
            ): GObject.Binding;
            /**
             * Complete version of g_object_bind_property().
             *
             * Creates a binding between `source_property` on `source` and `target_property`
             * on `target,` allowing you to set the transformation functions to be used by
             * the binding.
             *
             * If `flags` contains %G_BINDING_BIDIRECTIONAL then the binding will be mutual:
             * if `target_property` on `target` changes then the `source_property` on `source`
             * will be updated as well. The `transform_from` function is only used in case
             * of bidirectional bindings, otherwise it will be ignored
             *
             * The binding will automatically be removed when either the `source` or the
             * `target` instances are finalized. This will release the reference that is
             * being held on the #GBinding instance; if you want to hold on to the
             * #GBinding instance, you will need to hold a reference to it.
             *
             * To remove the binding, call g_binding_unbind().
             *
             * A #GObject can have multiple bindings.
             *
             * The same `user_data` parameter will be used for both `transform_to`
             * and `transform_from` transformation functions; the `notify` function will
             * be called once, when the binding is removed. If you need different data
             * for each transformation function, please use
             * g_object_bind_property_with_closures() instead.
             * @param source_property the property on @source to bind
             * @param target the target #GObject
             * @param target_property the property on @target to bind
             * @param flags flags to pass to #GBinding
             * @param transform_to the transformation function     from the @source to the @target, or %NULL to use the default
             * @param transform_from the transformation function     from the @target to the @source, or %NULL to use the default
             * @param notify a function to call when disposing the binding, to free     resources used by the transformation functions, or %NULL if not required
             * @returns the #GBinding instance representing the     binding between the two #GObject instances. The binding is released     whenever the #GBinding reference count reaches zero.
             */
            bind_property_full(
                source_property: string,
                target: GObject.Object,
                target_property: string,
                flags: GObject.BindingFlags | null,
                transform_to?: GObject.BindingTransformFunc | null,
                transform_from?: GObject.BindingTransformFunc | null,
                notify?: GLib.DestroyNotify | null,
            ): GObject.Binding;
            // Conflicted with GObject.Object.bind_property_full
            bind_property_full(...args: never[]): any;
            /**
             * This function is intended for #GObject implementations to re-enforce
             * a [floating][floating-ref] object reference. Doing this is seldom
             * required: all #GInitiallyUnowneds are created with a floating reference
             * which usually just needs to be sunken by calling g_object_ref_sink().
             */
            force_floating(): void;
            /**
             * Increases the freeze count on `object`. If the freeze count is
             * non-zero, the emission of "notify" signals on `object` is
             * stopped. The signals are queued until the freeze count is decreased
             * to zero. Duplicate notifications are squashed so that at most one
             * #GObject::notify signal is emitted for each property modified while the
             * object is frozen.
             *
             * This is necessary for accessors that modify multiple properties to prevent
             * premature notification while the object is still being modified.
             */
            freeze_notify(): void;
            /**
             * Gets a named field from the objects table of associations (see g_object_set_data()).
             * @param key name of the key for that association
             * @returns the data if found,          or %NULL if no such data exists.
             */
            get_data(key: string): any | null;
            /**
             * Gets a property of an object.
             *
             * The value can be:
             * - an empty GObject.Value initialized by G_VALUE_INIT, which will be automatically initialized with the expected type of the property (since GLib 2.60)
             * - a GObject.Value initialized with the expected type of the property
             * - a GObject.Value initialized with a type to which the expected type of the property can be transformed
             *
             * In general, a copy is made of the property contents and the caller is responsible for freeing the memory by calling GObject.Value.unset.
             *
             * Note that GObject.Object.get_property is really intended for language bindings, GObject.Object.get is much more convenient for C programming.
             * @param property_name The name of the property to get
             * @param value Return location for the property value. Can be an empty GObject.Value initialized by G_VALUE_INIT (auto-initialized with expected type since GLib 2.60), a GObject.Value initialized with the expected property type, or a GObject.Value initialized with a transformable type
             */
            get_property(property_name: string, value: GObject.Value | any): any;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            get_qdata(quark: GLib.Quark): any | null;
            /**
             * Gets `n_properties` properties for an `object`.
             * Obtained properties will be set to `values`. All properties must be valid.
             * Warnings will be emitted and undefined behaviour may result if invalid
             * properties are passed in.
             * @param names the names of each property to get
             * @param values the values of each property to get
             */
            getv(names: string[], values: (GObject.Value | any)[]): void;
            /**
             * Checks whether `object` has a [floating][floating-ref] reference.
             * @returns %TRUE if @object has a floating reference
             */
            is_floating(): boolean;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param property_name the name of a property installed on the class of @object.
             */
            notify(property_name: string): void;
            /**
             * Emits a "notify" signal for the property specified by `pspec` on `object`.
             *
             * This function omits the property name lookup, hence it is faster than
             * g_object_notify().
             *
             * One way to avoid using g_object_notify() from within the
             * class that registered the properties, and using g_object_notify_by_pspec()
             * instead, is to store the GParamSpec used with
             * g_object_class_install_property() inside a static array, e.g.:
             *
             *
             * ```c
             *   typedef enum
             *   {
             *     PROP_FOO = 1,
             *     PROP_LAST
             *   } MyObjectProperty;
             *
             *   static GParamSpec *properties[PROP_LAST];
             *
             *   static void
             *   my_object_class_init (MyObjectClass *klass)
             *   {
             *     properties[PROP_FOO] = g_param_spec_int ("foo", NULL, NULL,
             *                                              0, 100,
             *                                              50,
             *                                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
             *     g_object_class_install_property (gobject_class,
             *                                      PROP_FOO,
             *                                      properties[PROP_FOO]);
             *   }
             * ```
             *
             *
             * and then notify a change on the "foo" property with:
             *
             *
             * ```c
             *   g_object_notify_by_pspec (self, properties[PROP_FOO]);
             * ```
             *
             * @param pspec the #GParamSpec of a property installed on the class of @object.
             */
            notify_by_pspec(pspec: GObject.ParamSpec): void;
            /**
             * Increases the reference count of `object`.
             *
             * Since GLib 2.56, if `GLIB_VERSION_MAX_ALLOWED` is 2.56 or greater, the type
             * of `object` will be propagated to the return type (using the GCC typeof()
             * extension), so any casting the caller needs to do on the return type must be
             * explicit.
             * @returns the same @object
             */
            ref(): GObject.Object;
            /**
             * Increase the reference count of `object,` and possibly remove the
             * [floating][floating-ref] reference, if `object` has a floating reference.
             *
             * In other words, if the object is floating, then this call "assumes
             * ownership" of the floating reference, converting it to a normal
             * reference by clearing the floating flag while leaving the reference
             * count unchanged.  If the object is not floating, then this call
             * adds a new normal reference increasing the reference count by one.
             *
             * Since GLib 2.56, the type of `object` will be propagated to the return type
             * under the same conditions as for g_object_ref().
             * @returns @object
             */
            ref_sink(): GObject.Object;
            /**
             * Releases all references to other objects. This can be used to break
             * reference cycles.
             *
             * This function should only be called from object system implementations.
             */
            run_dispose(): void;
            /**
             * Each object carries around a table of associations from
             * strings to pointers.  This function lets you set an association.
             *
             * If the object already had an association with that name,
             * the old association will be destroyed.
             *
             * Internally, the `key` is converted to a #GQuark using g_quark_from_string().
             * This means a copy of `key` is kept permanently (even after `object` has been
             * finalized) — so it is recommended to only use a small, bounded set of values
             * for `key` in your program, to avoid the #GQuark storage growing unbounded.
             * @param key name of the key
             * @param data data to associate with that key
             */
            set_data(key: string, data?: any | null): void;
            /**
             * Sets a property on an object.
             * @param property_name The name of the property to set
             * @param value The value to set the property to
             */
            set_property(property_name: string, value: GObject.Value | any): void;
            /**
             * Remove a specified datum from the object's data associations,
             * without invoking the association's destroy handler.
             * @param key name of the key
             * @returns the data if found, or %NULL          if no such data exists.
             */
            steal_data(key: string): any | null;
            /**
             * This function gets back user data pointers stored via
             * g_object_set_qdata() and removes the `data` from object
             * without invoking its destroy() function (if any was
             * set).
             * Usually, calling this function is only required to update
             * user data pointers with a destroy notifier, for example:
             *
             * ```c
             * void
             * object_add_to_user_list (GObject     *object,
             *                          const gchar *new_string)
             * {
             *   // the quark, naming the object data
             *   GQuark quark_string_list = g_quark_from_static_string ("my-string-list");
             *   // retrieve the old string list
             *   GList *list = g_object_steal_qdata (object, quark_string_list);
             *
             *   // prepend new string
             *   list = g_list_prepend (list, g_strdup (new_string));
             *   // this changed 'list', so we need to set it again
             *   g_object_set_qdata_full (object, quark_string_list, list, free_string_list);
             * }
             * static void
             * free_string_list (gpointer data)
             * {
             *   GList *node, *list = data;
             *
             *   for (node = list; node; node = node->next)
             *     g_free (node->data);
             *   g_list_free (list);
             * }
             * ```
             *
             * Using g_object_get_qdata() in the above example, instead of
             * g_object_steal_qdata() would have left the destroy function set,
             * and thus the partial string list would have been freed upon
             * g_object_set_qdata_full().
             * @param quark A #GQuark, naming the user data pointer
             * @returns The user data pointer set, or %NULL
             */
            steal_qdata(quark: GLib.Quark): any | null;
            /**
             * Reverts the effect of a previous call to
             * g_object_freeze_notify(). The freeze count is decreased on `object`
             * and when it reaches zero, queued "notify" signals are emitted.
             *
             * Duplicate notifications for each property are squashed so that at most one
             * #GObject::notify signal is emitted for each property, in the reverse order
             * in which they have been queued.
             *
             * It is an error to call this function when the freeze count is zero.
             */
            thaw_notify(): void;
            /**
             * Decreases the reference count of `object`. When its reference count
             * drops to 0, the object is finalized (i.e. its memory is freed).
             *
             * If the pointer to the #GObject may be reused in future (for example, if it is
             * an instance variable of another object), it is recommended to clear the
             * pointer to %NULL rather than retain a dangling pointer to a potentially
             * invalid #GObject instance. Use g_clear_object() for this.
             */
            unref(): void;
            /**
             * This function essentially limits the life time of the `closure` to
             * the life time of the object. That is, when the object is finalized,
             * the `closure` is invalidated by calling g_closure_invalidate() on
             * it, in order to prevent invocations of the closure with a finalized
             * (nonexisting) object. Also, g_object_ref() and g_object_unref() are
             * added as marshal guards to the `closure,` to ensure that an extra
             * reference count is held on `object` during invocation of the
             * `closure`.  Usually, this function will be called on closures that
             * use this `object` as closure data.
             * @param closure #GClosure to watch
             */
            watch_closure(closure: GObject.Closure): void;
            /**
             * the `constructed` function is called by g_object_new() as the
             *  final step of the object creation process.  At the point of the call, all
             *  construction properties have been set on the object.  The purpose of this
             *  call is to allow for object initialisation steps that can only be performed
             *  after construction properties have been set.  `constructed` implementors
             *  should chain up to the `constructed` call of their parent class to allow it
             *  to complete its initialisation.
             */
            vfunc_constructed(): void;
            /**
             * emits property change notification for a bunch
             *  of properties. Overriding `dispatch_properties_changed` should be rarely
             *  needed.
             * @param n_pspecs
             * @param pspecs
             */
            vfunc_dispatch_properties_changed(n_pspecs: number, pspecs: GObject.ParamSpec): void;
            /**
             * the `dispose` function is supposed to drop all references to other
             *  objects, but keep the instance otherwise intact, so that client method
             *  invocations still work. It may be run multiple times (due to reference
             *  loops). Before returning, `dispose` should chain up to the `dispose` method
             *  of the parent class.
             */
            vfunc_dispose(): void;
            /**
             * instance finalization function, should finish the finalization of
             *  the instance begun in `dispose` and chain up to the `finalize` method of the
             *  parent class.
             */
            vfunc_finalize(): void;
            /**
             * the generic getter for all properties of this type. Should be
             *  overridden for every type with properties.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_get_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Emits a "notify" signal for the property `property_name` on `object`.
             *
             * When possible, eg. when signaling a property change from within the class
             * that registered the property, you should use g_object_notify_by_pspec()
             * instead.
             *
             * Note that emission of the notify signal may be blocked with
             * g_object_freeze_notify(). In this case, the signal emissions are queued
             * and will be emitted (in reverse order) when g_object_thaw_notify() is
             * called.
             * @param pspec
             */
            vfunc_notify(pspec: GObject.ParamSpec): void;
            /**
             * the generic setter for all properties of this type. Should be
             *  overridden for every type with properties. If implementations of
             *  `set_property` don't emit property change notification explicitly, this will
             *  be done implicitly by the type system. However, if the notify signal is
             *  emitted explicitly, the type system will not emit it a second time.
             * @param property_id
             * @param value
             * @param pspec
             */
            vfunc_set_property(property_id: number, value: GObject.Value | any, pspec: GObject.ParamSpec): void;
            /**
             * Disconnects a handler from an instance so it will not be called during any future or currently ongoing emissions of the signal it has been connected to.
             * @param id Handler ID of the handler to be disconnected
             */
            disconnect(id: number): void;
            /**
             * Sets multiple properties of an object at once. The properties argument should be a dictionary mapping property names to values.
             * @param properties Object containing the properties to set
             */
            set(properties: { [key: string]: any }): void;
            /**
             * Blocks a handler of an instance so it will not be called during any signal emissions
             * @param id Handler ID of the handler to be blocked
             */
            block_signal_handler(id: number): void;
            /**
             * Unblocks a handler so it will be called again during any signal emissions
             * @param id Handler ID of the handler to be unblocked
             */
            unblock_signal_handler(id: number): void;
            /**
             * Stops a signal's emission by the given signal name. This will prevent the default handler and any subsequent signal handlers from being invoked.
             * @param detailedName Name of the signal to stop emission of
             */
            stop_emission_by_name(detailedName: string): void;
        }

        type BlockIface = typeof Block;
        type BlockLVM2Iface = typeof BlockLVM2;
        type BlockLVM2ProxyClass = typeof BlockLVM2Proxy;
        abstract class BlockLVM2ProxyPrivate {
            static $gtype: GObject.GType<BlockLVM2ProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type BlockLVM2SkeletonClass = typeof BlockLVM2Skeleton;
        abstract class BlockLVM2SkeletonPrivate {
            static $gtype: GObject.GType<BlockLVM2SkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type BlockProxyClass = typeof BlockProxy;
        abstract class BlockProxyPrivate {
            static $gtype: GObject.GType<BlockProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type BlockSkeletonClass = typeof BlockSkeleton;
        abstract class BlockSkeletonPrivate {
            static $gtype: GObject.GType<BlockSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type DriveAtaIface = typeof DriveAta;
        type DriveAtaProxyClass = typeof DriveAtaProxy;
        abstract class DriveAtaProxyPrivate {
            static $gtype: GObject.GType<DriveAtaProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type DriveAtaSkeletonClass = typeof DriveAtaSkeleton;
        abstract class DriveAtaSkeletonPrivate {
            static $gtype: GObject.GType<DriveAtaSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type DriveIface = typeof Drive;
        type DriveProxyClass = typeof DriveProxy;
        abstract class DriveProxyPrivate {
            static $gtype: GObject.GType<DriveProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type DriveSkeletonClass = typeof DriveSkeleton;
        abstract class DriveSkeletonPrivate {
            static $gtype: GObject.GType<DriveSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type EncryptedIface = typeof Encrypted;
        type EncryptedProxyClass = typeof EncryptedProxy;
        abstract class EncryptedProxyPrivate {
            static $gtype: GObject.GType<EncryptedProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type EncryptedSkeletonClass = typeof EncryptedSkeleton;
        abstract class EncryptedSkeletonPrivate {
            static $gtype: GObject.GType<EncryptedSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type FilesystemBTRFSIface = typeof FilesystemBTRFS;
        type FilesystemBTRFSProxyClass = typeof FilesystemBTRFSProxy;
        abstract class FilesystemBTRFSProxyPrivate {
            static $gtype: GObject.GType<FilesystemBTRFSProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type FilesystemBTRFSSkeletonClass = typeof FilesystemBTRFSSkeleton;
        abstract class FilesystemBTRFSSkeletonPrivate {
            static $gtype: GObject.GType<FilesystemBTRFSSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type FilesystemIface = typeof Filesystem;
        type FilesystemProxyClass = typeof FilesystemProxy;
        abstract class FilesystemProxyPrivate {
            static $gtype: GObject.GType<FilesystemProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type FilesystemSkeletonClass = typeof FilesystemSkeleton;
        abstract class FilesystemSkeletonPrivate {
            static $gtype: GObject.GType<FilesystemSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type JobIface = typeof Job;
        type JobProxyClass = typeof JobProxy;
        abstract class JobProxyPrivate {
            static $gtype: GObject.GType<JobProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type JobSkeletonClass = typeof JobSkeleton;
        abstract class JobSkeletonPrivate {
            static $gtype: GObject.GType<JobSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type LogicalVolumeIface = typeof LogicalVolume;
        type LogicalVolumeProxyClass = typeof LogicalVolumeProxy;
        abstract class LogicalVolumeProxyPrivate {
            static $gtype: GObject.GType<LogicalVolumeProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type LogicalVolumeSkeletonClass = typeof LogicalVolumeSkeleton;
        abstract class LogicalVolumeSkeletonPrivate {
            static $gtype: GObject.GType<LogicalVolumeSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type LoopIface = typeof Loop;
        type LoopProxyClass = typeof LoopProxy;
        abstract class LoopProxyPrivate {
            static $gtype: GObject.GType<LoopProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type LoopSkeletonClass = typeof LoopSkeleton;
        abstract class LoopSkeletonPrivate {
            static $gtype: GObject.GType<LoopSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type MDRaidIface = typeof MDRaid;
        type MDRaidProxyClass = typeof MDRaidProxy;
        abstract class MDRaidProxyPrivate {
            static $gtype: GObject.GType<MDRaidProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type MDRaidSkeletonClass = typeof MDRaidSkeleton;
        abstract class MDRaidSkeletonPrivate {
            static $gtype: GObject.GType<MDRaidSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerBTRFSIface = typeof ManagerBTRFS;
        type ManagerBTRFSProxyClass = typeof ManagerBTRFSProxy;
        abstract class ManagerBTRFSProxyPrivate {
            static $gtype: GObject.GType<ManagerBTRFSProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerBTRFSSkeletonClass = typeof ManagerBTRFSSkeleton;
        abstract class ManagerBTRFSSkeletonPrivate {
            static $gtype: GObject.GType<ManagerBTRFSSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerIface = typeof Manager;
        type ManagerLVM2Iface = typeof ManagerLVM2;
        type ManagerLVM2ProxyClass = typeof ManagerLVM2Proxy;
        abstract class ManagerLVM2ProxyPrivate {
            static $gtype: GObject.GType<ManagerLVM2ProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerLVM2SkeletonClass = typeof ManagerLVM2Skeleton;
        abstract class ManagerLVM2SkeletonPrivate {
            static $gtype: GObject.GType<ManagerLVM2SkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerNVMeIface = typeof ManagerNVMe;
        type ManagerNVMeProxyClass = typeof ManagerNVMeProxy;
        abstract class ManagerNVMeProxyPrivate {
            static $gtype: GObject.GType<ManagerNVMeProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerNVMeSkeletonClass = typeof ManagerNVMeSkeleton;
        abstract class ManagerNVMeSkeletonPrivate {
            static $gtype: GObject.GType<ManagerNVMeSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerProxyClass = typeof ManagerProxy;
        abstract class ManagerProxyPrivate {
            static $gtype: GObject.GType<ManagerProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ManagerSkeletonClass = typeof ManagerSkeleton;
        abstract class ManagerSkeletonPrivate {
            static $gtype: GObject.GType<ManagerSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type NVMeControllerIface = typeof NVMeController;
        type NVMeControllerProxyClass = typeof NVMeControllerProxy;
        abstract class NVMeControllerProxyPrivate {
            static $gtype: GObject.GType<NVMeControllerProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type NVMeControllerSkeletonClass = typeof NVMeControllerSkeleton;
        abstract class NVMeControllerSkeletonPrivate {
            static $gtype: GObject.GType<NVMeControllerSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type NVMeFabricsIface = typeof NVMeFabrics;
        type NVMeFabricsProxyClass = typeof NVMeFabricsProxy;
        abstract class NVMeFabricsProxyPrivate {
            static $gtype: GObject.GType<NVMeFabricsProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type NVMeFabricsSkeletonClass = typeof NVMeFabricsSkeleton;
        abstract class NVMeFabricsSkeletonPrivate {
            static $gtype: GObject.GType<NVMeFabricsSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type NVMeNamespaceIface = typeof NVMeNamespace;
        type NVMeNamespaceProxyClass = typeof NVMeNamespaceProxy;
        abstract class NVMeNamespaceProxyPrivate {
            static $gtype: GObject.GType<NVMeNamespaceProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type NVMeNamespaceSkeletonClass = typeof NVMeNamespaceSkeleton;
        abstract class NVMeNamespaceSkeletonPrivate {
            static $gtype: GObject.GType<NVMeNamespaceSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ObjectIface = typeof Object;
        type ObjectManagerClientClass = typeof ObjectManagerClient;
        abstract class ObjectManagerClientPrivate {
            static $gtype: GObject.GType<ObjectManagerClientPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ObjectProxyClass = typeof ObjectProxy;
        abstract class ObjectProxyPrivate {
            static $gtype: GObject.GType<ObjectProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type ObjectSkeletonClass = typeof ObjectSkeleton;
        abstract class ObjectSkeletonPrivate {
            static $gtype: GObject.GType<ObjectSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type PartitionIface = typeof Partition;
        type PartitionProxyClass = typeof PartitionProxy;
        abstract class PartitionProxyPrivate {
            static $gtype: GObject.GType<PartitionProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type PartitionSkeletonClass = typeof PartitionSkeleton;
        abstract class PartitionSkeletonPrivate {
            static $gtype: GObject.GType<PartitionSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type PartitionTableIface = typeof PartitionTable;
        type PartitionTableProxyClass = typeof PartitionTableProxy;
        abstract class PartitionTableProxyPrivate {
            static $gtype: GObject.GType<PartitionTableProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type PartitionTableSkeletonClass = typeof PartitionTableSkeleton;
        abstract class PartitionTableSkeletonPrivate {
            static $gtype: GObject.GType<PartitionTableSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        /**
         * Detailed information about a partition type.
         *
         * `table_subtype` is used to break the set of partition types for
         * `table_type` into a logical subsets. It is typically only used in
         * user interfaces where the partition type is selected.
         *
         * This struct may grow in the future without it being considered an
         * ABI break.
         */
        class PartitionTypeInfo {
            static $gtype: GObject.GType<PartitionTypeInfo>;

            // Fields

            table_type: string;
            table_subtype: string;
            type: string;
            flags: PartitionTypeInfoFlags;

            // Constructors

            _init(...args: any[]): void;

            // Methods

            /**
             * Frees `info`.
             */
            free(): void;
        }

        type PhysicalVolumeIface = typeof PhysicalVolume;
        type PhysicalVolumeProxyClass = typeof PhysicalVolumeProxy;
        abstract class PhysicalVolumeProxyPrivate {
            static $gtype: GObject.GType<PhysicalVolumeProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type PhysicalVolumeSkeletonClass = typeof PhysicalVolumeSkeleton;
        abstract class PhysicalVolumeSkeletonPrivate {
            static $gtype: GObject.GType<PhysicalVolumeSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type SwapspaceIface = typeof Swapspace;
        type SwapspaceProxyClass = typeof SwapspaceProxy;
        abstract class SwapspaceProxyPrivate {
            static $gtype: GObject.GType<SwapspaceProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type SwapspaceSkeletonClass = typeof SwapspaceSkeleton;
        abstract class SwapspaceSkeletonPrivate {
            static $gtype: GObject.GType<SwapspaceSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type VDOVolumeIface = typeof VDOVolume;
        type VDOVolumeProxyClass = typeof VDOVolumeProxy;
        abstract class VDOVolumeProxyPrivate {
            static $gtype: GObject.GType<VDOVolumeProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type VDOVolumeSkeletonClass = typeof VDOVolumeSkeleton;
        abstract class VDOVolumeSkeletonPrivate {
            static $gtype: GObject.GType<VDOVolumeSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type VolumeGroupIface = typeof VolumeGroup;
        type VolumeGroupProxyClass = typeof VolumeGroupProxy;
        abstract class VolumeGroupProxyPrivate {
            static $gtype: GObject.GType<VolumeGroupProxyPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        type VolumeGroupSkeletonClass = typeof VolumeGroupSkeleton;
        abstract class VolumeGroupSkeletonPrivate {
            static $gtype: GObject.GType<VolumeGroupSkeletonPrivate>;

            // Constructors

            _init(...args: any[]): void;
        }

        namespace Block {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                configuration: GLib.Variant;
                crypto_backing_device: string;
                cryptoBackingDevice: string;
                device: string;
                device_number: number;
                deviceNumber: number;
                drive: string;
                hint_auto: boolean;
                hintAuto: boolean;
                hint_icon_name: string;
                hintIconName: string;
                hint_ignore: boolean;
                hintIgnore: boolean;
                hint_name: string;
                hintName: string;
                hint_partitionable: boolean;
                hintPartitionable: boolean;
                hint_symbolic_icon_name: string;
                hintSymbolicIconName: string;
                hint_system: boolean;
                hintSystem: boolean;
                id: string;
                id_label: string;
                idLabel: string;
                id_type: string;
                idType: string;
                id_usage: string;
                idUsage: string;
                id_uuid: string;
                idUuid: string;
                id_version: string;
                idVersion: string;
                mdraid: string;
                mdraid_member: string;
                mdraidMember: string;
                preferred_device: string;
                preferredDevice: string;
                read_only: boolean;
                readOnly: boolean;
                size: number;
                symlinks: string[];
                userspace_mount_options: string[];
                userspaceMountOptions: string[];
            }
        }

        export interface BlockNamespace {
            $gtype: GObject.GType<Block>;
            prototype: Block;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksBlock interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Block extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Configuration">"Configuration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get configuration(): GLib.Variant;
            set configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.CryptoBackingDevice">"CryptoBackingDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get crypto_backing_device(): string;
            set crypto_backing_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.CryptoBackingDevice">"CryptoBackingDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cryptoBackingDevice(): string;
            set cryptoBackingDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Device">"Device"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get device(): string;
            set device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.DeviceNumber">"DeviceNumber"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get device_number(): number;
            set device_number(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.DeviceNumber">"DeviceNumber"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get deviceNumber(): number;
            set deviceNumber(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Drive">"Drive"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get drive(): string;
            set drive(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintAuto">"HintAuto"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_auto(): boolean;
            set hint_auto(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintAuto">"HintAuto"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintAuto(): boolean;
            set hintAuto(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIconName">"HintIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_icon_name(): string;
            set hint_icon_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIconName">"HintIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintIconName(): string;
            set hintIconName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIgnore">"HintIgnore"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_ignore(): boolean;
            set hint_ignore(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintIgnore">"HintIgnore"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintIgnore(): boolean;
            set hintIgnore(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">"HintName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_name(): string;
            set hint_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintName">"HintName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintName(): string;
            set hintName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintPartitionable">"HintPartitionable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_partitionable(): boolean;
            set hint_partitionable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintPartitionable">"HintPartitionable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintPartitionable(): boolean;
            set hintPartitionable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSymbolicIconName">"HintSymbolicIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_symbolic_icon_name(): string;
            set hint_symbolic_icon_name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSymbolicIconName">"HintSymbolicIconName"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintSymbolicIconName(): string;
            set hintSymbolicIconName(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSystem">"HintSystem"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_system(): boolean;
            set hint_system(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.HintSystem">"HintSystem"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintSystem(): boolean;
            set hintSystem(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Id">"Id"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id(): string;
            set id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdLabel">"IdLabel"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_label(): string;
            set id_label(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdLabel">"IdLabel"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idLabel(): string;
            set idLabel(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdType">"IdType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_type(): string;
            set id_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdType">"IdType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idType(): string;
            set idType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUsage">"IdUsage"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_usage(): string;
            set id_usage(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUsage">"IdUsage"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idUsage(): string;
            set idUsage(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUUID">"IdUUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_uuid(): string;
            set id_uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdUUID">"IdUUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idUuid(): string;
            set idUuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdVersion">"IdVersion"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id_version(): string;
            set id_version(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.IdVersion">"IdVersion"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get idVersion(): string;
            set idVersion(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaid">"MDRaid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraid(): string;
            set mdraid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaidMember">"MDRaidMember"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraid_member(): string;
            set mdraid_member(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.MDRaidMember">"MDRaidMember"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mdraidMember(): string;
            set mdraidMember(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.PreferredDevice">"PreferredDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get preferred_device(): string;
            set preferred_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.PreferredDevice">"PreferredDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get preferredDevice(): string;
            set preferredDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.ReadOnly">"ReadOnly"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_only(): boolean;
            set read_only(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.ReadOnly">"ReadOnly"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readOnly(): boolean;
            set readOnly(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.Symlinks">"Symlinks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get symlinks(): string[];
            set symlinks(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.UserspaceMountOptions">"UserspaceMountOptions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get userspace_mount_options(): string[];
            set userspace_mount_options(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block.UserspaceMountOptions">"UserspaceMountOptions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get userspaceMountOptions(): string[];
            set userspaceMountOptions(val: string[]);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_add_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_add_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_add_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_add_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_add_configuration_item() for the asynchronous version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_configuration_item_sync(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_format_finish() to get the result of the operation.
             *
             * See udisks_block_call_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_format() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_get_secret_configuration_finish() to get the result of the operation.
             *
             * See udisks_block_call_get_secret_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_secret_configuration(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_block_call_get_secret_configuration().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_get_secret_configuration().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_secret_configuration_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_get_secret_configuration() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_secret_configuration_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_device_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_device_sync() for the synchronous, blocking version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_device(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_device_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_device() for the asynchronous version of this method.
             * @param arg_mode Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_device_sync(
                arg_mode: string,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_backup_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_backup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_backup(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_backup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_backup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_backup_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_backup() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_backup_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_benchmark_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_benchmark_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_benchmark(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_benchmark().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_benchmark().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_benchmark_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_benchmark() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_benchmark_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_open_for_restore_finish() to get the result of the operation.
             *
             * See udisks_block_call_open_for_restore_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_open_for_restore(
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_block_call_open_for_restore().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_open_for_restore().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_restore_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_open_for_restore() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_open_for_restore_sync(
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_remove_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_remove_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_configuration_item(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_remove_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_remove_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_remove_configuration_item() for the asynchronous version of this method.
             * @param arg_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_configuration_item_sync(
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rescan(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rescan(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_rescan_finish() to get the result of the operation.
             *
             * See udisks_block_call_rescan_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rescan(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_rescan().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_rescan().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rescan_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_rescan() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rescan_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_block_call_update_configuration_item_finish() to get the result of the operation.
             *
             * See udisks_block_call_update_configuration_item_sync() for the synchronous, blocking version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_update_configuration_item(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_block_call_update_configuration_item().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_block_call_update_configuration_item().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_update_configuration_item_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_block_call_update_configuration_item() for the asynchronous version of this method.
             * @param arg_old_item Argument to pass with the method invocation.
             * @param arg_new_item Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_update_configuration_item_sync(
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.AddConfigurationItem">AddConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Format">Format()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_format(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.GetSecretConfiguration">GetSecretConfiguration()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param configuration Parameter to return.
             */
            complete_get_secret_configuration(invocation: Gio.DBusMethodInvocation, configuration: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenDevice">OpenDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_device(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBackup">OpenForBackup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_backup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForBenchmark">OpenForBenchmark()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_benchmark(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.OpenForRestore">OpenForRestore()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param fd Parameter to return.
             */
            complete_open_for_restore(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                fd: GLib.Variant,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.RemoveConfigurationItem">RemoveConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_configuration_item(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.Rescan">Rescan()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_rescan(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Block.UpdateConfigurationItem">UpdateConfigurationItem()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_update_configuration_item(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksBlock::handle-add-configuration-item signal.
             * @param invocation
             * @param arg_item
             * @param arg_options
             */
            vfunc_handle_add_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-format signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_format(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-get-secret-configuration signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_secret_configuration(
                invocation: Gio.DBusMethodInvocation,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-device signal.
             * @param invocation
             * @param fd_list
             * @param arg_mode
             * @param arg_options
             */
            vfunc_handle_open_device(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_mode: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-backup signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_backup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-benchmark signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_benchmark(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-open-for-restore signal.
             * @param invocation
             * @param fd_list
             * @param arg_options
             */
            vfunc_handle_open_for_restore(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-remove-configuration-item signal.
             * @param invocation
             * @param arg_item
             * @param arg_options
             */
            vfunc_handle_remove_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksBlock::handle-rescan signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_rescan(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksBlock::handle-update-configuration-item signal.
             * @param invocation
             * @param arg_old_item
             * @param arg_new_item
             * @param arg_options
             */
            vfunc_handle_update_configuration_item(
                invocation: Gio.DBusMethodInvocation,
                arg_old_item: GLib.Variant,
                arg_new_item: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const Block: BlockNamespace & {
            new (): Block; // This allows `obj instanceof Block`
        };

        namespace BlockLVM2 {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                logical_volume: string;
                logicalVolume: string;
            }
        }

        export interface BlockLVM2Namespace {
            $gtype: GObject.GType<BlockLVM2>;
            prototype: BlockLVM2;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksBlockLVM2 interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface BlockLVM2 extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block-LVM2.LogicalVolume">"LogicalVolume"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get logical_volume(): string;
            set logical_volume(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Block-LVM2.LogicalVolume">"LogicalVolume"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get logicalVolume(): string;
            set logicalVolume(val: string);
        }

        export const BlockLVM2: BlockLVM2Namespace & {
            new (): BlockLVM2; // This allows `obj instanceof BlockLVM2`
        };

        namespace Drive {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                can_power_off: boolean;
                canPowerOff: boolean;
                configuration: GLib.Variant;
                connection_bus: string;
                connectionBus: string;
                ejectable: boolean;
                id: string;
                media: string;
                media_available: boolean;
                mediaAvailable: boolean;
                media_change_detected: boolean;
                mediaChangeDetected: boolean;
                media_compatibility: string[];
                mediaCompatibility: string[];
                media_removable: boolean;
                mediaRemovable: boolean;
                model: string;
                optical: boolean;
                optical_blank: boolean;
                opticalBlank: boolean;
                optical_num_audio_tracks: number;
                opticalNumAudioTracks: number;
                optical_num_data_tracks: number;
                opticalNumDataTracks: number;
                optical_num_sessions: number;
                opticalNumSessions: number;
                optical_num_tracks: number;
                opticalNumTracks: number;
                removable: boolean;
                revision: string;
                rotation_rate: number;
                rotationRate: number;
                seat: string;
                serial: string;
                sibling_id: string;
                siblingId: string;
                size: number;
                sort_key: string;
                sortKey: string;
                time_detected: number;
                timeDetected: number;
                time_media_detected: number;
                timeMediaDetected: number;
                vendor: string;
                wwn: string;
            }
        }

        export interface DriveNamespace {
            $gtype: GObject.GType<Drive>;
            prototype: Drive;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksDrive interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Drive extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.CanPowerOff">"CanPowerOff"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get can_power_off(): boolean;
            set can_power_off(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.CanPowerOff">"CanPowerOff"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get canPowerOff(): boolean;
            set canPowerOff(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Configuration">"Configuration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get configuration(): GLib.Variant;
            set configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.ConnectionBus">"ConnectionBus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get connection_bus(): string;
            set connection_bus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.ConnectionBus">"ConnectionBus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get connectionBus(): string;
            set connectionBus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Ejectable">"Ejectable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get ejectable(): boolean;
            set ejectable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Id">"Id"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get id(): string;
            set id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Media">"Media"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media(): string;
            set media(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaAvailable">"MediaAvailable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_available(): boolean;
            set media_available(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaAvailable">"MediaAvailable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaAvailable(): boolean;
            set mediaAvailable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaChangeDetected">"MediaChangeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_change_detected(): boolean;
            set media_change_detected(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaChangeDetected">"MediaChangeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaChangeDetected(): boolean;
            set mediaChangeDetected(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaCompatibility">"MediaCompatibility"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_compatibility(): string[];
            set media_compatibility(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaCompatibility">"MediaCompatibility"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaCompatibility(): string[];
            set mediaCompatibility(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaRemovable">"MediaRemovable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get media_removable(): boolean;
            set media_removable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.MediaRemovable">"MediaRemovable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mediaRemovable(): boolean;
            set mediaRemovable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Model">"Model"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get model(): string;
            set model(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Optical">"Optical"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical(): boolean;
            set optical(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalBlank">"OpticalBlank"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_blank(): boolean;
            set optical_blank(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalBlank">"OpticalBlank"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalBlank(): boolean;
            set opticalBlank(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumAudioTracks">"OpticalNumAudioTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_audio_tracks(): number;
            set optical_num_audio_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumAudioTracks">"OpticalNumAudioTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumAudioTracks(): number;
            set opticalNumAudioTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumDataTracks">"OpticalNumDataTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_data_tracks(): number;
            set optical_num_data_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumDataTracks">"OpticalNumDataTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumDataTracks(): number;
            set opticalNumDataTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumSessions">"OpticalNumSessions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_sessions(): number;
            set optical_num_sessions(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumSessions">"OpticalNumSessions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumSessions(): number;
            set opticalNumSessions(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumTracks">"OpticalNumTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get optical_num_tracks(): number;
            set optical_num_tracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.OpticalNumTracks">"OpticalNumTracks"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get opticalNumTracks(): number;
            set opticalNumTracks(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Removable">"Removable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get removable(): boolean;
            set removable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Revision">"Revision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get revision(): string;
            set revision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.RotationRate">"RotationRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rotation_rate(): number;
            set rotation_rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.RotationRate">"RotationRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rotationRate(): number;
            set rotationRate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Seat">"Seat"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get seat(): string;
            set seat(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Serial">"Serial"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get serial(): string;
            set serial(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SiblingId">"SiblingId"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sibling_id(): string;
            set sibling_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SiblingId">"SiblingId"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get siblingId(): string;
            set siblingId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SortKey">"SortKey"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sort_key(): string;
            set sort_key(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.SortKey">"SortKey"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sortKey(): string;
            set sortKey(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeDetected">"TimeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get time_detected(): number;
            set time_detected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeDetected">"TimeDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get timeDetected(): number;
            set timeDetected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeMediaDetected">"TimeMediaDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get time_media_detected(): number;
            set time_media_detected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.TimeMediaDetected">"TimeMediaDetected"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get timeMediaDetected(): number;
            set timeMediaDetected(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.Vendor">"Vendor"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vendor(): string;
            set vendor(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive.WWN">"WWN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get wwn(): string;
            set wwn(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_eject(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_eject(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_eject_finish() to get the result of the operation.
             *
             * See udisks_drive_call_eject_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_eject(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_eject().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_eject().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_eject_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_eject() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_eject_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_power_off(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_power_off(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_power_off_finish() to get the result of the operation.
             *
             * See udisks_drive_call_power_off_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_power_off(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_power_off().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_power_off().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_power_off_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_power_off() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_power_off_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_call_set_configuration_finish() to get the result of the operation.
             *
             * See udisks_drive_call_set_configuration_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_configuration(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_call_set_configuration().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_call_set_configuration().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_configuration_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_call_set_configuration() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_configuration_sync(
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.Eject">Eject()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_eject(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.PowerOff">PowerOff()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_power_off(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive.SetConfiguration">SetConfiguration()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_configuration(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksDrive::handle-eject signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_eject(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDrive::handle-power-off signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_power_off(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDrive::handle-set-configuration signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_configuration(
                invocation: Gio.DBusMethodInvocation,
                arg_value: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const Drive: DriveNamespace & {
            new (): Drive; // This allows `obj instanceof Drive`
        };

        namespace DriveAta {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                aam_enabled: boolean;
                aamEnabled: boolean;
                aam_supported: boolean;
                aamSupported: boolean;
                aam_vendor_recommended_value: number;
                aamVendorRecommendedValue: number;
                apm_enabled: boolean;
                apmEnabled: boolean;
                apm_supported: boolean;
                apmSupported: boolean;
                pm_enabled: boolean;
                pmEnabled: boolean;
                pm_supported: boolean;
                pmSupported: boolean;
                read_lookahead_enabled: boolean;
                readLookaheadEnabled: boolean;
                read_lookahead_supported: boolean;
                readLookaheadSupported: boolean;
                security_enhanced_erase_unit_minutes: number;
                securityEnhancedEraseUnitMinutes: number;
                security_erase_unit_minutes: number;
                securityEraseUnitMinutes: number;
                security_frozen: boolean;
                securityFrozen: boolean;
                smart_enabled: boolean;
                smartEnabled: boolean;
                smart_failing: boolean;
                smartFailing: boolean;
                smart_num_attributes_failed_in_the_past: number;
                smartNumAttributesFailedInThePast: number;
                smart_num_attributes_failing: number;
                smartNumAttributesFailing: number;
                smart_num_bad_sectors: number;
                smartNumBadSectors: number;
                smart_power_on_seconds: number;
                smartPowerOnSeconds: number;
                smart_selftest_percent_remaining: number;
                smartSelftestPercentRemaining: number;
                smart_selftest_status: string;
                smartSelftestStatus: string;
                smart_supported: boolean;
                smartSupported: boolean;
                smart_temperature: number;
                smartTemperature: number;
                smart_updated: number;
                smartUpdated: number;
                write_cache_enabled: boolean;
                writeCacheEnabled: boolean;
                write_cache_supported: boolean;
                writeCacheSupported: boolean;
            }
        }

        export interface DriveAtaNamespace {
            $gtype: GObject.GType<DriveAta>;
            prototype: DriveAta;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksDriveAta interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface DriveAta extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamEnabled">"AamEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_enabled(): boolean;
            set aam_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamEnabled">"AamEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamEnabled(): boolean;
            set aamEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamSupported">"AamSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_supported(): boolean;
            set aam_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamSupported">"AamSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamSupported(): boolean;
            set aamSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamVendorRecommendedValue">"AamVendorRecommendedValue"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aam_vendor_recommended_value(): number;
            set aam_vendor_recommended_value(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.AamVendorRecommendedValue">"AamVendorRecommendedValue"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get aamVendorRecommendedValue(): number;
            set aamVendorRecommendedValue(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmEnabled">"ApmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apm_enabled(): boolean;
            set apm_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmEnabled">"ApmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apmEnabled(): boolean;
            set apmEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmSupported">"ApmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apm_supported(): boolean;
            set apm_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ApmSupported">"ApmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get apmSupported(): boolean;
            set apmSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmEnabled">"PmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pm_enabled(): boolean;
            set pm_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmEnabled">"PmEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pmEnabled(): boolean;
            set pmEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmSupported">"PmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pm_supported(): boolean;
            set pm_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.PmSupported">"PmSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get pmSupported(): boolean;
            set pmSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadEnabled">"ReadLookaheadEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_lookahead_enabled(): boolean;
            set read_lookahead_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadEnabled">"ReadLookaheadEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readLookaheadEnabled(): boolean;
            set readLookaheadEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadSupported">"ReadLookaheadSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get read_lookahead_supported(): boolean;
            set read_lookahead_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.ReadLookaheadSupported">"ReadLookaheadSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get readLookaheadSupported(): boolean;
            set readLookaheadSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEnhancedEraseUnitMinutes">"SecurityEnhancedEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_enhanced_erase_unit_minutes(): number;
            set security_enhanced_erase_unit_minutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEnhancedEraseUnitMinutes">"SecurityEnhancedEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityEnhancedEraseUnitMinutes(): number;
            set securityEnhancedEraseUnitMinutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnitMinutes">"SecurityEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_erase_unit_minutes(): number;
            set security_erase_unit_minutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnitMinutes">"SecurityEraseUnitMinutes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityEraseUnitMinutes(): number;
            set securityEraseUnitMinutes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityFrozen">"SecurityFrozen"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get security_frozen(): boolean;
            set security_frozen(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SecurityFrozen">"SecurityFrozen"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get securityFrozen(): boolean;
            set securityFrozen(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartEnabled">"SmartEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_enabled(): boolean;
            set smart_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartEnabled">"SmartEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartEnabled(): boolean;
            set smartEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartFailing">"SmartFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_failing(): boolean;
            set smart_failing(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartFailing">"SmartFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartFailing(): boolean;
            set smartFailing(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailedInThePast">"SmartNumAttributesFailedInThePast"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_attributes_failed_in_the_past(): number;
            set smart_num_attributes_failed_in_the_past(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailedInThePast">"SmartNumAttributesFailedInThePast"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumAttributesFailedInThePast(): number;
            set smartNumAttributesFailedInThePast(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailing">"SmartNumAttributesFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_attributes_failing(): number;
            set smart_num_attributes_failing(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumAttributesFailing">"SmartNumAttributesFailing"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumAttributesFailing(): number;
            set smartNumAttributesFailing(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumBadSectors">"SmartNumBadSectors"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_num_bad_sectors(): number;
            set smart_num_bad_sectors(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartNumBadSectors">"SmartNumBadSectors"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartNumBadSectors(): number;
            set smartNumBadSectors(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartPowerOnSeconds">"SmartPowerOnSeconds"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_power_on_seconds(): number;
            set smart_power_on_seconds(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartPowerOnSeconds">"SmartPowerOnSeconds"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartPowerOnSeconds(): number;
            set smartPowerOnSeconds(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_percent_remaining(): number;
            set smart_selftest_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestPercentRemaining(): number;
            set smartSelftestPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_status(): string;
            set smart_selftest_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestStatus(): string;
            set smartSelftestStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSupported">"SmartSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_supported(): boolean;
            set smart_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartSupported">"SmartSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSupported(): boolean;
            set smartSupported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_temperature(): number;
            set smart_temperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartTemperature(): number;
            set smartTemperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_updated(): number;
            set smart_updated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartUpdated(): number;
            set smartUpdated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheEnabled">"WriteCacheEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get write_cache_enabled(): boolean;
            set write_cache_enabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheEnabled">"WriteCacheEnabled"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get writeCacheEnabled(): boolean;
            set writeCacheEnabled(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheSupported">"WriteCacheSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get write_cache_supported(): boolean;
            set write_cache_supported(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Drive-Ata.WriteCacheSupported">"WriteCacheSupported"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get writeCacheSupported(): boolean;
            set writeCacheSupported(val: boolean);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_get_state(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<number>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_get_state(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_get_state_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_get_state_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_get_state(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<number> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_get_state().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_get_state().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_get_state_finish(res: Gio.AsyncResult): [boolean, number];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_get_state() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_get_state_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, number];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_standby(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_standby(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_standby_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_standby_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_standby(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_standby().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_standby().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_standby_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_standby() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_standby_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_pm_wakeup(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_wakeup(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_pm_wakeup_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_pm_wakeup_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_pm_wakeup(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_pm_wakeup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_pm_wakeup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_wakeup_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_pm_wakeup() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_pm_wakeup_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_security_erase_unit(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_security_erase_unit(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_security_erase_unit_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_security_erase_unit_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_security_erase_unit(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_security_erase_unit().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_security_erase_unit().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_security_erase_unit_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_security_erase_unit() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_security_erase_unit_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_get_attributes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_get_attributes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_get_attributes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_selftest_abort().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_selftest_abort().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_selftest_abort() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_selftest_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_selftest_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_selftest_start() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_set_enabled_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_set_enabled_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_set_enabled(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_set_enabled().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_set_enabled().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_set_enabled_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_set_enabled() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_set_enabled_sync(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_update(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_drive_ata_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_drive_ata_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_drive_ata_call_smart_update().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_drive_ata_call_smart_update().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_drive_ata_call_smart_update() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmGetState">PmGetState()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param state Parameter to return.
             */
            complete_pm_get_state(invocation: Gio.DBusMethodInvocation, state: number): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmStandby">PmStandby()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_pm_standby(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.PmWakeup">PmWakeup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_pm_wakeup(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SecurityEraseUnit">SecurityEraseUnit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_security_erase_unit(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param attributes Parameter to return.
             */
            complete_smart_get_attributes(invocation: Gio.DBusMethodInvocation, attributes: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_abort(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartSetEnabled">SmartSetEnabled()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_set_enabled(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Drive-Ata.SmartUpdate">SmartUpdate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_update(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksDriveAta::handle-pm-get-state signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_get_state(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-standby signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_standby(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-pm-wakeup signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_pm_wakeup(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-security-erase-unit signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_security_erase_unit(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-get-attributes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_get_attributes(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-selftest-abort signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_selftest_abort(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-selftest-start signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_smart_selftest_start(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-set-enabled signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_smart_set_enabled(
                invocation: Gio.DBusMethodInvocation,
                arg_value: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksDriveAta::handle-smart-update signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_update(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const DriveAta: DriveAtaNamespace & {
            new (): DriveAta; // This allows `obj instanceof DriveAta`
        };

        namespace Encrypted {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                child_configuration: GLib.Variant;
                childConfiguration: GLib.Variant;
                cleartext_device: string;
                cleartextDevice: string;
                hint_encryption_type: string;
                hintEncryptionType: string;
                metadata_size: number;
                metadataSize: number;
            }
        }

        export interface EncryptedNamespace {
            $gtype: GObject.GType<Encrypted>;
            prototype: Encrypted;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksEncrypted interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Encrypted extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.CleartextDevice">"CleartextDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cleartext_device(): string;
            set cleartext_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.CleartextDevice">"CleartextDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cleartextDevice(): string;
            set cleartextDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.HintEncryptionType">"HintEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hint_encryption_type(): string;
            set hint_encryption_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.HintEncryptionType">"HintEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hintEncryptionType(): string;
            set hintEncryptionType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.MetadataSize">"MetadataSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadata_size(): number;
            set metadata_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Encrypted.MetadataSize">"MetadataSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadataSize(): number;
            set metadataSize(val: number);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_change_passphrase_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_change_passphrase_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_change_passphrase(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_change_passphrase().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_change_passphrase().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_change_passphrase_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_change_passphrase() for the asynchronous version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_new_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_change_passphrase_sync(
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_lock(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_lock(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_lock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_lock_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_lock(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_lock().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_lock().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_lock_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_lock() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_lock_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_resize_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_encrypted_call_unlock_finish() to get the result of the operation.
             *
             * See udisks_encrypted_call_unlock_sync() for the synchronous, blocking version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unlock(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_encrypted_call_unlock().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_encrypted_call_unlock().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unlock_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_encrypted_call_unlock() for the asynchronous version of this method.
             * @param arg_passphrase Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unlock_sync(
                arg_passphrase: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.ChangePassphrase">ChangePassphrase()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_change_passphrase(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Lock">Lock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_lock(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Encrypted.Unlock">Unlock()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param cleartext_device Parameter to return.
             */
            complete_unlock(invocation: Gio.DBusMethodInvocation, cleartext_device: string): void;

            // Virtual methods

            /**
             * Handler for the #UDisksEncrypted::handle-change-passphrase signal.
             * @param invocation
             * @param arg_passphrase
             * @param arg_new_passphrase
             * @param arg_options
             */
            vfunc_handle_change_passphrase(
                invocation: Gio.DBusMethodInvocation,
                arg_passphrase: string,
                arg_new_passphrase: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-lock signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_lock(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksEncrypted::handle-unlock signal.
             * @param invocation
             * @param arg_passphrase
             * @param arg_options
             */
            vfunc_handle_unlock(
                invocation: Gio.DBusMethodInvocation,
                arg_passphrase: string,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const Encrypted: EncryptedNamespace & {
            new (): Encrypted; // This allows `obj instanceof Encrypted`
        };

        namespace Filesystem {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                mount_points: string[];
                mountPoints: string[];
                size: number;
            }
        }

        export interface FilesystemNamespace {
            $gtype: GObject.GType<Filesystem>;
            prototype: Filesystem;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksFilesystem interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Filesystem extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.MountPoints">"MountPoints"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mount_points(): string[];
            set mount_points(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.MountPoints">"MountPoints"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get mountPoints(): string[];
            set mountPoints(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_check(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_check(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_check_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_check_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_check(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_check().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_check().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_check_finish(res: Gio.AsyncResult): [boolean, boolean];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_check() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_check_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, boolean];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_mount(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mount(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_mount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_mount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mount(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_mount().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_mount().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mount_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_mount() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mount_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): [boolean, boolean];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_repair() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, boolean];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_take_ownership(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_take_ownership(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_take_ownership_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_take_ownership_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_take_ownership(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_take_ownership().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_take_ownership().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_take_ownership_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_take_ownership() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_take_ownership_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_unmount(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unmount(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_call_unmount_finish() to get the result of the operation.
             *
             * See udisks_filesystem_call_unmount_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_unmount(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_call_unmount().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_call_unmount().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unmount_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_call_unmount() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_unmount_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Check">Check()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param consistent Parameter to return.
             */
            complete_check(invocation: Gio.DBusMethodInvocation, consistent: boolean): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Mount">Mount()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param mount_path Parameter to return.
             */
            complete_mount(invocation: Gio.DBusMethodInvocation, mount_path: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param repaired Parameter to return.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation, repaired: boolean): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.TakeOwnership">TakeOwnership()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_take_ownership(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem.Unmount">Unmount()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_unmount(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksFilesystem::handle-check signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_check(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-mount signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_mount(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-repair signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_repair(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-take-ownership signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_take_ownership(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystem::handle-unmount signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_unmount(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const Filesystem: FilesystemNamespace & {
            new (): Filesystem; // This allows `obj instanceof Filesystem`
        };

        namespace FilesystemBTRFS {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                label: string;
                num_devices: number;
                numDevices: number;
                used: number;
                uuid: string;
            }
        }

        export interface FilesystemBTRFSNamespace {
            $gtype: GObject.GType<FilesystemBTRFS>;
            prototype: FilesystemBTRFS;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksFilesystemBTRFS interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface FilesystemBTRFS extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.label">"label"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get label(): string;
            set label(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.num_devices">"num_devices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get num_devices(): number;
            set num_devices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.num_devices">"num_devices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get numDevices(): number;
            set numDevices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.used">"used"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get used(): number;
            set used(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Filesystem-BTRFS.uuid">"uuid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_add_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_create_snapshot().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_create_snapshot().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_create_snapshot() for the asynchronous version of this method.
             * @param arg_source Argument to pass with the method invocation.
             * @param arg_dest Argument to pass with the method invocation.
             * @param arg_ro Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_sync(
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_create_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_create_subvolume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_create_subvolume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_subvolume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_create_subvolume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_subvolume_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[GLib.Variant | null, number]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_get_subvolumes_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes_sync() for the synchronous, blocking version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_subvolumes(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[GLib.Variant | null, number]> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_get_subvolumes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_get_subvolumes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_subvolumes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null, number];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_get_subvolumes() for the asynchronous version of this method.
             * @param arg_snapshots_only Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_subvolumes_sync(
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null, number];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_remove_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_remove_subvolume_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_subvolume(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_remove_subvolume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_remove_subvolume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_subvolume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_remove_subvolume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_subvolume_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_repair_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_repair() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_resize_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_filesystem_btrfs_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_filesystem_btrfs_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_filesystem_btrfs_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_filesystem_btrfs_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_filesystem_btrfs_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSnapshot">CreateSnapshot()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_snapshot(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.CreateSubvolume">CreateSubvolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_subvolume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.GetSubvolumes">GetSubvolumes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param subvolumes Parameter to return.
             * @param subvolumes_cnt Parameter to return.
             */
            complete_get_subvolumes(
                invocation: Gio.DBusMethodInvocation,
                subvolumes: GLib.Variant,
                subvolumes_cnt: number,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.RemoveSubvolume">RemoveSubvolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_subvolume(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Filesystem-BTRFS.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-add-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-create-snapshot signal.
             * @param invocation
             * @param arg_source
             * @param arg_dest
             * @param arg_ro
             * @param arg_options
             */
            vfunc_handle_create_snapshot(
                invocation: Gio.DBusMethodInvocation,
                arg_source: string,
                arg_dest: string,
                arg_ro: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-create-subvolume signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_create_subvolume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-get-subvolumes signal.
             * @param invocation
             * @param arg_snapshots_only
             * @param arg_options
             */
            vfunc_handle_get_subvolumes(
                invocation: Gio.DBusMethodInvocation,
                arg_snapshots_only: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-remove-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-remove-subvolume signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_remove_subvolume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-repair signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_repair(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksFilesystemBTRFS::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const FilesystemBTRFS: FilesystemBTRFSNamespace & {
            new (): FilesystemBTRFS; // This allows `obj instanceof FilesystemBTRFS`
        };

        namespace Job {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                bytes: number;
                cancelable: boolean;
                expected_end_time: number;
                expectedEndTime: number;
                objects: string[];
                operation: string;
                progress: number;
                progress_valid: boolean;
                progressValid: boolean;
                rate: number;
                start_time: number;
                startTime: number;
                started_by_uid: number;
                startedByUid: number;
            }
        }

        export interface JobNamespace {
            $gtype: GObject.GType<Job>;
            prototype: Job;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksJob interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Job extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Bytes">"Bytes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bytes(): number;
            set bytes(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Cancelable">"Cancelable"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get cancelable(): boolean;
            set cancelable(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ExpectedEndTime">"ExpectedEndTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get expected_end_time(): number;
            set expected_end_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ExpectedEndTime">"ExpectedEndTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get expectedEndTime(): number;
            set expectedEndTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Objects">"Objects"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get objects(): string[];
            set objects(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Operation">"Operation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operation(): string;
            set operation(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Progress">"Progress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progress(): number;
            set progress(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ProgressValid">"ProgressValid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progress_valid(): boolean;
            set progress_valid(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.ProgressValid">"ProgressValid"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get progressValid(): boolean;
            set progressValid(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.Rate">"Rate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get rate(): number;
            set rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartTime">"StartTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get start_time(): number;
            set start_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartTime">"StartTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get startTime(): number;
            set startTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartedByUID">"StartedByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get started_by_uid(): number;
            set started_by_uid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Job.StartedByUID">"StartedByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get startedByUid(): number;
            set startedByUid(val: number);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cancel(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cancel(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_job_call_cancel_finish() to get the result of the operation.
             *
             * See udisks_job_call_cancel_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cancel(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_job_call_cancel().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_job_call_cancel().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cancel_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_job_call_cancel() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cancel_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Job.Cancel">Cancel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cancel(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Emits the <link linkend="gdbus-signal-org-freedesktop-UDisks2-Job.Completed">"Completed"</link> D-Bus signal.
             * @param arg_success Argument to pass with the signal.
             * @param arg_message Argument to pass with the signal.
             */
            emit_completed(arg_success: boolean, arg_message: string): void;

            // Virtual methods

            /**
             * Handler for the #UDisksJob::completed signal.
             * @param arg_success
             * @param arg_message
             */
            vfunc_completed(arg_success: boolean, arg_message: string): void;
            /**
             * Handler for the #UDisksJob::handle-cancel signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cancel(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const Job: JobNamespace & {
            new (): Job; // This allows `obj instanceof Job`
        };

        namespace LogicalVolume {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                active: boolean;
                block_device: string;
                blockDevice: string;
                child_configuration: GLib.Variant;
                childConfiguration: GLib.Variant;
                data_allocated_ratio: number;
                dataAllocatedRatio: number;
                layout: string;
                metadata_allocated_ratio: number;
                metadataAllocatedRatio: number;
                name: string;
                origin: string;
                size: number;
                structure: GLib.Variant;
                sync_ratio: number;
                syncRatio: number;
                thin_pool: string;
                thinPool: string;
                type: string;
                uuid: string;
                volume_group: string;
                volumeGroup: string;
            }
        }

        export interface LogicalVolumeNamespace {
            $gtype: GObject.GType<LogicalVolume>;
            prototype: LogicalVolume;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksLogicalVolume interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface LogicalVolume extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Active">"Active"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active(): boolean;
            set active(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.BlockDevice">"BlockDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get block_device(): string;
            set block_device(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.BlockDevice">"BlockDevice"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get blockDevice(): string;
            set blockDevice(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.DataAllocatedRatio">"DataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get data_allocated_ratio(): number;
            set data_allocated_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.DataAllocatedRatio">"DataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get dataAllocatedRatio(): number;
            set dataAllocatedRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Layout">"Layout"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get layout(): string;
            set layout(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.MetadataAllocatedRatio">"MetadataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadata_allocated_ratio(): number;
            set metadata_allocated_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.MetadataAllocatedRatio">"MetadataAllocatedRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get metadataAllocatedRatio(): number;
            set metadataAllocatedRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Origin">"Origin"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get origin(): string;
            set origin(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Structure">"Structure"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get structure(): GLib.Variant;
            set structure(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.SyncRatio">"SyncRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_ratio(): number;
            set sync_ratio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.SyncRatio">"SyncRatio"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRatio(): number;
            set syncRatio(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ThinPool">"ThinPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get thin_pool(): string;
            set thin_pool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.ThinPool">"ThinPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get thinPool(): string;
            set thinPool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volume_group(): string;
            set volume_group(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-LogicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volumeGroup(): string;
            set volumeGroup(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_activate(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_activate(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_activate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_activate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_activate(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_activate().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_activate().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_activate_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_activate() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_activate_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_attach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_attach_sync() for the synchronous, blocking version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_attach(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_attach().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_attach().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_attach_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_attach() for the asynchronous version of this method.
             * @param arg_cache_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_attach_sync(
                arg_cache_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_detach(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_detach(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_detach_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_detach_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_detach(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_detach().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_detach().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_detach_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_detach() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_detach_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_cache_split(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_split(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_cache_split_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_cache_split_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_cache_split(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_cache_split().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_cache_split().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_split_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_cache_split() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_cache_split_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_create_snapshot_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_create_snapshot_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_snapshot(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_create_snapshot().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_create_snapshot().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_create_snapshot() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_snapshot_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_deactivate(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_deactivate(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_deactivate_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_deactivate_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_deactivate(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_deactivate().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_deactivate().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_deactivate_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_deactivate() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_deactivate_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_delete_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_rename_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_rename().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_rename().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_rename() for the asynchronous version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_sync(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_repair_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_repair(
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_repair() for the asynchronous version of this method.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_repair_sync(arg_pvs: string, arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_logical_volume_call_resize_finish() to get the result of the operation.
             *
             * See udisks_logical_volume_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_logical_volume_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_logical_volume_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_logical_volume_call_resize() for the asynchronous version of this method.
             * @param arg_new_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_new_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Activate">Activate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_activate(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheAttach">CacheAttach()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_attach(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheDetach">CacheDetach()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_detach(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CacheSplit">CacheSplit()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_cache_split(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.CreateSnapshot">CreateSnapshot()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_snapshot(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Deactivate">Deactivate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_deactivate(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Rename">Rename()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_rename(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Repair">Repair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_repair(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-LogicalVolume.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksLogicalVolume::handle-activate signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_activate(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-attach signal.
             * @param invocation
             * @param arg_cache_name
             * @param arg_options
             */
            vfunc_handle_cache_attach(
                invocation: Gio.DBusMethodInvocation,
                arg_cache_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-detach signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cache_detach(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-cache-split signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_cache_split(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-create-snapshot signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_snapshot(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-deactivate signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_deactivate(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-rename signal.
             * @param invocation
             * @param arg_new_name
             * @param arg_options
             */
            vfunc_handle_rename(
                invocation: Gio.DBusMethodInvocation,
                arg_new_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-repair signal.
             * @param invocation
             * @param arg_pvs
             * @param arg_options
             */
            vfunc_handle_repair(
                invocation: Gio.DBusMethodInvocation,
                arg_pvs: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksLogicalVolume::handle-resize signal.
             * @param invocation
             * @param arg_new_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_new_size: number,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const LogicalVolume: LogicalVolumeNamespace & {
            new (): LogicalVolume; // This allows `obj instanceof LogicalVolume`
        };

        namespace Loop {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                autoclear: boolean;
                backing_file: string;
                backingFile: string;
                setup_by_uid: number;
                setupByUid: number;
            }
        }

        export interface LoopNamespace {
            $gtype: GObject.GType<Loop>;
            prototype: Loop;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksLoop interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Loop extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.Autoclear">"Autoclear"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get autoclear(): boolean;
            set autoclear(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.BackingFile">"BackingFile"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get backing_file(): string;
            set backing_file(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.BackingFile">"BackingFile"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get backingFile(): string;
            set backingFile(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.SetupByUID">"SetupByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get setup_by_uid(): number;
            set setup_by_uid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Loop.SetupByUID">"SetupByUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get setupByUid(): number;
            set setupByUid(val: number);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_delete_finish() to get the result of the operation.
             *
             * See udisks_loop_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_loop_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_loop_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_loop_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_loop_call_set_autoclear_finish() to get the result of the operation.
             *
             * See udisks_loop_call_set_autoclear_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_autoclear(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_loop_call_set_autoclear().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_loop_call_set_autoclear().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_autoclear_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_loop_call_set_autoclear() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_autoclear_sync(
                arg_value: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Loop.SetAutoclear">SetAutoclear()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_autoclear(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksLoop::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksLoop::handle-set-autoclear signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_autoclear(
                invocation: Gio.DBusMethodInvocation,
                arg_value: boolean,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const Loop: LoopNamespace & {
            new (): Loop; // This allows `obj instanceof Loop`
        };

        namespace MDRaid {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                active_devices: GLib.Variant;
                activeDevices: GLib.Variant;
                bitmap_location: string;
                bitmapLocation: string;
                child_configuration: GLib.Variant;
                childConfiguration: GLib.Variant;
                chunk_size: number;
                chunkSize: number;
                degraded: number;
                level: string;
                name: string;
                num_devices: number;
                numDevices: number;
                running: boolean;
                size: number;
                sync_action: string;
                syncAction: string;
                sync_completed: number;
                syncCompleted: number;
                sync_rate: number;
                syncRate: number;
                sync_remaining_time: number;
                syncRemainingTime: number;
                uuid: string;
            }
        }

        export interface MDRaidNamespace {
            $gtype: GObject.GType<MDRaid>;
            prototype: MDRaid;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksMDRaid interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface MDRaid extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ActiveDevices">"ActiveDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active_devices(): GLib.Variant;
            set active_devices(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ActiveDevices">"ActiveDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get activeDevices(): GLib.Variant;
            set activeDevices(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.BitmapLocation">"BitmapLocation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bitmap_location(): string;
            set bitmap_location(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.BitmapLocation">"BitmapLocation"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get bitmapLocation(): string;
            set bitmapLocation(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get child_configuration(): GLib.Variant;
            set child_configuration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChildConfiguration">"ChildConfiguration"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get childConfiguration(): GLib.Variant;
            set childConfiguration(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChunkSize">"ChunkSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get chunk_size(): number;
            set chunk_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.ChunkSize">"ChunkSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get chunkSize(): number;
            set chunkSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Degraded">"Degraded"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get degraded(): number;
            set degraded(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Level">"Level"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get level(): string;
            set level(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.NumDevices">"NumDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get num_devices(): number;
            set num_devices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.NumDevices">"NumDevices"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get numDevices(): number;
            set numDevices(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Running">"Running"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get running(): boolean;
            set running(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncAction">"SyncAction"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_action(): string;
            set sync_action(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncAction">"SyncAction"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncAction(): string;
            set syncAction(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncCompleted">"SyncCompleted"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_completed(): number;
            set sync_completed(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncCompleted">"SyncCompleted"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncCompleted(): number;
            set syncCompleted(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRate">"SyncRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_rate(): number;
            set sync_rate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRate">"SyncRate"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRate(): number;
            set syncRate(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRemainingTime">"SyncRemainingTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sync_remaining_time(): number;
            set sync_remaining_time(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.SyncRemainingTime">"SyncRemainingTime"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get syncRemainingTime(): number;
            set syncRemainingTime(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-MDRaid.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_add_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_delete_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_remove_device() for the asynchronous version of this method.
             * @param arg_device Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_device: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_request_sync_action_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_request_sync_action_sync() for the synchronous, blocking version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_request_sync_action(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_request_sync_action().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_request_sync_action().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_request_sync_action_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_request_sync_action() for the asynchronous version of this method.
             * @param arg_sync_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_request_sync_action_sync(
                arg_sync_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_set_bitmap_location_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_set_bitmap_location_sync() for the synchronous, blocking version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_bitmap_location(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_set_bitmap_location().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_set_bitmap_location().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_bitmap_location_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_set_bitmap_location() for the asynchronous version of this method.
             * @param arg_value Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_bitmap_location_sync(
                arg_value: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_start(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_start_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_start() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_stop(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_mdraid_call_stop_finish() to get the result of the operation.
             *
             * See udisks_mdraid_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_mdraid_call_stop().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_mdraid_call_stop().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_mdraid_call_stop() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.RequestSyncAction">RequestSyncAction()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_request_sync_action(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.SetBitmapLocation">SetBitmapLocation()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_bitmap_location(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Start">Start()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-MDRaid.Stop">Stop()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_stop(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksMDRaid::handle-add-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-remove-device signal.
             * @param invocation
             * @param arg_device
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_device: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-request-sync-action signal.
             * @param invocation
             * @param arg_sync_action
             * @param arg_options
             */
            vfunc_handle_request_sync_action(
                invocation: Gio.DBusMethodInvocation,
                arg_sync_action: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-set-bitmap-location signal.
             * @param invocation
             * @param arg_value
             * @param arg_options
             */
            vfunc_handle_set_bitmap_location(
                invocation: Gio.DBusMethodInvocation,
                arg_value: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-start signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_start(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksMDRaid::handle-stop signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_stop(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const MDRaid: MDRaidNamespace & {
            new (): MDRaid; // This allows `obj instanceof MDRaid`
        };

        namespace Manager {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                default_encryption_type: string;
                defaultEncryptionType: string;
                supported_encryption_types: string[];
                supportedEncryptionTypes: string[];
                supported_filesystems: string[];
                supportedFilesystems: string[];
                version: string;
            }
        }

        export interface ManagerNamespace {
            $gtype: GObject.GType<Manager>;
            prototype: Manager;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksManager interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Manager extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.DefaultEncryptionType">"DefaultEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get default_encryption_type(): string;
            set default_encryption_type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.DefaultEncryptionType">"DefaultEncryptionType"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get defaultEncryptionType(): string;
            set defaultEncryptionType(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedEncryptionTypes">"SupportedEncryptionTypes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supported_encryption_types(): string[];
            set supported_encryption_types(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedEncryptionTypes">"SupportedEncryptionTypes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supportedEncryptionTypes(): string[];
            set supportedEncryptionTypes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedFilesystems">"SupportedFilesystems"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supported_filesystems(): string[];
            set supported_filesystems(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.SupportedFilesystems">"SupportedFilesystems"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get supportedFilesystems(): string[];
            set supportedFilesystems(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager.Version">"Version"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get version(): string;
            set version(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_check(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_check(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_check_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_check_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_check(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_check().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_check().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_check_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_check() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_check_sync(arg_type: string, cancellable?: Gio.Cancellable | null): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_format(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_format(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_format_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_format_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_format(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_format_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_format() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_format_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_repair(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_repair(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_repair_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_repair_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_repair(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_repair().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_repair().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_repair_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_repair() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_repair_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_can_resize(arg_type: string, cancellable?: Gio.Cancellable | null): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_resize(
                arg_type: string,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_can_resize_finish() to get the result of the operation.
             *
             * See udisks_manager_call_can_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_can_resize(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_can_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_can_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_resize_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_can_resize() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_can_resize_sync(
                arg_type: string,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_module_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_module_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_module(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_call_enable_module().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_enable_module().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_module_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_enable_module() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_module_sync(
                arg_name: string,
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_modules(arg_enable: boolean, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_modules(
                arg_enable: boolean,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_enable_modules_finish() to get the result of the operation.
             *
             * See udisks_manager_call_enable_modules_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_modules(
                arg_enable: boolean,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_call_enable_modules().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_enable_modules().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_modules_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_enable_modules() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_modules_sync(arg_enable: boolean, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string[] | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_get_block_devices_finish() to get the result of the operation.
             *
             * See udisks_manager_call_get_block_devices_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_block_devices(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string[] | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_get_block_devices().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_get_block_devices().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_block_devices_finish(res: Gio.AsyncResult): [boolean, string[] | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_get_block_devices() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_block_devices_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string[] | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): Promise<[string, Gio.UnixFDList | null]>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_loop_setup_finish() to get the result of the operation.
             *
             * See udisks_manager_call_loop_setup_sync() for the synchronous, blocking version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_loop_setup(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list?: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<[string, Gio.UnixFDList | null]> | void;
            /**
             * Finishes an operation started with udisks_manager_call_loop_setup().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_loop_setup().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_loop_setup_finish(res: Gio.AsyncResult): [boolean, string, Gio.UnixFDList | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_loop_setup() for the asynchronous version of this method.
             * @param arg_fd Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_loop_setup_sync(
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
                fd_list: Gio.UnixFDList | null,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string, Gio.UnixFDList];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_mdraid_create_finish() to get the result of the operation.
             *
             * See udisks_manager_call_mdraid_create_sync() for the synchronous, blocking version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_mdraid_create(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_call_mdraid_create().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_mdraid_create().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mdraid_create_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_mdraid_create() for the asynchronous version of this method.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_level Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_chunk Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_mdraid_create_sync(
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string[] | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_call_resolve_device_finish() to get the result of the operation.
             *
             * See udisks_manager_call_resolve_device_sync() for the synchronous, blocking version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resolve_device(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string[] | null> | void;
            /**
             * Finishes an operation started with udisks_manager_call_resolve_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_call_resolve_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resolve_device_finish(res: Gio.AsyncResult): [boolean, string[] | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_call_resolve_device() for the asynchronous version of this method.
             * @param arg_devspec Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resolve_device_sync(
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string[] | null];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanCheck">CanCheck()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_check(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanFormat">CanFormat()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_format(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanRepair">CanRepair()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_repair(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.CanResize">CanResize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param available Parameter to return.
             */
            complete_can_resize(invocation: Gio.DBusMethodInvocation, available: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModule">EnableModule()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_module(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.EnableModules">EnableModules()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_modules(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.GetBlockDevices">GetBlockDevices()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param block_objects Parameter to return.
             */
            complete_get_block_devices(invocation: Gio.DBusMethodInvocation, block_objects: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.LoopSetup">LoopSetup()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param fd_list A #GUnixFDList or %NULL.
             * @param resulting_device Parameter to return.
             */
            complete_loop_setup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList | null,
                resulting_device: string,
            ): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.MDRaidCreate">MDRaidCreate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param resulting_array Parameter to return.
             */
            complete_mdraid_create(invocation: Gio.DBusMethodInvocation, resulting_array: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager.ResolveDevice">ResolveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param devices Parameter to return.
             */
            complete_resolve_device(invocation: Gio.DBusMethodInvocation, devices: string): void;

            // Virtual methods

            /**
             * Handler for the #UDisksManager::handle-can-check signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_check(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-format signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_format(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-repair signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_repair(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-can-resize signal.
             * @param invocation
             * @param arg_type
             */
            vfunc_handle_can_resize(invocation: Gio.DBusMethodInvocation, arg_type: string): boolean;
            /**
             * Handler for the #UDisksManager::handle-enable-module signal.
             * @param invocation
             * @param arg_name
             * @param arg_enable
             */
            vfunc_handle_enable_module(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_enable: boolean,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-enable-modules signal.
             * @param invocation
             * @param arg_enable
             */
            vfunc_handle_enable_modules(invocation: Gio.DBusMethodInvocation, arg_enable: boolean): boolean;
            /**
             * Handler for the #UDisksManager::handle-get-block-devices signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_block_devices(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksManager::handle-loop-setup signal.
             * @param invocation
             * @param fd_list
             * @param arg_fd
             * @param arg_options
             */
            vfunc_handle_loop_setup(
                invocation: Gio.DBusMethodInvocation,
                fd_list: Gio.UnixFDList,
                arg_fd: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-mdraid-create signal.
             * @param invocation
             * @param arg_blocks
             * @param arg_level
             * @param arg_name
             * @param arg_chunk
             * @param arg_options
             */
            vfunc_handle_mdraid_create(
                invocation: Gio.DBusMethodInvocation,
                arg_blocks: string,
                arg_level: string,
                arg_name: string,
                arg_chunk: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManager::handle-resolve-device signal.
             * @param invocation
             * @param arg_devspec
             * @param arg_options
             */
            vfunc_handle_resolve_device(
                invocation: Gio.DBusMethodInvocation,
                arg_devspec: GLib.Variant,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const Manager: ManagerNamespace & {
            new (): Manager; // This allows `obj instanceof Manager`
        };

        namespace ManagerBTRFS {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {}
        }

        export interface ManagerBTRFSNamespace {
            $gtype: GObject.GType<ManagerBTRFS>;
            prototype: ManagerBTRFS;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksManagerBTRFS interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface ManagerBTRFS extends GObject.Object {
            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_btrfs_call_create_volume_finish() to get the result of the operation.
             *
             * See udisks_manager_btrfs_call_create_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_volume(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_btrfs_call_create_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_btrfs_call_create_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_volume_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_btrfs_call_create_volume() for the asynchronous version of this method.
             * @param arg_devices Argument to pass with the method invocation.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_data_level Argument to pass with the method invocation.
             * @param arg_md_level Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_volume_sync(
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-BTRFS.CreateVolume">CreateVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_create_volume(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksManagerBTRFS::handle-create-volume signal.
             * @param invocation
             * @param arg_devices
             * @param arg_label
             * @param arg_data_level
             * @param arg_md_level
             * @param arg_options
             */
            vfunc_handle_create_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_devices: string,
                arg_label: string,
                arg_data_level: string,
                arg_md_level: string,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const ManagerBTRFS: ManagerBTRFSNamespace & {
            new (): ManagerBTRFS; // This allows `obj instanceof ManagerBTRFS`
        };

        namespace ManagerLVM2 {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {}
        }

        export interface ManagerLVM2Namespace {
            $gtype: GObject.GType<ManagerLVM2>;
            prototype: ManagerLVM2;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksManagerLVM2 interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface ManagerLVM2 extends GObject.Object {
            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_lvm2_call_volume_group_create_finish() to get the result of the operation.
             *
             * See udisks_manager_lvm2_call_volume_group_create_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_volume_group_create(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_lvm2_call_volume_group_create().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_lvm2_call_volume_group_create().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_volume_group_create_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_lvm2_call_volume_group_create() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_blocks Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_volume_group_create_sync(
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-LVM2.VolumeGroupCreate">VolumeGroupCreate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_volume_group_create(invocation: Gio.DBusMethodInvocation, result: string): void;

            // Virtual methods

            /**
             * Handler for the #UDisksManagerLVM2::handle-volume-group-create signal.
             * @param invocation
             * @param arg_name
             * @param arg_blocks
             * @param arg_options
             */
            vfunc_handle_volume_group_create(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_blocks: string,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const ManagerLVM2: ManagerLVM2Namespace & {
            new (): ManagerLVM2; // This allows `obj instanceof ManagerLVM2`
        };

        namespace ManagerNVMe {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                host_id: string;
                hostId: string;
                host_nqn: string;
                hostNqn: string;
            }
        }

        export interface ManagerNVMeNamespace {
            $gtype: GObject.GType<ManagerNVMe>;
            prototype: ManagerNVMe;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksManagerNVMe interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface ManagerNVMe extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_id(): string;
            set host_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostId(): string;
            set hostId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_nqn(): string;
            set host_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Manager-NVMe.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostNqn(): string;
            set hostNqn(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_connect_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_connect_sync() for the synchronous, blocking version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_connect(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_connect().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_connect().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_connect_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_connect() for the asynchronous version of this method.
             * @param arg_subsysnqn Argument to pass with the method invocation.
             * @param arg_transport Argument to pass with the method invocation.
             * @param arg_transport_addr Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_connect_sync(
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_id_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_id_sync() for the synchronous, blocking version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_id(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_set_host_id().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_set_host_id().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_id_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_set_host_id() for the asynchronous version of this method.
             * @param arg_hostid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_id_sync(
                arg_hostid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_manager_nvme_call_set_host_nqn_finish() to get the result of the operation.
             *
             * See udisks_manager_nvme_call_set_host_nqn_sync() for the synchronous, blocking version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_host_nqn(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_manager_nvme_call_set_host_nqn().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_manager_nvme_call_set_host_nqn().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_nqn_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_manager_nvme_call_set_host_nqn() for the asynchronous version of this method.
             * @param arg_hostnqn Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_host_nqn_sync(
                arg_hostnqn: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.Connect">Connect()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param nvme_ctrl Parameter to return.
             */
            complete_connect(invocation: Gio.DBusMethodInvocation, nvme_ctrl: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostID">SetHostID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_host_id(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Manager-NVMe.SetHostNQN">SetHostNQN()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_host_nqn(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksManagerNVMe::handle-connect signal.
             * @param invocation
             * @param arg_subsysnqn
             * @param arg_transport
             * @param arg_transport_addr
             * @param arg_options
             */
            vfunc_handle_connect(
                invocation: Gio.DBusMethodInvocation,
                arg_subsysnqn: string,
                arg_transport: string,
                arg_transport_addr: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManagerNVMe::handle-set-host-id signal.
             * @param invocation
             * @param arg_hostid
             * @param arg_options
             */
            vfunc_handle_set_host_id(
                invocation: Gio.DBusMethodInvocation,
                arg_hostid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksManagerNVMe::handle-set-host-nqn signal.
             * @param invocation
             * @param arg_hostnqn
             * @param arg_options
             */
            vfunc_handle_set_host_nqn(
                invocation: Gio.DBusMethodInvocation,
                arg_hostnqn: string,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const ManagerNVMe: ManagerNVMeNamespace & {
            new (): ManagerNVMe; // This allows `obj instanceof ManagerNVMe`
        };

        namespace NVMeController {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                controller_id: number;
                controllerId: number;
                fguid: string;
                nvme_revision: string;
                nvmeRevision: string;
                sanitize_percent_remaining: number;
                sanitizePercentRemaining: number;
                sanitize_status: string;
                sanitizeStatus: string;
                smart_critical_warning: string[];
                smartCriticalWarning: string[];
                smart_power_on_hours: number;
                smartPowerOnHours: number;
                smart_selftest_percent_remaining: number;
                smartSelftestPercentRemaining: number;
                smart_selftest_status: string;
                smartSelftestStatus: string;
                smart_temperature: number;
                smartTemperature: number;
                smart_updated: number;
                smartUpdated: number;
                state: string;
                subsystem_nqn: string;
                subsystemNqn: string;
                unallocated_capacity: number;
                unallocatedCapacity: number;
            }
        }

        export interface NVMeControllerNamespace {
            $gtype: GObject.GType<NVMeController>;
            prototype: NVMeController;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksNVMeController interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface NVMeController extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.ControllerID">"ControllerID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get controller_id(): number;
            set controller_id(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.ControllerID">"ControllerID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get controllerId(): number;
            set controllerId(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.FGUID">"FGUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get fguid(): string;
            set fguid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.NVMeRevision">"NVMeRevision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nvme_revision(): string;
            set nvme_revision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.NVMeRevision">"NVMeRevision"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nvmeRevision(): string;
            set nvmeRevision(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizePercentRemaining">"SanitizePercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitize_percent_remaining(): number;
            set sanitize_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizePercentRemaining">"SanitizePercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitizePercentRemaining(): number;
            set sanitizePercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStatus">"SanitizeStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitize_status(): string;
            set sanitize_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStatus">"SanitizeStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get sanitizeStatus(): string;
            set sanitizeStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartCriticalWarning">"SmartCriticalWarning"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_critical_warning(): string[];
            set smart_critical_warning(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartCriticalWarning">"SmartCriticalWarning"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartCriticalWarning(): string[];
            set smartCriticalWarning(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartPowerOnHours">"SmartPowerOnHours"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_power_on_hours(): number;
            set smart_power_on_hours(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartPowerOnHours">"SmartPowerOnHours"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartPowerOnHours(): number;
            set smartPowerOnHours(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_percent_remaining(): number;
            set smart_selftest_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestPercentRemaining">"SmartSelftestPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestPercentRemaining(): number;
            set smartSelftestPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_selftest_status(): string;
            set smart_selftest_status(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStatus">"SmartSelftestStatus"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartSelftestStatus(): string;
            set smartSelftestStatus(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_temperature(): number;
            set smart_temperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartTemperature">"SmartTemperature"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartTemperature(): number;
            set smartTemperature(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smart_updated(): number;
            set smart_updated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdated">"SmartUpdated"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get smartUpdated(): number;
            set smartUpdated(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.State">"State"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get state(): string;
            set state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SubsystemNQN">"SubsystemNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get subsystem_nqn(): string;
            set subsystem_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.SubsystemNQN">"SubsystemNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get subsystemNqn(): string;
            set subsystemNqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.UnallocatedCapacity">"UnallocatedCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get unallocated_capacity(): number;
            set unallocated_capacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Controller.UnallocatedCapacity">"UnallocatedCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get unallocatedCapacity(): number;
            set unallocatedCapacity(val: number);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_sanitize_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_sanitize_start_sync() for the synchronous, blocking version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_sanitize_start(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_sanitize_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_sanitize_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_sanitize_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_sanitize_start() for the asynchronous version of this method.
             * @param arg_action Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_sanitize_start_sync(
                arg_action: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_get_attributes_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_get_attributes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_get_attributes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_get_attributes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_get_attributes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_get_attributes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_get_attributes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_abort_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_abort(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_selftest_abort().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_selftest_abort().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_selftest_abort() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_abort_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_selftest_start_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_selftest_start_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_selftest_start(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_selftest_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_selftest_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_selftest_start() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_selftest_start_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_smart_update(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_controller_call_smart_update_finish() to get the result of the operation.
             *
             * See udisks_nvme_controller_call_smart_update_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_smart_update(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_controller_call_smart_update().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_controller_call_smart_update().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_controller_call_smart_update() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_smart_update_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SanitizeStart">SanitizeStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_sanitize_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartGetAttributes">SmartGetAttributes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param attributes Parameter to return.
             */
            complete_smart_get_attributes(invocation: Gio.DBusMethodInvocation, attributes: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestAbort">SmartSelftestAbort()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_abort(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartSelftestStart">SmartSelftestStart()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_selftest_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Controller.SmartUpdate">SmartUpdate()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_smart_update(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksNVMeController::handle-sanitize-start signal.
             * @param invocation
             * @param arg_action
             * @param arg_options
             */
            vfunc_handle_sanitize_start(
                invocation: Gio.DBusMethodInvocation,
                arg_action: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-get-attributes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_get_attributes(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-selftest-abort signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_selftest_abort(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-selftest-start signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_smart_selftest_start(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksNVMeController::handle-smart-update signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_smart_update(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const NVMeController: NVMeControllerNamespace & {
            new (): NVMeController; // This allows `obj instanceof NVMeController`
        };

        namespace NVMeFabrics {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                host_id: string;
                hostId: string;
                host_nqn: string;
                hostNqn: string;
                transport: string;
                transport_address: string;
                transportAddress: string;
            }
        }

        export interface NVMeFabricsNamespace {
            $gtype: GObject.GType<NVMeFabrics>;
            prototype: NVMeFabrics;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksNVMeFabrics interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface NVMeFabrics extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_id(): string;
            set host_id(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostID">"HostID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostId(): string;
            set hostId(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get host_nqn(): string;
            set host_nqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.HostNQN">"HostNQN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get hostNqn(): string;
            set hostNqn(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.Transport">"Transport"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transport(): string;
            set transport(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.TransportAddress">"TransportAddress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transport_address(): string;
            set transport_address(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Fabrics.TransportAddress">"TransportAddress"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get transportAddress(): string;
            set transportAddress(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_disconnect(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_disconnect(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_fabrics_call_disconnect_finish() to get the result of the operation.
             *
             * See udisks_nvme_fabrics_call_disconnect_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_disconnect(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_fabrics_call_disconnect().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_fabrics_call_disconnect().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_disconnect_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_fabrics_call_disconnect() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_disconnect_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Fabrics.Disconnect">Disconnect()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_disconnect(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksNVMeFabrics::handle-disconnect signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_disconnect(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const NVMeFabrics: NVMeFabricsNamespace & {
            new (): NVMeFabrics; // This allows `obj instanceof NVMeFabrics`
        };

        namespace NVMeNamespace {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                eui64: string;
                format_percent_remaining: number;
                formatPercentRemaining: number;
                formatted_lbasize: GLib.Variant;
                formattedLbasize: GLib.Variant;
                lbaformats: GLib.Variant;
                namespace_capacity: number;
                namespaceCapacity: number;
                namespace_size: number;
                namespaceSize: number;
                namespace_utilization: number;
                namespaceUtilization: number;
                nguid: string;
                nsid: number;
                uuid: string;
                wwn: string;
            }
        }

        export interface NVMeNamespaceNamespace {
            $gtype: GObject.GType<NVMeNamespace>;
            prototype: NVMeNamespace;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksNVMeNamespace interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface NVMeNamespace extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.EUI64">"EUI64"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get eui64(): string;
            set eui64(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormatPercentRemaining">"FormatPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get format_percent_remaining(): number;
            set format_percent_remaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormatPercentRemaining">"FormatPercentRemaining"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formatPercentRemaining(): number;
            set formatPercentRemaining(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormattedLBASize">"FormattedLBASize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formatted_lbasize(): GLib.Variant;
            set formatted_lbasize(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.FormattedLBASize">"FormattedLBASize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get formattedLbasize(): GLib.Variant;
            set formattedLbasize(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.LBAFormats">"LBAFormats"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get lbaformats(): GLib.Variant;
            set lbaformats(val: GLib.Variant);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceCapacity">"NamespaceCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_capacity(): number;
            set namespace_capacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceCapacity">"NamespaceCapacity"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceCapacity(): number;
            set namespaceCapacity(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceSize">"NamespaceSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_size(): number;
            set namespace_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceSize">"NamespaceSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceSize(): number;
            set namespaceSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceUtilization">"NamespaceUtilization"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespace_utilization(): number;
            set namespace_utilization(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NamespaceUtilization">"NamespaceUtilization"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get namespaceUtilization(): number;
            set namespaceUtilization(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NGUID">"NGUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nguid(): string;
            set nguid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.NSID">"NSID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get nsid(): number;
            set nsid(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-NVMe-Namespace.WWN">"WWN"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get wwn(): string;
            set wwn(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_format_namespace(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format_namespace(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_nvme_namespace_call_format_namespace_finish() to get the result of the operation.
             *
             * See udisks_nvme_namespace_call_format_namespace_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_format_namespace(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_nvme_namespace_call_format_namespace().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_nvme_namespace_call_format_namespace().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_namespace_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_nvme_namespace_call_format_namespace() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_format_namespace_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-NVMe-Namespace.FormatNamespace">FormatNamespace()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_format_namespace(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksNVMeNamespace::handle-format-namespace signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_format_namespace(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const NVMeNamespace: NVMeNamespaceNamespace & {
            new (): NVMeNamespace; // This allows `obj instanceof NVMeNamespace`
        };

        namespace Object {
            // Constructor properties interface

            interface ConstructorProps extends Gio.DBusObject.ConstructorProps {
                block: Block;
                block_lvm2: BlockLVM2;
                blockLvm2: BlockLVM2;
                drive: Drive;
                drive_ata: DriveAta;
                driveAta: DriveAta;
                encrypted: Encrypted;
                filesystem: Filesystem;
                filesystem_btrfs: FilesystemBTRFS;
                filesystemBtrfs: FilesystemBTRFS;
                job: Job;
                logical_volume: LogicalVolume;
                logicalVolume: LogicalVolume;
                loop: Loop;
                manager: Manager;
                manager_btrfs: ManagerBTRFS;
                managerBtrfs: ManagerBTRFS;
                manager_lvm2: ManagerLVM2;
                managerLvm2: ManagerLVM2;
                manager_nvme: ManagerNVMe;
                managerNvme: ManagerNVMe;
                mdraid: MDRaid;
                nvme_controller: NVMeController;
                nvmeController: NVMeController;
                nvme_fabrics: NVMeFabrics;
                nvmeFabrics: NVMeFabrics;
                nvme_namespace: NVMeNamespace;
                nvmeNamespace: NVMeNamespace;
                partition: Partition;
                partition_table: PartitionTable;
                partitionTable: PartitionTable;
                physical_volume: PhysicalVolume;
                physicalVolume: PhysicalVolume;
                swapspace: Swapspace;
                vdo_volume: VDOVolume;
                vdoVolume: VDOVolume;
                volume_group: VolumeGroup;
                volumeGroup: VolumeGroup;
            }
        }

        export interface ObjectNamespace {
            $gtype: GObject.GType<Object>;
            prototype: Object;
        }
        interface Object extends Gio.DBusObject {
            // Properties

            /**
             * The #UDisksBlock instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get block(): Block;
            set block(val: Block);
            /**
             * The #UDisksBlockLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get block_lvm2(): BlockLVM2;
            set block_lvm2(val: BlockLVM2);
            /**
             * The #UDisksBlockLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get blockLvm2(): BlockLVM2;
            set blockLvm2(val: BlockLVM2);
            /**
             * The #UDisksDrive instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get drive(): Drive;
            set drive(val: Drive);
            /**
             * The #UDisksDriveAta instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get drive_ata(): DriveAta;
            set drive_ata(val: DriveAta);
            /**
             * The #UDisksDriveAta instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get driveAta(): DriveAta;
            set driveAta(val: DriveAta);
            /**
             * The #UDisksEncrypted instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get encrypted(): Encrypted;
            set encrypted(val: Encrypted);
            /**
             * The #UDisksFilesystem instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystem(): Filesystem;
            set filesystem(val: Filesystem);
            /**
             * The #UDisksFilesystemBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystem_btrfs(): FilesystemBTRFS;
            set filesystem_btrfs(val: FilesystemBTRFS);
            /**
             * The #UDisksFilesystemBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get filesystemBtrfs(): FilesystemBTRFS;
            set filesystemBtrfs(val: FilesystemBTRFS);
            /**
             * The #UDisksJob instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get job(): Job;
            set job(val: Job);
            /**
             * The #UDisksLogicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get logical_volume(): LogicalVolume;
            set logical_volume(val: LogicalVolume);
            /**
             * The #UDisksLogicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get logicalVolume(): LogicalVolume;
            set logicalVolume(val: LogicalVolume);
            /**
             * The #UDisksLoop instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get loop(): Loop;
            set loop(val: Loop);
            /**
             * The #UDisksManager instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager(): Manager;
            set manager(val: Manager);
            /**
             * The #UDisksManagerBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_btrfs(): ManagerBTRFS;
            set manager_btrfs(val: ManagerBTRFS);
            /**
             * The #UDisksManagerBTRFS instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerBtrfs(): ManagerBTRFS;
            set managerBtrfs(val: ManagerBTRFS);
            /**
             * The #UDisksManagerLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_lvm2(): ManagerLVM2;
            set manager_lvm2(val: ManagerLVM2);
            /**
             * The #UDisksManagerLVM2 instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerLvm2(): ManagerLVM2;
            set managerLvm2(val: ManagerLVM2);
            /**
             * The #UDisksManagerNVMe instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get manager_nvme(): ManagerNVMe;
            set manager_nvme(val: ManagerNVMe);
            /**
             * The #UDisksManagerNVMe instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get managerNvme(): ManagerNVMe;
            set managerNvme(val: ManagerNVMe);
            /**
             * The #UDisksMDRaid instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get mdraid(): MDRaid;
            set mdraid(val: MDRaid);
            /**
             * The #UDisksNVMeController instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_controller(): NVMeController;
            set nvme_controller(val: NVMeController);
            /**
             * The #UDisksNVMeController instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeController(): NVMeController;
            set nvmeController(val: NVMeController);
            /**
             * The #UDisksNVMeFabrics instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_fabrics(): NVMeFabrics;
            set nvme_fabrics(val: NVMeFabrics);
            /**
             * The #UDisksNVMeFabrics instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeFabrics(): NVMeFabrics;
            set nvmeFabrics(val: NVMeFabrics);
            /**
             * The #UDisksNVMeNamespace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvme_namespace(): NVMeNamespace;
            set nvme_namespace(val: NVMeNamespace);
            /**
             * The #UDisksNVMeNamespace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get nvmeNamespace(): NVMeNamespace;
            set nvmeNamespace(val: NVMeNamespace);
            /**
             * The #UDisksPartition instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partition(): Partition;
            set partition(val: Partition);
            /**
             * The #UDisksPartitionTable instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partition_table(): PartitionTable;
            set partition_table(val: PartitionTable);
            /**
             * The #UDisksPartitionTable instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get partitionTable(): PartitionTable;
            set partitionTable(val: PartitionTable);
            /**
             * The #UDisksPhysicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get physical_volume(): PhysicalVolume;
            set physical_volume(val: PhysicalVolume);
            /**
             * The #UDisksPhysicalVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get physicalVolume(): PhysicalVolume;
            set physicalVolume(val: PhysicalVolume);
            /**
             * The #UDisksSwapspace instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get swapspace(): Swapspace;
            set swapspace(val: Swapspace);
            /**
             * The #UDisksVDOVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get vdo_volume(): VDOVolume;
            set vdo_volume(val: VDOVolume);
            /**
             * The #UDisksVDOVolume instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get vdoVolume(): VDOVolume;
            set vdoVolume(val: VDOVolume);
            /**
             * The #UDisksVolumeGroup instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get volume_group(): VolumeGroup;
            set volume_group(val: VolumeGroup);
            /**
             * The #UDisksVolumeGroup instance corresponding to the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link>, if any.
             *
             * Connect to the #GObject::notify signal to get informed of property changes.
             */
            get volumeGroup(): VolumeGroup;
            set volumeGroup(val: VolumeGroup);

            // Methods

            /**
             * Gets the #UDisksBlock instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block.top_of_page">org.freedesktop.UDisks2.Block</link> on `object,` if any.
             * @returns A #UDisksBlock that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_block(): Block | null;
            /**
             * Gets the #UDisksBlockLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Block-LVM2.top_of_page">org.freedesktop.UDisks2.Block.LVM2</link> on `object,` if any.
             * @returns A #UDisksBlockLVM2 that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_block_lvm2(): BlockLVM2 | null;
            /**
             * Gets the #UDisksDrive instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive.top_of_page">org.freedesktop.UDisks2.Drive</link> on `object,` if any.
             * @returns A #UDisksDrive that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_drive(): Drive | null;
            /**
             * Gets the #UDisksDriveAta instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Drive-Ata.top_of_page">org.freedesktop.UDisks2.Drive.Ata</link> on `object,` if any.
             * @returns A #UDisksDriveAta that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_drive_ata(): DriveAta | null;
            /**
             * Gets the #UDisksEncrypted instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Encrypted.top_of_page">org.freedesktop.UDisks2.Encrypted</link> on `object,` if any.
             * @returns A #UDisksEncrypted that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_encrypted(): Encrypted | null;
            /**
             * Gets the #UDisksFilesystem instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem.top_of_page">org.freedesktop.UDisks2.Filesystem</link> on `object,` if any.
             * @returns A #UDisksFilesystem that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_filesystem(): Filesystem | null;
            /**
             * Gets the #UDisksFilesystemBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Filesystem-BTRFS.top_of_page">org.freedesktop.UDisks2.Filesystem.BTRFS</link> on `object,` if any.
             * @returns A #UDisksFilesystemBTRFS that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_filesystem_btrfs(): FilesystemBTRFS | null;
            /**
             * Gets the #UDisksJob instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Job.top_of_page">org.freedesktop.UDisks2.Job</link> on `object,` if any.
             * @returns A #UDisksJob that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_job(): Job | null;
            /**
             * Gets the #UDisksLogicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-LogicalVolume.top_of_page">org.freedesktop.UDisks2.LogicalVolume</link> on `object,` if any.
             * @returns A #UDisksLogicalVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_logical_volume(): LogicalVolume | null;
            /**
             * Gets the #UDisksLoop instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Loop.top_of_page">org.freedesktop.UDisks2.Loop</link> on `object,` if any.
             * @returns A #UDisksLoop that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_loop(): Loop | null;
            /**
             * Gets the #UDisksManager instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager.top_of_page">org.freedesktop.UDisks2.Manager</link> on `object,` if any.
             * @returns A #UDisksManager that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager(): Manager | null;
            /**
             * Gets the #UDisksManagerBTRFS instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-BTRFS.top_of_page">org.freedesktop.UDisks2.Manager.BTRFS</link> on `object,` if any.
             * @returns A #UDisksManagerBTRFS that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_btrfs(): ManagerBTRFS | null;
            /**
             * Gets the #UDisksManagerLVM2 instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-LVM2.top_of_page">org.freedesktop.UDisks2.Manager.LVM2</link> on `object,` if any.
             * @returns A #UDisksManagerLVM2 that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_lvm2(): ManagerLVM2 | null;
            /**
             * Gets the #UDisksManagerNVMe instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Manager-NVMe.top_of_page">org.freedesktop.UDisks2.Manager.NVMe</link> on `object,` if any.
             * @returns A #UDisksManagerNVMe that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_manager_nvme(): ManagerNVMe | null;
            /**
             * Gets the #UDisksMDRaid instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-MDRaid.top_of_page">org.freedesktop.UDisks2.MDRaid</link> on `object,` if any.
             * @returns A #UDisksMDRaid that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_mdraid(): MDRaid | null;
            /**
             * Gets the #UDisksNVMeController instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Controller.top_of_page">org.freedesktop.UDisks2.NVMe.Controller</link> on `object,` if any.
             * @returns A #UDisksNVMeController that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_controller(): NVMeController | null;
            /**
             * Gets the #UDisksNVMeFabrics instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Fabrics.top_of_page">org.freedesktop.UDisks2.NVMe.Fabrics</link> on `object,` if any.
             * @returns A #UDisksNVMeFabrics that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_fabrics(): NVMeFabrics | null;
            /**
             * Gets the #UDisksNVMeNamespace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-NVMe-Namespace.top_of_page">org.freedesktop.UDisks2.NVMe.Namespace</link> on `object,` if any.
             * @returns A #UDisksNVMeNamespace that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_nvme_namespace(): NVMeNamespace | null;
            /**
             * Gets the #UDisksPartition instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link> on `object,` if any.
             * @returns A #UDisksPartition that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_partition(): Partition | null;
            /**
             * Gets the #UDisksPartitionTable instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link> on `object,` if any.
             * @returns A #UDisksPartitionTable that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_partition_table(): PartitionTable | null;
            /**
             * Gets the #UDisksPhysicalVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link> on `object,` if any.
             * @returns A #UDisksPhysicalVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_physical_volume(): PhysicalVolume | null;
            /**
             * Gets the #UDisksSwapspace instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link> on `object,` if any.
             * @returns A #UDisksSwapspace that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_swapspace(): Swapspace | null;
            /**
             * Gets the #UDisksVDOVolume instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link> on `object,` if any.
             * @returns A #UDisksVDOVolume that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_vdo_volume(): VDOVolume | null;
            /**
             * Gets the #UDisksVolumeGroup instance for the D-Bus interface <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link> on `object,` if any.
             * @returns A #UDisksVolumeGroup that must be freed with g_object_unref() or %NULL if @object does not implement the interface.
             */
            get_volume_group(): VolumeGroup | null;
        }

        export const Object: ObjectNamespace & {
            new (): Object; // This allows `obj instanceof Object`
        };

        namespace Partition {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                flags: number;
                is_contained: boolean;
                isContained: boolean;
                is_container: boolean;
                isContainer: boolean;
                name: string;
                number: number;
                offset: number;
                size: number;
                table: string;
                type: string;
                uuid: string;
            }
        }

        export interface PartitionNamespace {
            $gtype: GObject.GType<Partition>;
            prototype: Partition;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Partition.top_of_page">org.freedesktop.UDisks2.Partition</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksPartition interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Partition extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Flags">"Flags"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get flags(): number;
            set flags(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContained">"IsContained"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get is_contained(): boolean;
            set is_contained(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContained">"IsContained"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get isContained(): boolean;
            set isContained(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContainer">"IsContainer"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get is_container(): boolean;
            set is_container(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.IsContainer">"IsContainer"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get isContainer(): boolean;
            set isContainer(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Number">"Number"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get number(): number;
            set number(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Offset">"Offset"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get offset(): number;
            set offset(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Table">"Table"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get table(): string;
            set table(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Partition.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_delete_finish() to get the result of the operation.
             *
             * See udisks_partition_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_delete() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_resize_finish() to get the result of the operation.
             *
             * See udisks_partition_call_resize_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_resize().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_resize().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_resize() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_flags_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_flags_sync() for the synchronous, blocking version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_flags(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_flags().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_flags().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_flags_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_flags() for the asynchronous version of this method.
             * @param arg_flags Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_flags_sync(
                arg_flags: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_name_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_name_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_name(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_name().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_name().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_name_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_name() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_name_sync(
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_type_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_type_sync() for the synchronous, blocking version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_type(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_type().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_type().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_type_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_type() for the asynchronous version of this method.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_type_sync(
                arg_type: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_partition_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_partition_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.Resize">Resize()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetFlags">SetFlags()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_flags(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetName">SetName()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_name(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetType">SetType()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_type(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Partition.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksPartition::handle-delete signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_delete(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksPartition::handle-resize signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-flags signal.
             * @param invocation
             * @param arg_flags
             * @param arg_options
             */
            vfunc_handle_set_flags(
                invocation: Gio.DBusMethodInvocation,
                arg_flags: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-name signal.
             * @param invocation
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_set_name(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-type signal.
             * @param invocation
             * @param arg_type
             * @param arg_options
             */
            vfunc_handle_set_type(
                invocation: Gio.DBusMethodInvocation,
                arg_type: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartition::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const Partition: PartitionNamespace & {
            new (): Partition; // This allows `obj instanceof Partition`
        };

        namespace PartitionTable {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                partitions: string[];
                type: string;
            }
        }

        export interface PartitionTableNamespace {
            $gtype: GObject.GType<PartitionTable>;
            prototype: PartitionTable;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-PartitionTable.top_of_page">org.freedesktop.UDisks2.PartitionTable</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksPartitionTable interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface PartitionTable extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PartitionTable.Partitions">"Partitions"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get partitions(): string[];
            set partitions(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PartitionTable.Type">"Type"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get type(): string;
            set type(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_partition_table_call_create_partition_and_format_finish() to get the result of the operation.
             *
             * See udisks_partition_table_call_create_partition_and_format_sync() for the synchronous, blocking version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_partition_and_format(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_partition_table_call_create_partition_and_format().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_table_call_create_partition_and_format().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_and_format_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_table_call_create_partition_and_format() for the asynchronous version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param arg_format_type Argument to pass with the method invocation.
             * @param arg_format_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_and_format_sync(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Finishes an operation started with udisks_partition_table_call_create_partition().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_partition_table_call_create_partition().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_partition_table_call_create_partition() for the asynchronous version of this method.
             * @param arg_offset Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_type Argument to pass with the method invocation.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_partition_sync(
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartition">CreatePartition()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param created_partition Parameter to return.
             */
            complete_create_partition(invocation: Gio.DBusMethodInvocation, created_partition: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-PartitionTable.CreatePartitionAndFormat">CreatePartitionAndFormat()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param created_partition Parameter to return.
             */
            complete_create_partition_and_format(invocation: Gio.DBusMethodInvocation, created_partition: string): void;

            // Virtual methods

            /**
             * Handler for the #UDisksPartitionTable::handle-create-partition signal.
             * @param invocation
             * @param arg_offset
             * @param arg_size
             * @param arg_type
             * @param arg_name
             * @param arg_options
             */
            vfunc_handle_create_partition(
                invocation: Gio.DBusMethodInvocation,
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksPartitionTable::handle-create-partition-and-format signal.
             * @param invocation
             * @param arg_offset
             * @param arg_size
             * @param arg_type
             * @param arg_name
             * @param arg_options
             * @param arg_format_type
             * @param arg_format_options
             */
            vfunc_handle_create_partition_and_format(
                invocation: Gio.DBusMethodInvocation,
                arg_offset: number,
                arg_size: number,
                arg_type: string,
                arg_name: string,
                arg_options: GLib.Variant,
                arg_format_type: string,
                arg_format_options: GLib.Variant,
            ): boolean;
        }

        export const PartitionTable: PartitionTableNamespace & {
            new (): PartitionTable; // This allows `obj instanceof PartitionTable`
        };

        namespace PhysicalVolume {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                free_size: number;
                freeSize: number;
                size: number;
                volume_group: string;
                volumeGroup: string;
            }
        }

        export interface PhysicalVolumeNamespace {
            $gtype: GObject.GType<PhysicalVolume>;
            prototype: PhysicalVolume;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-PhysicalVolume.top_of_page">org.freedesktop.UDisks2.PhysicalVolume</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksPhysicalVolume interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface PhysicalVolume extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get free_size(): number;
            set free_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get freeSize(): number;
            set freeSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volume_group(): string;
            set volume_group(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-PhysicalVolume.VolumeGroup">"VolumeGroup"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get volumeGroup(): string;
            set volumeGroup(val: string);
        }

        export const PhysicalVolume: PhysicalVolumeNamespace & {
            new (): PhysicalVolume; // This allows `obj instanceof PhysicalVolume`
        };

        namespace Swapspace {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                active: boolean;
            }
        }

        export interface SwapspaceNamespace {
            $gtype: GObject.GType<Swapspace>;
            prototype: Swapspace;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-Swapspace.top_of_page">org.freedesktop.UDisks2.Swapspace</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksSwapspace interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface Swapspace extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-Swapspace.Active">"Active"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get active(): boolean;
            set active(val: boolean);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_label_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_label_sync() for the synchronous, blocking version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_label(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_set_label().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_set_label().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_set_label() for the asynchronous version of this method.
             * @param arg_label Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_label_sync(
                arg_label: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_set_uuid_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_set_uuid_sync() for the synchronous, blocking version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_set_uuid(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_set_uuid().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_set_uuid().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_set_uuid() for the asynchronous version of this method.
             * @param arg_uuid Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_set_uuid_sync(
                arg_uuid: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_start(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_start_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_start_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_start(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_start().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_start().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_start() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_start_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_stop(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_swapspace_call_stop_finish() to get the result of the operation.
             *
             * See udisks_swapspace_call_stop_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_stop(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_swapspace_call_stop().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_swapspace_call_stop().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_swapspace_call_stop() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_stop_sync(arg_options: GLib.Variant, cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetLabel">SetLabel()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_label(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.SetUUID">SetUUID()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_set_uuid(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Start">Start()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_start(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-Swapspace.Stop">Stop()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_stop(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksSwapspace::handle-set-label signal.
             * @param invocation
             * @param arg_label
             * @param arg_options
             */
            vfunc_handle_set_label(
                invocation: Gio.DBusMethodInvocation,
                arg_label: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-set-uuid signal.
             * @param invocation
             * @param arg_uuid
             * @param arg_options
             */
            vfunc_handle_set_uuid(
                invocation: Gio.DBusMethodInvocation,
                arg_uuid: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-start signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_start(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksSwapspace::handle-stop signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_stop(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
        }

        export const Swapspace: SwapspaceNamespace & {
            new (): Swapspace; // This allows `obj instanceof Swapspace`
        };

        namespace VDOVolume {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                compression: boolean;
                compression_state: string;
                compressionState: string;
                deduplication: boolean;
                index_state: string;
                indexState: string;
                operating_mode: string;
                operatingMode: string;
                used_size: number;
                usedSize: number;
                vdo_pool: string;
                vdoPool: string;
            }
        }

        export interface VDOVolumeNamespace {
            $gtype: GObject.GType<VDOVolume>;
            prototype: VDOVolume;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-VDOVolume.top_of_page">org.freedesktop.UDisks2.VDOVolume</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksVDOVolume interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface VDOVolume extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.Compression">"Compression"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compression(): boolean;
            set compression(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.CompressionState">"CompressionState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compression_state(): string;
            set compression_state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.CompressionState">"CompressionState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get compressionState(): string;
            set compressionState(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.Deduplication">"Deduplication"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get deduplication(): boolean;
            set deduplication(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.IndexState">"IndexState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get index_state(): string;
            set index_state(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.IndexState">"IndexState"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get indexState(): string;
            set indexState(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.OperatingMode">"OperatingMode"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operating_mode(): string;
            set operating_mode(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.OperatingMode">"OperatingMode"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get operatingMode(): string;
            set operatingMode(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.UsedSize">"UsedSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get used_size(): number;
            set used_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.UsedSize">"UsedSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get usedSize(): number;
            set usedSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.VDOPool">"VDOPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vdo_pool(): string;
            set vdo_pool(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VDOVolume.VDOPool">"VDOPool"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get vdoPool(): string;
            set vdoPool(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_compression_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_compression_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_compression(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_enable_compression().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_enable_compression().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_compression_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_enable_compression() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_compression_sync(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_enable_deduplication_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_enable_deduplication_sync() for the synchronous, blocking version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_enable_deduplication(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_enable_deduplication().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_enable_deduplication().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_deduplication_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_enable_deduplication() for the asynchronous version of this method.
             * @param arg_enable Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_enable_deduplication_sync(
                arg_enable: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<GLib.Variant | null>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_get_statistics_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_get_statistics_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_get_statistics(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<GLib.Variant | null> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_get_statistics().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_get_statistics().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_statistics_finish(res: Gio.AsyncResult): [boolean, GLib.Variant | null];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_get_statistics() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_get_statistics_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, GLib.Variant | null];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_logical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_logical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_logical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_resize_logical().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_resize_logical().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_logical_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_resize_logical() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_logical_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_vdo_volume_call_resize_physical_finish() to get the result of the operation.
             *
             * See udisks_vdo_volume_call_resize_physical_sync() for the synchronous, blocking version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_resize_physical(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_vdo_volume_call_resize_physical().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_vdo_volume_call_resize_physical().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_physical_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_vdo_volume_call_resize_physical() for the asynchronous version of this method.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_resize_physical_sync(
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableCompression">EnableCompression()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_compression(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.EnableDeduplication">EnableDeduplication()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_enable_deduplication(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.GetStatistics">GetStatistics()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param stats Parameter to return.
             */
            complete_get_statistics(invocation: Gio.DBusMethodInvocation, stats: GLib.Variant): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizeLogical">ResizeLogical()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize_logical(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VDOVolume.ResizePhysical">ResizePhysical()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_resize_physical(invocation: Gio.DBusMethodInvocation): void;

            // Virtual methods

            /**
             * Handler for the #UDisksVDOVolume::handle-enable-compression signal.
             * @param invocation
             * @param arg_enable
             * @param arg_options
             */
            vfunc_handle_enable_compression(
                invocation: Gio.DBusMethodInvocation,
                arg_enable: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-enable-deduplication signal.
             * @param invocation
             * @param arg_enable
             * @param arg_options
             */
            vfunc_handle_enable_deduplication(
                invocation: Gio.DBusMethodInvocation,
                arg_enable: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-get-statistics signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_get_statistics(invocation: Gio.DBusMethodInvocation, arg_options: GLib.Variant): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-resize-logical signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize_logical(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVDOVolume::handle-resize-physical signal.
             * @param invocation
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_resize_physical(
                invocation: Gio.DBusMethodInvocation,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const VDOVolume: VDOVolumeNamespace & {
            new (): VDOVolume; // This allows `obj instanceof VDOVolume`
        };

        namespace VolumeGroup {
            // Constructor properties interface

            interface ConstructorProps extends GObject.Object.ConstructorProps {
                extent_size: number;
                extentSize: number;
                free_size: number;
                freeSize: number;
                missing_physical_volumes: string[];
                missingPhysicalVolumes: string[];
                name: string;
                needs_polling: boolean;
                needsPolling: boolean;
                size: number;
                uuid: string;
            }
        }

        export interface VolumeGroupNamespace {
            $gtype: GObject.GType<VolumeGroup>;
            prototype: VolumeGroup;

            /**
             * Gets a machine-readable description of the <link linkend="gdbus-interface-org-freedesktop-UDisks2-VolumeGroup.top_of_page">org.freedesktop.UDisks2.VolumeGroup</link> D-Bus interface.
             */
            interface_info(): Gio.DBusInterfaceInfo;
            /**
             * Overrides all #GObject properties in the #UDisksVolumeGroup interface for a concrete class.
             * The properties are overridden in the order they are defined.
             * @param klass The class structure for a #GObject derived class.
             * @param property_id_begin The property id to assign to the first overridden property.
             */
            override_properties(klass: typeof GObject.Object, property_id_begin: number): number;
        }
        interface VolumeGroup extends GObject.Object {
            // Properties

            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.ExtentSize">"ExtentSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get extent_size(): number;
            set extent_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.ExtentSize">"ExtentSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get extentSize(): number;
            set extentSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get free_size(): number;
            set free_size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.FreeSize">"FreeSize"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get freeSize(): number;
            set freeSize(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.MissingPhysicalVolumes">"MissingPhysicalVolumes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get missing_physical_volumes(): string[];
            set missing_physical_volumes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.MissingPhysicalVolumes">"MissingPhysicalVolumes"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get missingPhysicalVolumes(): string[];
            set missingPhysicalVolumes(val: string[]);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.Name">"Name"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get name(): string;
            set name(val: string);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.NeedsPolling">"NeedsPolling"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get needs_polling(): boolean;
            set needs_polling(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.NeedsPolling">"NeedsPolling"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get needsPolling(): boolean;
            set needsPolling(val: boolean);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.Size">"Size"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get size(): number;
            set size(val: number);
            /**
             * Represents the D-Bus property <link linkend="gdbus-property-org-freedesktop-UDisks2-VolumeGroup.UUID">"UUID"</link>.
             *
             * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
             */
            get uuid(): string;
            set uuid(val: string);

            // Methods

            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_add_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_add_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_add_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_add_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_add_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_add_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_add_device_sync(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_plain_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_plain_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_plain_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_plain_volume_with_layout_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_plain_volume_with_layout(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_plain_volume_with_layout().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_plain_volume_with_layout().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_with_layout_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_plain_volume_with_layout() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_layout Argument to pass with the method invocation.
             * @param arg_pvs Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_plain_volume_with_layout_sync(
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_pool_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_pool_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_pool_volume(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_thin_pool_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_thin_pool_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_pool_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_thin_pool_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_pool_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_thin_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_thin_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_thin_volume(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_thin_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_thin_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_thin_volume() for the asynchronous version of this method.
             * @param arg_name Argument to pass with the method invocation.
             * @param arg_size Argument to pass with the method invocation.
             * @param arg_pool Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_thin_volume_sync(
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_create_vdo_volume_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_create_vdo_volume_sync() for the synchronous, blocking version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_create_vdo_volume(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_create_vdo_volume().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_create_vdo_volume().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_vdo_volume_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_create_vdo_volume() for the asynchronous version of this method.
             * @param arg_lv_name Argument to pass with the method invocation.
             * @param arg_pool_name Argument to pass with the method invocation.
             * @param arg_data_size Argument to pass with the method invocation.
             * @param arg_virtual_size Argument to pass with the method invocation.
             * @param arg_index_memory Argument to pass with the method invocation.
             * @param arg_compression Argument to pass with the method invocation.
             * @param arg_deduplication Argument to pass with the method invocation.
             * @param arg_write_policy Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_create_vdo_volume_sync(
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_delete_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_delete_sync() for the synchronous, blocking version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_delete(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_delete().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_delete().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_delete() for the asynchronous version of this method.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_delete_sync(
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_empty_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_empty_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_empty_device(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_empty_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_empty_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_empty_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_empty_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_empty_device_sync(
                arg_block: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_poll(cancellable?: Gio.Cancellable | null): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_poll(cancellable: Gio.Cancellable | null, callback: Gio.AsyncReadyCallback<this> | null): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_poll_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_poll_sync() for the synchronous, blocking version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_poll(
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_poll().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_poll().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_poll_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_poll() for the asynchronous version of this method.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_poll_sync(cancellable?: Gio.Cancellable | null): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_device_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_device_sync() for the synchronous, blocking version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_device(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_remove_device().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_remove_device().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_remove_device() for the asynchronous version of this method.
             * @param arg_block Argument to pass with the method invocation.
             * @param arg_wipe Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_device_sync(
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<boolean>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_remove_missing_physical_volumes_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes_sync() for the synchronous, blocking version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_remove_missing_physical_volumes(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<boolean> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_remove_missing_physical_volumes().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_remove_missing_physical_volumes().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_missing_physical_volumes_finish(res: Gio.AsyncResult): boolean;
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_remove_missing_physical_volumes() for the asynchronous version of this method.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_remove_missing_physical_volumes_sync(
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): boolean;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): Promise<string>;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable: Gio.Cancellable | null,
                callback: Gio.AsyncReadyCallback<this> | null,
            ): void;
            /**
             * Asynchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`.
             * When the operation is finished, `callback` will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
             * You can then call udisks_volume_group_call_rename_finish() to get the result of the operation.
             *
             * See udisks_volume_group_call_rename_sync() for the synchronous, blocking version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @param callback A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
             */
            call_rename(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
                callback?: Gio.AsyncReadyCallback<this> | null,
            ): Promise<string> | void;
            /**
             * Finishes an operation started with udisks_volume_group_call_rename().
             * @param res The #GAsyncResult obtained from the #GAsyncReadyCallback passed to udisks_volume_group_call_rename().
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_finish(res: Gio.AsyncResult): [boolean, string];
            /**
             * Synchronously invokes the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method on `proxy`. The calling thread is blocked until a reply is received.
             *
             * See udisks_volume_group_call_rename() for the asynchronous version of this method.
             * @param arg_new_name Argument to pass with the method invocation.
             * @param arg_options Argument to pass with the method invocation.
             * @param cancellable A #GCancellable or %NULL.
             * @returns %TRUE if the call succeeded, %FALSE if @error is set.
             */
            call_rename_sync(
                arg_new_name: string,
                arg_options: GLib.Variant,
                cancellable?: Gio.Cancellable | null,
            ): [boolean, string];
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.AddDevice">AddDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_add_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolume">CreatePlainVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_plain_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreatePlainVolumeWithLayout">CreatePlainVolumeWithLayout()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_plain_volume_with_layout(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinPoolVolume">CreateThinPoolVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_thin_pool_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateThinVolume">CreateThinVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_thin_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.CreateVDOVolume">CreateVDOVolume()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_create_vdo_volume(invocation: Gio.DBusMethodInvocation, result: string): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Delete">Delete()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_delete(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.EmptyDevice">EmptyDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_empty_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Poll">Poll()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_poll(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveDevice">RemoveDevice()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_device(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.RemoveMissingPhysicalVolumes">RemoveMissingPhysicalVolumes()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             */
            complete_remove_missing_physical_volumes(invocation: Gio.DBusMethodInvocation): void;
            /**
             * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-org-freedesktop-UDisks2-VolumeGroup.Rename">Rename()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
             *
             * This method will free `invocation,` you cannot use it afterwards.
             * @param invocation A #GDBusMethodInvocation.
             * @param result Parameter to return.
             */
            complete_rename(invocation: Gio.DBusMethodInvocation, result: string): void;

            // Virtual methods

            /**
             * Handler for the #UDisksVolumeGroup::handle-add-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_options
             */
            vfunc_handle_add_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-plain-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_plain_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-plain-volume-with-layout signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_layout
             * @param arg_pvs
             * @param arg_options
             */
            vfunc_handle_create_plain_volume_with_layout(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_layout: string,
                arg_pvs: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-thin-pool-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_options
             */
            vfunc_handle_create_thin_pool_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-thin-volume signal.
             * @param invocation
             * @param arg_name
             * @param arg_size
             * @param arg_pool
             * @param arg_options
             */
            vfunc_handle_create_thin_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_name: string,
                arg_size: number,
                arg_pool: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-create-vdo-volume signal.
             * @param invocation
             * @param arg_lv_name
             * @param arg_pool_name
             * @param arg_data_size
             * @param arg_virtual_size
             * @param arg_index_memory
             * @param arg_compression
             * @param arg_deduplication
             * @param arg_write_policy
             * @param arg_options
             */
            vfunc_handle_create_vdo_volume(
                invocation: Gio.DBusMethodInvocation,
                arg_lv_name: string,
                arg_pool_name: string,
                arg_data_size: number,
                arg_virtual_size: number,
                arg_index_memory: number,
                arg_compression: boolean,
                arg_deduplication: boolean,
                arg_write_policy: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-delete signal.
             * @param invocation
             * @param arg_wipe
             * @param arg_options
             */
            vfunc_handle_delete(
                invocation: Gio.DBusMethodInvocation,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-empty-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_options
             */
            vfunc_handle_empty_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-poll signal.
             * @param invocation
             */
            vfunc_handle_poll(invocation: Gio.DBusMethodInvocation): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-remove-device signal.
             * @param invocation
             * @param arg_block
             * @param arg_wipe
             * @param arg_options
             */
            vfunc_handle_remove_device(
                invocation: Gio.DBusMethodInvocation,
                arg_block: string,
                arg_wipe: boolean,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-remove-missing-physical-volumes signal.
             * @param invocation
             * @param arg_options
             */
            vfunc_handle_remove_missing_physical_volumes(
                invocation: Gio.DBusMethodInvocation,
                arg_options: GLib.Variant,
            ): boolean;
            /**
             * Handler for the #UDisksVolumeGroup::handle-rename signal.
             * @param invocation
             * @param arg_new_name
             * @param arg_options
             */
            vfunc_handle_rename(
                invocation: Gio.DBusMethodInvocation,
                arg_new_name: string,
                arg_options: GLib.Variant,
            ): boolean;
        }

        export const VolumeGroup: VolumeGroupNamespace & {
            new (): VolumeGroup; // This allows `obj instanceof VolumeGroup`
        };

        /**
         * Name of the imported GIR library
         * `see` https://gitlab.gnome.org/GNOME/gjs/-/blob/master/gi/ns.cpp#L188
         */
        const __name__: string;
        /**
         * Version of the imported GIR library
         * `see` https://gitlab.gnome.org/GNOME/gjs/-/blob/master/gi/ns.cpp#L189
         */
        const __version__: string;
    }

    export default UDisks;
}

declare module 'gi://UDisks' {
    import UDisks20 from 'gi://UDisks?version=2.0';
    export default UDisks20;
}
// END
